package com.lantone.common.util;

import java.lang.reflect.Field;

/**
 * @Description:
 * @author: rengb
 * @time: 2020/4/20 16:18
 */
public class ObjectUtil {

    /**
     * 将对象object中私有属性为字符串"NULL"或者"null"的字段值改为null
     *
     * @param object
     * @param <T>
     * @return
     */
    public static <T> T attributeNullHandle(Object object) {
        if (object == null) {
            return null;
        }
        try {
            Field[] fields = object.getClass().getDeclaredFields();
            String fieldObjectValue = null;
            for (Field field : fields) {
                field.setAccessible(true);
                Object fieldObject = field.get(object);
                if (fieldObject != null && fieldObject.getClass().getName().equals("java.lang.String")) {
                    fieldObjectValue = fieldObject.toString();
                    if (fieldObjectValue.equals("NULL") || fieldObjectValue.equals("null")) {
                        field.set(object, null);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return (T) object;
    }

    /**
     * 单个对象的某个键的值
     *
     * @param obj 对象
     * @param key 键
     * @return Object 键在对象中所对应得值 没有查到时返回空字符串
     */
    public static Object getValueByKey(Object obj, String key) {
        // 得到类对象
        Class userCla = (Class) obj.getClass();
        // 得到类中的所有属性集合
        Field[] fs = userCla.getDeclaredFields();
        for (int i = 0; i < fs.length; i++) {
            Field f = fs[i];
            f.setAccessible(true); // 设置些属性是可以访问的
            try {
                if (f.getName().endsWith(key)) {
                    return f.get(obj);
                }
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        // 没有查到时返回空字符串
        return null;
    }

}