package com.lantone.dblayermbg.facade.report;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.lantone.common.dto.report.QuestionDTO;
import com.lantone.common.dto.report.QuestionEntryDTO;
import com.lantone.common.dto.report.wrapper.QuestionInfoWrapper;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.EntityUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.vo.report.QuestionIdsVO;
import com.lantone.dblayermbg.entity.report.QuestionInfo;
import com.lantone.dblayermbg.service.impl.report.QuestionInfoServiceImpl;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 表名：qc_question_info 业务类
 * </p>
 */
@Component
public class QuestionInfoFacade extends QuestionInfoServiceImpl {

    public Map<Long, List<QuestionEntryDTO>> getByCaseEntryIdsFac(Map map) {
        List<QuestionEntryDTO> questionEntryDTOList = this.getBaseMapper().getByCaseEntryIds(map);
        return  EntityUtil.makeEntityListMap(questionEntryDTOList, "casesEntryId");
    }

    /**
     * 查询多个id返回标签内容【优化版】
     *
     * @param questionIdsVO 获取标签内容参数
     * @return 标签内容Fac
     */
    public Map<Long, QuestionDTO> getByIds(QuestionIdsVO questionIdsVO) {
        List<Long> idList = questionIdsVO.getIds();
        if (ListUtil.isEmpty(idList)) {
            return new HashMap<>();
        }
        List<QuestionInfo> qcQuestionInfoList = this.list(new QueryWrapper<QuestionInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("id", idList)
        );
        List<QuestionDTO> questionDTOList = BeanUtil.listCopyTo(qcQuestionInfoList, QuestionDTO.class);
        if (ListUtil.isNotEmpty(questionDTOList)) {
            List<Long> ids = questionDTOList.stream().map(r -> r.getId()).collect(Collectors.toList());
            Map paramMap = new HashMap<>();
            paramMap.put("ids", ids);
            //获取子标签
            List<QuestionInfoWrapper> questionMapping = this.getBaseMapper().getByQuestionMapping(paramMap);
            Map<Long, List<QuestionInfoWrapper>> map = EntityUtil.makeEntityListMap(questionMapping, "parentQuestion");
            for (QuestionDTO bean : questionDTOList) {
                List<QuestionInfoWrapper> questionInfoWrapperList = map.get(bean.getId());
                if (ListUtil.isNotEmpty(questionInfoWrapperList)) {
                    bean.setQuestionMapping(BeanUtil.listCopyTo(questionInfoWrapperList, QuestionDTO.class));
                }
            }
        }
        return questionDTOList.stream().collect(Collectors.toMap(r -> r.getId(), r -> r));
    }
}