package com.lantone.dblayermbg.mapper.report;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.dto.report.BehospitalInfoAgeDTO;
import com.lantone.common.dto.report.BehospitalInfoDTO;
import com.lantone.common.dto.report.CaseScoreDTO;
import com.lantone.common.dto.report.EntryStatisticsDTO;
import com.lantone.common.dto.report.ExportExcelDTO;
import com.lantone.common.dto.report.HomePageImproveDTO;
import com.lantone.common.dto.report.HomePageNumDTO;
import com.lantone.common.dto.report.LevelStatisticsDTO;
import com.lantone.common.dto.report.LevelStatisticsTZDTO;
import com.lantone.common.dto.report.MedManageParamsDTO;
import com.lantone.common.dto.report.MedicalCheckDTO;
import com.lantone.common.dto.report.MedicalCheckExportDTO;
import com.lantone.common.dto.report.MedicalCheckIdNameDTO;
import com.lantone.common.dto.report.MsgDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.dto.report.QcResultShortDTO;
import com.lantone.common.dto.report.QualityControlDTO;
import com.lantone.common.dto.report.ReBeHosMergeDTO;
import com.lantone.common.dto.report.UnModifyMRDetailDTO;
import com.lantone.common.vo.report.AnalyzeVO;
import com.lantone.common.vo.report.BehospitalPageVO;
import com.lantone.common.vo.report.CaseScoreVO;
import com.lantone.common.vo.report.EntryStatisticsVO;
import com.lantone.common.vo.report.ExportQcresultVO;
import com.lantone.common.vo.report.FilterMedicalCheckVO;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.common.vo.report.FilterUnModifyMRVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.MedIndexFilterVO;
import com.lantone.common.vo.report.QcResultPageVO;
import com.lantone.common.vo.report.QcResultShortPageVO;
import com.lantone.common.vo.report.ReBeHosPageVO;
import com.lantone.dblayermbg.entity.report.BehospitalInfo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 表名：med_behospital_info Mapper接口
 * </p>
 */
public interface BehospitalInfoMapper extends BaseMapper<BehospitalInfo> {
    /**
     * 出院总人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int leaveHosCount(FilterVO filterVO);

    /**
     * 新生儿出院人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int newBornCount(FilterVO filterVO);

    /**
     * 死亡人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int deathCount(FilterVO filterVO);

    /**
     * 手术人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int operationCount(FilterVO filterVO);

    /**
     * 非医嘱离院病人记录
     *
     * @param filterVO
     * @return
     */
    public int nonAdviceCount(FilterVO filterVO);

    /**
     * 31天再入院详情页
     *
     * @param reBeHosPageVO
     * @return
     */
    public IPage<ReBeHosMergeDTO> reHos31DaysPage(@Param("reBeHosPageVO") ReBeHosPageVO reBeHosPageVO);

    /**
     * 离院病人报表统计
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> leaveHosMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 离院病人质控评分详情页导出到excel
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> leaveHosMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计（台州）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsTZDTO> levelStatisticsByDeptClass_TZ(FilterOrderVO filterOrderVO);

    /**
     * 病案首页质控病历数统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageNumDTO homePageQcPercent(FilterVO filterVO);

    /**
     * 病案首页完整率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageNumDTO hmEmptyEntryPercent(FilterVO filterVO);

    /**
     * 改善率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageImproveDTO homePageImproveCount(FilterVO filterVO);

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    public List<HomePageNumDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 入院人数统计-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> beHosCount(FilterVO filterVO);

    /**
     * 单条条目缺陷统计-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> casesEntryStatisticsById(FilterVO filterVO);

    /**
     * 关键条目缺陷占比统计
     *
     * @param entryStatisticsVO
     * @return
     */
    public List<EntryStatisticsDTO> entryStatistics(EntryStatisticsVO entryStatisticsVO);

    /**
     * 质控核查统计（内页）
     *
     * @param filterOrderVO
     * @return
     */
    public List<HomePageImproveDTO> qcCheckStatistics(FilterOrderVO filterOrderVO);

    /**
     * 时效性相关统计
     *
     * @param filterUnModifyMRVO
     * @return
     */
    public List<UnModifyMRDetailDTO> unModifyMRStatistics(FilterUnModifyMRVO filterUnModifyMRVO);

    /**
     * 病历稽查表-1
     *
     * @param filterVO
     * @return
     */
    public IPage<MedicalCheckDTO> getMedicalCheck(@Param("filterVO") FilterMedicalCheckVO filterVO);

    /**
     * 病历稽查表-2
     *
     * @param casesEntryIds
     * @return
     */

    public List<MedicalCheckIdNameDTO> getMedicalEntryIds(@Param("casesEntryIds") List<Long> casesEntryIds);

    /**
     * 病历稽查表导出
     *
     * @param filterVO
     * @return
     */
    public List<MedicalCheckExportDTO> getMedicalCheckExport(@Param("filterVO") FilterMedicalCheckVO filterVO);

    /**
     * 病历质控报表
     *
     * @param behospitalPageVO
     * @return
     */
    public IPage<QualityControlDTO> getQualityControl(BehospitalPageVO behospitalPageVO);

    /**
     * 病历质控报表-扣分数据
     *
     * @param caseScoreVO
     * @return
     */
    public List<CaseScoreDTO> getQualityCaseSorce(CaseScoreVO caseScoreVO);

    /**
     * 病历分页列表
     * @param behospitalPageVO
     * @return
     */
    public IPage<BehospitalInfoDTO> getPage(BehospitalPageVO behospitalPageVO);

    /**
     * 病历列表-报表导出
     *
     * @param exportQcresultVO
     * @return
     */
    public List<ExportExcelDTO> exportQcresult(ExportQcresultVO exportQcresultVO);

    /**
     * 条目缺陷质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> qcResultShortPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 条目缺陷病历列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> qcResultShortPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 病案首页合格|不合格病历列表-报表
     *
     * @param qcResultPageVO
     * @return
     */
    public IPage<QcResultShortDTO> badLevelPage(@Param("qcResultPageVO") QcResultPageVO qcResultPageVO);

    /**
     *  病案首页合格|不合格病历列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */

    public List<ExportExcelDTO> badLevelPagePageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 质控核查统计病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> qcCheckMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 质控核查质控评分页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> qcCheckMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 病案首页改善率质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> hmImproveMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 时效性|稽查条目病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> unModifyMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 未整改病历缺陷评分详情页导出
     *
     * @param qcResultShortPageVO
     * @return
     */

    public List<ExportExcelDTO> unModifyMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    public List<MsgDTO> getMsg(AnalyzeVO analyzeVO);

    /**
     * 病理CT/MR触发多规则人数
     * @param filterVO
     * @return
     */
    public Map<String,Long> triggeringRules(@Param("filterVO") FilterVO filterVO);

    /**
     * 病案管理指标获取基础数据
     *
     * @param timeVo
     * @return
     */
    public Map<String,Object> getBaseIndex( MedIndexFilterVO timeVo);

    /**
     * 手术费，抗菌药物，手术和病理费等含有人数
     *
     * @param filterVO
     * @return
     */
    public List<Map<String, String>> selectOperationNum(@Param("filterVO") FilterVO filterVO);

    /**
     * 病案管理规则质控病历参数查询
     *
     * @param filterVO
     * @return
     */

    public List<MedManageParamsDTO> getMedManageParams(FilterVO filterVO);

    /**
     * 病案首页关联的出院病历
     * @param filterVO
     * @return
     */
    public List<Map<String,String>> getMedicalRecords(@Param("filterVO") FilterVO filterVO);

    /**
     * 首页恶性肿瘤人数
     * @param filterVO
     * @return
     */
    public List<Map<String,String>> malignancy(@Param("filterVO") FilterVO filterVO);

    /**
     * 获取患者年龄信息(通过出生日期和入院日期重新计算)
     * @param hospitalId
     * @param behospitalCodes
     * @return
     */
    List<BehospitalInfoAgeDTO> getBehospitalInfoAge(Long hospitalId, List<String> behospitalCodes);


}