package com.lantone.dblayermbg.mapper.report;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.dto.report.AverageStatisticsDTO;
import com.lantone.common.dto.report.DeptNumDTO;
import com.lantone.common.dto.report.EntryNumDTO;
import com.lantone.common.dto.report.EntryNumGroupDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.vo.report.FilterPageVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.QcresultFilterVO;
import com.lantone.dblayermbg.entity.report.QcresultInfo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 表名：med_qcresult_info Mapper接口
 * </p>
 */
public interface QcresultInfoMapper extends BaseMapper<QcresultInfo> {
    /**
     * 按模块统计缺陷数
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> entryCountGroupByCase(FilterVO filterVO);

    /**
     * 各模块缺陷占比报表
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> entryCountGroupByCasePage(@Param("filterPageVO") FilterPageVO filterPageVO);


    /**
     * 条目缺陷分组统计
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> entryCountGroupByEntry(FilterVO filterVO);

    /**
     * 条目缺陷占比基础数据（内页）
     *
     * @return
     */
    public IPage<EntryNumDTO> entryGroupByEntryInnerPage(@Param("filterPageVO") FilterPageVO filterPageVO);

    /**
     * 条目缺陷总数（内页）
     *
     * @return
     */
    public Integer entryGroupByEntrySum(@Param("hospitalId") String hospitalId, @Param("isPlacefile") String isPlacefile, @Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 质控病历数统计
     *
     * @param qcresultFilterVO
     * @return
     */
    public Map resultCount(QcresultFilterVO qcresultFilterVO);

    /**
     * 各科室甲/乙/丙级病历占比
     *
     * @param filterVO
     * @return
     */
    public List<DeptNumDTO> qcResultLevelPercent(FilterVO filterVO);

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScore(FilterVO filterVO);

    /**
     * 各科室质控平均分（首页）-根据内外科系统统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScoreByDeptClass(FilterVO filterVO);

    /**
     * 单项否决缺陷占比
     *
     * @param filterVO
     * @return
     */
    public List<EntryNumDTO> entryRejectPercent(FilterVO filterVO);

    /**
     * 缺陷详情（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<EntryNumGroupDTO> entryCountGroupByEntryPage(@Param("filterPageVO") FilterPageVO filterPageVO);


}