package com.lantone.message.component;

import com.lantone.common.dto.AbnormalLogDTO;
import com.lantone.common.dto.LoginLogDTO;
import com.lantone.common.dto.OperationLogDTO;
import com.lantone.common.util.AddressUtils;
import com.lantone.dblayermbg.facade.AbnormalLogFacade;
import com.lantone.dblayermbg.facade.DataAuthFacade;
import com.lantone.dblayermbg.facade.DictionaryInfoFacade;
import com.lantone.dblayermbg.facade.HospitalSetFacade;
import com.lantone.dblayermbg.facade.LoginLogFacade;
import com.lantone.dblayermbg.facade.OperationLogFacade;
import com.lantone.dblayermbg.facade.ResourceFacade;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @Description: 消息的处理者
 * @author: rengb
 * @time: 2021/1/5 18:27
 */
@Slf4j
@Component
public class MessageReceiver {

    @Autowired
    private DictionaryInfoFacade dictionaryInfoFacade;
    @Autowired
    private HospitalSetFacade hospitalSetFacade;
    @Autowired
    private ResourceFacade resourceFacade;
    @Autowired
    private LoginLogFacade loginLogFacade;
    @Autowired
    private OperationLogFacade operationLogFacade;
    @Autowired
    private AbnormalLogFacade abnormalLogFacade;
    @Autowired
    private DataAuthFacade dataAuthFacade;


    @RabbitListener(queues = "emrais.redis.cache.refresh.queue", ackMode = "MANUAL")
    public void RedisCacheRefreshHandle(String type, Message message, Channel channel) throws Exception {
        if (type.equals(DictionaryInfoFacade.class.getSimpleName())) {
            dictionaryInfoFacade.initDicMap();
        } else if (type.equals(HospitalSetFacade.class.getSimpleName())) {
            hospitalSetFacade.initHospitalSetMap();
        } else if (type.equals(ResourceFacade.class.getSimpleName())) {
            resourceFacade.initResourceRolesMap();
        } else if (type.equals(DataAuthFacade.class.getSimpleName())) {
            dataAuthFacade.initUserDataAuth();
        }
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

    @RabbitListener(queues = "emrais.login.success.record.queue", ackMode = "MANUAL")
    public void sendLogSuccessMessageHandle(LoginLogDTO loginLog, Message message, Channel channel) throws Exception {
        loginLog.setLoginAddress(AddressUtils.getRealAddressByIP(loginLog.getLoginIp()));
        loginLogFacade.getBaseMapper().addLoginLog(loginLog);
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

    @RabbitListener(queues = "emrais.oper.log.record.queue", ackMode = "MANUAL")
    public void sendOperLogRecordMessage(OperationLogDTO operationLog, Message message, Channel channel) throws Exception {
        operationLog.setOperationAddress(AddressUtils.getRealAddressByIP(operationLog.getOperationIp()));
        operationLogFacade.getBaseMapper().addOperLog(operationLog);
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

    @RabbitListener(queues = "emrais.oper.error.record.queue", ackMode = "MANUAL")
    public void sendOperErrorRecordMessage(AbnormalLogDTO abnormalLogDTO, Message message, Channel channel) throws Exception {
        abnormalLogFacade.getBaseMapper().addOperErrorLog(abnormalLogDTO);
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

}