package com.lantone.analyze.facade.analyze;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.lantone.common.dto.analyze.SysUserPagesetDTO;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.DateUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.analyze.SysUserPagesetQueryVO;
import com.lantone.common.vo.analyze.SysUserPagesetSaveVO;
import com.lantone.common.vo.analyze.SysUserPagesetVO;
import com.lantone.dblayermbg.entity.analyze.UserPageset;
import com.lantone.dblayermbg.service.impl.analyze.UserPagesetServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description:
 * @Author songxl
 * @Date 2021/9/28
 */
@Component
public class UserPagesetManagementFacade {
    @Autowired
    @Qualifier("userPagesetServiceImpl")
    private UserPagesetServiceImpl userPagesetService;

    /**
     * 获取用户对应页面设置
     *
     * @param sysUserPagesetQueryVO 获取用户对应页面设置参数
     * @return 对应页面设置列表
     */
    public List<SysUserPagesetDTO> getPageSet(SysUserPagesetQueryVO sysUserPagesetQueryVO) {
        List<UserPageset> sysUserPagesetList = userPagesetService.list(
                new QueryWrapper<UserPageset>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("user_id", SysUserUtils.getCurrentPrincipleId())
                        .eq("page_type", sysUserPagesetQueryVO.getPageType())
                        .orderByAsc("order_no")
        );

        List<SysUserPagesetDTO> sysUserPagesetDTOList = new ArrayList<>();
        if (ListUtil.isEmpty(sysUserPagesetList)) {
            List<UserPageset> sysUserPagesetListDefault = userPagesetService.list(
                    new QueryWrapper<UserPageset>()
                            .eq("is_deleted", IsDeleteEnum.N.getKey())
                            .eq("user_id", -1)
                            .eq("page_type", sysUserPagesetQueryVO.getPageType())
                            .orderByAsc("order_no")
            );
            sysUserPagesetDTOList = BeanUtil.listCopyTo(sysUserPagesetListDefault, SysUserPagesetDTO.class);
        } else {
            sysUserPagesetDTOList = BeanUtil.listCopyTo(sysUserPagesetList, SysUserPagesetDTO.class);
        }
        return sysUserPagesetDTOList;
    }

    /**
     * 保存用户对应页面设置
     *
     * @param sysUserPagesetSaveVO 保存用户对应页面设置参数
     * @return 是否成功
     */
    public Boolean savePageSet(SysUserPagesetSaveVO sysUserPagesetSaveVO) {
        if (ListUtil.isEmpty(sysUserPagesetSaveVO.getSysUserPagesetVOList())) {
            Asserts.fail("列名输入列表不能为空");
        }
        userPagesetService.remove(
                new QueryWrapper<UserPageset>()
                        .eq("user_id", SysUserUtils.getCurrentPrincipleId())
                        .eq("page_type", sysUserPagesetSaveVO.getPageType())
        );
        List<UserPageset> saveList = new ArrayList<>();
        Date now = DateUtil.now();
            String userId = SysUserUtils.getCurrentPrincipleId().toString();
        for (SysUserPagesetVO sysUserPagesetVO : sysUserPagesetSaveVO.getSysUserPagesetVOList()) {
            UserPageset sysUserPageset = new UserPageset();
            BeanUtil.copyProperties(sysUserPagesetVO, sysUserPageset);
            sysUserPageset.setPageType(sysUserPagesetSaveVO.getPageType());
            sysUserPageset.setUserId(Long.valueOf(userId));
            sysUserPageset.setCreator(userId);
            sysUserPageset.setModifier(userId);
            sysUserPageset.setGmtCreate(now);
            sysUserPageset.setGmtModified(now);
            saveList.add(sysUserPageset);
        }
        return userPagesetService.saveBatch(saveList);
    }

    /**
     * 获取默认列设置
     *
     * @param sysUserPagesetQueryVO
     * @return
     */
    public List<SysUserPagesetDTO> getDefaultPageSet(SysUserPagesetQueryVO sysUserPagesetQueryVO) {
        List<SysUserPagesetDTO> sysUserPagesetDTOList = new ArrayList<>();
        List<UserPageset> sysUserPagesetListDefault = userPagesetService.list(
                new QueryWrapper<UserPageset>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("user_id", -1)
                        .eq("page_type", sysUserPagesetQueryVO.getPageType())
                        .orderByAsc("order_no")
        );
        sysUserPagesetDTOList = BeanUtil.listCopyTo(sysUserPagesetListDefault, SysUserPagesetDTO.class);
        return sysUserPagesetDTOList;
    }
}
