package com.lantone.analyze.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.analyze.facade.analyze.BehospitalInfoManagementFacade;
import com.lantone.analyze.facade.analyze.compute.BehospitalInfoManagementFacade_New;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.analyze.AnalyzeDTO;
import com.lantone.common.dto.analyze.BasDeptInfoDTO;
import com.lantone.common.dto.analyze.BehospitalInfoDTO;
import com.lantone.common.vo.analyze.AnalyzeVO;
import com.lantone.common.vo.analyze.BasDeptInfoVO;
import com.lantone.common.vo.analyze.BehospitalPageVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Description:病历处理类
 * @Author songxl
 * @Date 2021/9/17
 */
@RestController
@RequestMapping("/beHosInfoManage")
@Api(value = "病历API", tags = { "病历API" })
public class BehospitalInfoManagementController {
    @Autowired
    private BehospitalInfoManagementFacade behospitalInfoManagementFacade;
    @Autowired
    private BehospitalInfoManagementFacade_New behospitalInfoManagementFacadeNew;

    @ApiOperation(value = "评分[by:zhoutg]",
            notes = "")
    @PostMapping("/analyze")
    @Transactional
    public CommonResult<AnalyzeDTO> analyze(@RequestBody AnalyzeVO analyzeVO) {
        return CommonResult.success(behospitalInfoManagementFacade.analyze(analyzeVO));
    }

    @ApiOperation(value = "评分[by:zhoutg]",notes = "")
    @PostMapping("/analyzeNew")
    @Transactional
    public CommonResult<AnalyzeDTO> analyzeNew(@RequestBody AnalyzeVO analyzeVO) {
        return CommonResult.success(behospitalInfoManagementFacadeNew.analyze(analyzeVO));
    }

    @ApiOperation(value = "获取病历质控一览下医院科室下拉列表信息[by:gaodm]",
            notes = "inputStr: 搜索参数")
    @PostMapping("/getList")
    public CommonResult<List<BasDeptInfoDTO>> getList(@RequestBody BasDeptInfoVO basDeptInfoVO) {
        return CommonResult.success(behospitalInfoManagementFacade.listForUser(basDeptInfoVO));
    }

    @ApiOperation(value = "病历分页列表[by:zhoutg]",
            notes = "behospitalCode：病人住院序号<br>" +
                    "name：病人姓名<br>" +
                    "fileCode：档案号<br>" +
                    "statisticsType: 统计维度 1-本月，2-本年<br>" +
                    "behosDateStart：入院时间开始时间<br>" +
                    "behosDateEnd：入院时间结束时间<br>" +
                    "leaveHosDateStart：出院时间开始时间<br>" +
                    "leaveHosDateEnd：出院时间结束时间<br>" +
                    "doctorName：医生姓名<br>" +
                    "doctorCode：医生工号<br>" +
                    "level：等级<br>" +
                    "deptId：科室id<br>")
    @PostMapping("/page")
    public CommonResult<IPage<BehospitalInfoDTO>> page(@RequestBody BehospitalPageVO behospitalPageVO) {
        IPage<BehospitalInfoDTO> data = behospitalInfoManagementFacade.pageFac(behospitalPageVO);
        return CommonResult.success(data);
    }
}
