package com.lantone.report.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.collect.Lists;
import com.lantone.common.dto.report.EntryNumDTO;
import com.lantone.common.dto.report.EntryNumGroupDTO;
import com.lantone.common.dto.report.EntryStatisticsDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.util.ExcelUtils;
import com.lantone.common.util.ListUtil;
import com.lantone.common.vo.report.EntryStatisticsVO;
import com.lantone.common.vo.report.FilterPageVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import com.lantone.dblayermbg.facade.report.QcresultInfoFacade;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;


@Component
public class EntryCaseFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    /**
     * 条目缺陷占比-控制台
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByEntry(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterFacade.filterVOSet(filterVO);
        List<NumDTO> entryList = qcresultInfoFacade.getBaseMapper().entryCountGroupByEntry(filterVO);
        if (ListUtil.isNotEmpty(entryList)) {
            retMap.put("条目缺陷占比", entryList);
        }
        return retMap;
    }

    /**
     * 条目缺陷占比-报表
     *
     * @return
     */
    public IPage<EntryNumDTO> entryGroupByEntryInnerPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        //基础数据
        IPage<EntryNumDTO> entryNumDTOIPage = qcresultInfoFacade.getBaseMapper().entryGroupByEntryInnerPage(filterPageVO);
        List<EntryNumDTO> records = entryNumDTOIPage.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return entryNumDTOIPage;
        }

        //缺陷总数
        Integer sumInteget = qcresultInfoFacade.getBaseMapper().entryGroupByEntrySum(filterPageVO.getHospitalId(), filterPageVO.getIsPlacefile(),
                filterPageVO.getStartDate(), filterPageVO.getEndDate());
        float nsum = 0;
        Set<Long> ids = new HashSet<Long>();
        for (EntryNumDTO record : records) {
            ids.add(record.getId());
            int num = record.getNum().intValue();
            if (sumInteget != null && sumInteget != 0) {
                int sum = sumInteget.intValue();
                nsum = (float) num / sum;
            }
            DecimalFormat df = new DecimalFormat("0.00");
            String percent = df.format(nsum * 100);
            record.setPercentStr(percent + "%");
            record.setTotleNum(sumInteget);
        }
        return entryNumDTOIPage;
    }

    /**
     * 条目缺陷占比-报表导出
     *
     * @param filterPageVO
     * @return
     */
    public void entryGroupByEntryExport(HttpServletResponse response, FilterPageVO filterPageVO) {
        filterPageVO.setCurrent(1L);
        filterPageVO.setSize(Long.MAX_VALUE);
        filterPageVO.setSearchCount(false);
        filterFacade.filterPageVOSet(filterPageVO);
        IPage<EntryNumDTO> page = this.entryGroupByEntryInnerPage(filterPageVO);
        String fileName = "条目缺陷占比.xls";
        ExcelUtils.exportExcel(page.getRecords(), null, "sheet1", EntryNumDTO.class, fileName, response, 12.8f);
    }

    /**
     * 单项否决缺陷占比
     *
     * @param filterVO
     * @return
     */
    public List<EntryNumDTO> entryRejectPercent(FilterVO filterVO) {
        List<EntryNumDTO> retList = Lists.newLinkedList();
        filterFacade.filterVOSet(filterVO);
        Integer limitCount = filterVO.getLimitCount();
        List<EntryNumDTO> numDTOList = qcresultInfoFacade.getBaseMapper().entryRejectPercent(filterVO);
        if (ListUtil.isNotEmpty(numDTOList)) {
            if (numDTOList.size() <= limitCount) {
                retList = numDTOList;
            } else {
                retList = numDTOList.subList(0, limitCount);
            }
        }
        return retList;
    }

    /**
     * 单条条目缺陷统计（首页）
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> casesEntryStatisticsById(FilterVO filterVO) {
        filterFacade.filterVOSet(filterVO);
        List<NumDTO> records = behospitalInfoFacade.getBaseMapper().casesEntryStatisticsById(filterVO);

        if (ListUtil.isNotEmpty(records)) {
            NumDTO globleRecord = new NumDTO();
            globleRecord.setName("全院");
            Integer num = records.stream()
                    .map(NumDTO::getNum)
                    .reduce(0, Integer::sum);
            globleRecord.setNum(num);
            records.add(0, globleRecord);
        }

        if (ListUtil.isNotEmpty(records) && records.size() > filterVO.getLimitCount()) {
            records = records.subList(0, 10);
        }
        return records;
    }

    /**
     * 关键条目缺陷占比统计
     *
     * @param entryStatisticsVO
     * @return
     */
    public List<EntryStatisticsDTO> entryStatistics(EntryStatisticsVO entryStatisticsVO) {
        filterFacade.entryStatisticsVOSet(entryStatisticsVO);
        if (entryStatisticsVO.getDeptName().equals("全院")) {
            entryStatisticsVO.setDeptName("");
        }
        List<EntryStatisticsDTO> records = behospitalInfoFacade.getBaseMapper().entryStatistics(entryStatisticsVO);
        //增加全院数据
        if (entryStatisticsVO.getDeptName().equals("")) {
            EntryStatisticsDTO record = getEntryGlobleRecord(records);
            if (record != null) {
                records.add(0, record);
            }
        }
        return records;
    }

    /**
     * 缺陷详情（分页）
     * @param filterPageVO
     * @return
     */
    public IPage<EntryNumGroupDTO> entryCountGroupByEntryPage(FilterPageVO filterPageVO) {
        filterFacade.filterPageVOSet(filterPageVO);
        if (filterPageVO.getDeptName().equals("全院")) {
            filterPageVO.setDeptName("");
        }
        IPage<EntryNumGroupDTO> page = qcresultInfoFacade.getBaseMapper().entryCountGroupByEntryPage(filterPageVO);
        return page;
    }

    /**
     * 关键条目缺陷占比-缺陷列表-报表导出
     *
     * @param filterPageVO
     * @return
     */
    public void entryCountGroupByEntryExport(HttpServletResponse response, FilterPageVO filterPageVO) {
        filterPageVO.setCurrent(1L);
        filterPageVO.setSize(Long.MAX_VALUE);
        filterPageVO.setSearchCount(false);
        IPage<EntryNumGroupDTO> page = this.entryCountGroupByEntryPage(filterPageVO);
        String fileName = "缺陷详情.xls";
        ExcelUtils.exportExcel(page.getRecords(), null, "sheet1", EntryNumGroupDTO.class, fileName, response, 12.8f);
    }

    /**
     * 关键条目缺陷占比统计
     *
     * @param entryStatisticsVO
     * @return
     */
    public void entryStatisticsExport(HttpServletResponse response, EntryStatisticsVO entryStatisticsVO) {
        List<EntryStatisticsDTO> records = this.entryStatistics(entryStatisticsVO);
        String fileName = "关键条目缺陷占比统计.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", EntryStatisticsDTO.class, fileName, response, 12.8f);
    }

    /**
     * 关键条目缺陷占比统计增加全院记录
     *
     * @param records
     * @return
     */
    private EntryStatisticsDTO getEntryGlobleRecord(List<EntryStatisticsDTO> records) {
        DecimalFormat df = new DecimalFormat("#0.00");
        if (ListUtil.isEmpty(records)) {
            return null;
        }

        EntryStatisticsDTO record = new EntryStatisticsDTO();
        record.setDeptName("全院");
        //未在24小时内完成会诊
        Integer consultationNum = records
                .stream()
                .map(EntryStatisticsDTO::getConsultationNum)
                .reduce(0, Integer::sum);
        record.setConsultationNum(consultationNum);
        record.setConsultationEntryId(records.get(0).getConsultationEntryId());
        Integer consultationMRNum = records
                .stream()
                .map(EntryStatisticsDTO::getConsultationMRNum)
                .reduce(0, Integer::sum);
        record.setConsultationMRNum(consultationMRNum);
        if (consultationMRNum != null && !consultationMRNum.equals(0)) {
            Double consultationPercent = BigDecimal.valueOf(consultationNum)
                    .divide(BigDecimal.valueOf(consultationMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String consultationPercentStr
                    = df.format(BigDecimal.valueOf(consultationPercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setConsultationPercent(consultationPercent);
            record.setConsultationPercentStr(consultationPercentStr);
        }
        //手术记录名称不匹配
        Integer operationNameNum = records
                .stream()
                .map(EntryStatisticsDTO::getOperationNameNum)
                .reduce(0, Integer::sum);
        record.setOperationNameNum(operationNameNum);
        record.setOperationNameEntryId(records.get(0).getOperationNameEntryId());
        Integer operationMRNum = records
                .stream()
                .map(EntryStatisticsDTO::getOperationMRNum)
                .reduce(0, Integer::sum);
        record.setOperationMRNum(operationMRNum);
        if (operationMRNum != null && !operationMRNum.equals(0)) {
            Double operationNamePercent = BigDecimal.valueOf(operationNameNum)
                    .divide(BigDecimal.valueOf(operationMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String operationNamePercentStr
                    = df.format(BigDecimal.valueOf(operationNamePercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setOperationNamePercent(operationNamePercent);
            record.setOperationNamePercentStr(operationNamePercentStr);
        }
        //术后首程未在15分钟内完成
        Integer operation15MinuteNum = records
                .stream()
                .map(EntryStatisticsDTO::getOperation15MinuteNum)
                .reduce(0, Integer::sum);
        record.setOperation15MinuteNum(operation15MinuteNum);
        record.setOperation15MinuteEntryId(records.get(0).getOperation15MinuteEntryId());
        if (operationMRNum != null && !operationMRNum.equals(0)) {
            Double operation15MinutePercent = BigDecimal.valueOf(operation15MinuteNum)
                    .divide(BigDecimal.valueOf(operationMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String operation15MinutePercentStr
                    = df.format(BigDecimal.valueOf(operation15MinutePercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setOperation15MinutePercent(operation15MinutePercent);
            record.setOperation15MinutePercentStr(operation15MinutePercentStr);
        }
        //病案首页手术时间不匹配
        Integer operationTimeNum = records
                .stream()
                .map(EntryStatisticsDTO::getOperationTimeNum)
                .reduce(0, Integer::sum);
        record.setOperationTimeNum(operationTimeNum);
        record.setOperationTimeEntryId(records.get(0).getOperationTimeEntryId());
        if (operationMRNum != null && !operationMRNum.equals(0)) {
            Double operationTimePercent = BigDecimal.valueOf(operationTimeNum)
                    .divide(BigDecimal.valueOf(operationMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String operationTimePercentStr
                    = df.format(BigDecimal.valueOf(operationTimePercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setOperationTimePercent(operationTimePercent);
            record.setOperationTimePercentStr(operationTimePercentStr);
        }
        //未在6小时内书写危机值记录
        Integer crisisNum = records
                .stream()
                .map(EntryStatisticsDTO::getCrisisNum)
                .reduce(0, Integer::sum);
        record.setCrisisNum(crisisNum);
        record.setCrisisEntryId(records.get(0).getCrisisEntryId());
        Integer crisisMRNum = records
                .stream()
                .map(EntryStatisticsDTO::getCrisisMRNum)
                .reduce(0, Integer::sum);
        record.setCrisisMRNum(crisisMRNum);
        if (crisisMRNum != null && !crisisMRNum.equals(0)) {
            Double crisisPercent = BigDecimal.valueOf(crisisNum)
                    .divide(BigDecimal.valueOf(crisisMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String crisisPercentStr
                    = df.format(BigDecimal.valueOf(crisisPercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setCrisisPercent(crisisPercent);
            record.setCrisisPercentStr(crisisPercentStr);
        }
        //病人住院超过30天，阶段小节书写
        Integer stageSummaryNum = records
                .stream()
                .map(EntryStatisticsDTO::getStageSummaryNum)
                .reduce(0, Integer::sum);
        record.setStageSummaryNum(stageSummaryNum);
        record.setStageSummaryEntryId(records.get(0).getStageSummaryEntryId());
        Integer stageSummaryMRNum = records
                .stream()
                .map(EntryStatisticsDTO::getStageSummaryMRNum)
                .reduce(0, Integer::sum);
        record.setStageSummaryMRNum(stageSummaryMRNum);
        if (stageSummaryMRNum != null && !stageSummaryMRNum.equals(0)) {
            Double stageSummaryPercent = BigDecimal.valueOf(stageSummaryNum)
                    .divide(BigDecimal.valueOf(stageSummaryMRNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String stageSummaryPercentStr
                    = df.format(BigDecimal.valueOf(stageSummaryPercent).multiply(BigDecimal.valueOf(100))) + "%";
            record.setStageSummaryPercent(stageSummaryPercent);
            record.setStageSummaryPercentStr(stageSummaryPercentStr);
        }
        //住院超过31日病人数
        Integer beHosGT31DaysMRNum = records
                .stream()
                .map(EntryStatisticsDTO::getBeHosGT31DaysMRNum)
                .reduce(0, Integer::sum);
        record.setBeHosGT31DaysMRNum(beHosGT31DaysMRNum);
        return record;
    }
}