package com.lantone.report.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.ResultCode;
import com.lantone.common.dto.report.ExportExcelBehospitalDTO;
import com.lantone.common.dto.report.ExportExcelDTO;
import com.lantone.common.dto.report.ExportExcelWideBehospitalDTO;
import com.lantone.common.dto.report.ExportWideExcelDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.dto.report.QcResultShortDTO;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.ExcelUtils;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.QcResultShortPageVO;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

import java.util.List;
import java.util.Map;


@Component
public class LeaveBeHospitalFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    /**
     * 出院人数统计控制台
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> leaveHosCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterFacade.filterVOSet(filterVO);
        Map<String, Object> leaveHosMap = new HashMap<>();
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            leaveHosMap
                    = dataBeanAggregateQueryFacade.get("setAllLeaveHos", invokeParams, Map.class);
        } catch (Exception e) {
            throw new ApiException(ResultCode.SERVER_IS_ERROR);
        }
        if (leaveHosMap != null) {
            retMap.put("出院人数统计", leaveHosMap);
        }
        return retMap;
    }

    /**
     * 出院人数报表统计
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> leaveHosMRPage(QcResultShortPageVO qcResultShortPageVO) {
        filterFacade.qcResultShortPageVOSet(qcResultShortPageVO);
        IPage<QcResultShortDTO> page = behospitalInfoFacade.getBaseMapper().leaveHosMRPage(qcResultShortPageVO);
        return page;
    }

    /**
     * 出院人数报表统计导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    public void leaveHosMrPageExportMethod(HttpServletResponse response, QcResultShortPageVO qcResultShortPageVO) {
        String hospitalId = String.valueOf(SysUserUtils.getCurrentHospitalId());
        qcResultShortPageVO.setHospitalId(hospitalId);
        if (StringUtil.isNotBlank(qcResultShortPageVO.getDeptName())
                && qcResultShortPageVO.getDeptName().equals("全部")) {
            qcResultShortPageVO.setDeptName("");
        }
        Date startDate = qcResultShortPageVO.getStartDate();
        Date endDate = qcResultShortPageVO.getEndDate();
        //时间间隔7天
        long interval_7 = 7 * 24 * 60 * 60 * 1000;
        long interval_90 = 90 * 24 * 60 * 60 * 1000l;

        if ("1".equals(qcResultShortPageVO.getRadioCheck())) {
            //时间间隔7天
            if (endDate.getTime() < startDate.getTime()) {
                throw new ApiException("截止时间不能小于起始时间");
            }
            if (endDate.getTime() - startDate.getTime() > interval_7) {
                throw new ApiException("统计区间不能大于7天");
            }
        } else {
            //时间间隔90天
            if (endDate.getTime() < startDate.getTime()) {
                throw new ApiException("截止时间不能小于起始时间");
            }
            if (endDate.getTime() - startDate.getTime() > interval_90) {
                throw new ApiException("统计区间不能大于90天");
            }
        }

        qcResultShortPageVO.setCurrent(1L);
        qcResultShortPageVO.setSize(Long.MAX_VALUE);
        qcResultShortPageVO.setSearchCount(false);
        List<ExportExcelDTO> records = behospitalInfoFacade.getBaseMapper().leaveHosMRPageExport(qcResultShortPageVO);
        List<ExportWideExcelDTO> resWide = new ArrayList<ExportWideExcelDTO>();
        if ("2".equals(qcResultShortPageVO.getRadioCheck())) {
            resWide = transList(records, resWide);
        }
        String fileName = "出院人数统计.xls";
        if ("1".equals(qcResultShortPageVO.getRadioCheck())) {
            ExcelUtils.exportExcelUser(records, null, "sheet1", ExportExcelDTO.class, fileName, response);
        }
        if ("2".equals(qcResultShortPageVO.getRadioCheck())) {
            ExcelUtils.exportExcelUser(resWide, null, "sheet1", ExportWideExcelDTO.class, fileName, response);
        }
    }

    public List<ExportWideExcelDTO> transList(List<ExportExcelDTO> res, List<ExportWideExcelDTO> resWide) {
        if (ListUtil.isNotEmpty(res)) {
            res.forEach(exportExcelDTO -> {
                ExportWideExcelDTO exportWideExcelDTO = new ExportWideExcelDTO();
                exportWideExcelDTO.setAvgScore(exportExcelDTO.getAvgScore());
                exportWideExcelDTO.setBehDeptName(exportExcelDTO.getBehDeptName());
                List<ExportExcelBehospitalDTO> excelBehospitalDTOS = exportExcelDTO.getExcelBehospitalDTOS();
                List<ExportExcelWideBehospitalDTO> exportExcelWideBehospitalDTOs = new ArrayList<>();
                excelBehospitalDTOS.forEach(exportExcelBehospitalDTO -> {
                    ExportExcelWideBehospitalDTO exportExcelWideBehospitalDTO = new ExportExcelWideBehospitalDTO();
                    exportExcelWideBehospitalDTO.setDoctorName(exportExcelBehospitalDTO.getDoctorName());
                    exportExcelWideBehospitalDTO.setBehospitalCode(exportExcelBehospitalDTO.getBehospitalCode());
                    exportExcelWideBehospitalDTO.setBehospitalDate(exportExcelBehospitalDTO.getBehospitalDate());
                    exportExcelWideBehospitalDTO.setLeaveHospitalDate(exportExcelBehospitalDTO.getLeaveHospitalDate());
                    exportExcelWideBehospitalDTO.setPatName(exportExcelBehospitalDTO.getPatName());
                    exportExcelWideBehospitalDTO.setScore(exportExcelBehospitalDTO.getScore());
                    exportExcelWideBehospitalDTO.setScoreBn(exportExcelBehospitalDTO.getScoreBn());
                    exportExcelWideBehospitalDTOs.add(exportExcelWideBehospitalDTO);
                });
                exportWideExcelDTO.setExcelBehospitalDTOS(exportExcelWideBehospitalDTOs);
                resWide.add(exportWideExcelDTO);
            });
        }
        return resWide;
    }

    /**
     * 入院人数统计（首页）
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> beHosCount(FilterVO filterVO) {
        filterFacade.filterVOSet(filterVO);
        List<NumDTO> records = behospitalInfoFacade.getBaseMapper().beHosCount(filterVO);

        if (ListUtil.isNotEmpty(records)) {
            NumDTO globleRecord = new NumDTO();
            globleRecord.setName("全院");
            Integer num = records.stream()
                    .map(NumDTO::getNum)
                    .reduce(0, Integer::sum);
            globleRecord.setNum(num);
            records.add(0, globleRecord);
        }

        if (ListUtil.isNotEmpty(records) && records.size() > filterVO.getLimitCount()) {
            records = records.subList(0, 10);
        }
        return records;
    }
}