package com.lantone.report.facade;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.lantone.common.constant.QualityContent;
import com.lantone.common.constant.TimeContent;
import com.lantone.common.dto.report.MedManageParamsDTO;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.IndexTimeVO;
import com.lantone.common.vo.report.MedIndexFilterVO;
import com.lantone.dblayermbg.entity.report.BehospitalInfo;
import com.lantone.dblayermbg.entity.report.IndexRelevance;
import com.lantone.dblayermbg.entity.report.IndexResult;
import com.lantone.dblayermbg.entity.report.ManagementInfo;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import com.lantone.dblayermbg.facade.report.IndexRelevanceFacade;
import com.lantone.dblayermbg.facade.report.IndexResultFacade;
import com.lantone.dblayermbg.facade.report.ManagementInfoFacade;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.stream.Collectors;


@Component
public class MrIndicatorFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;
    @Autowired
    private IndexResultFacade medIndexResultFacade;
    @Autowired
    private IndexRelevanceFacade medIndexRelevanceFacade;
    @Autowired
    private ManagementInfoFacade managementInfoFacade;

    /**
     * 病案指标
     * @param filterVO
     * @return
     */
    public List<Map<String,String>> medicalRecordIndicator(FilterVO filterVO){
        filterFacade.filterSet(filterVO);
        List<Map<String, String>> hashMapArrayList = new ArrayList<>();
        String indexName = filterVO.getIndexName();
        //各指标未完成人数情况 ---分子
        Map<String, Long> triggeringRulesMap = behospitalInfoFacade.getBaseMapper().triggeringRules(filterVO);
        Long operationCount = triggeringRulesMap.get(QualityContent.SSJLWCC_24H);
        Long InHospitalRecords = triggeringRulesMap.get(QualityContent.RYJLWWC_24H);
        Long OutHospitalRecords = triggeringRulesMap.get(QualityContent.CYJLWWC_24H);
        Long recordHomePage = triggeringRulesMap.get(QualityContent.BASYWWC_24H);
        //病理手术
        Long operationPathologyFee = triggeringRulesMap.get("operationPathologyFee");
        //CT
        Long ct = triggeringRulesMap.get("CT");
        //MR
        Long mr = triggeringRulesMap.get("MR");
        //细菌培养
        Long bacterialCulture = triggeringRulesMap.get("bacterialCulture");
        //抗菌药物
        Long antibiosis = triggeringRulesMap.get("antibiosis");
        //恶性肿瘤化学治疗
        Long chemotherapyMalignantTumors = triggeringRulesMap.get("chemotherapyMalignantTumors");

        Map<String, Double> medicalRecordIndicator = medIndexResultFacade.getBaseMapper().medicalRecordIndicator(filterVO);
        Double startCount = medicalRecordIndicator.get("startCount");
        Double operationFeeNum = medicalRecordIndicator.get("operationFeeNum");
        Double CTNum = medicalRecordIndicator.get("CTNum");
        Double MRNum = medicalRecordIndicator.get("MRNum");
        Double operationPathologyFeeNum = medicalRecordIndicator.get("operationPathologyFeeNum");
        Double bacterialCultureNum = medicalRecordIndicator.get("bacterialCultureNum");
        Double antibiosisFeeNum = medicalRecordIndicator.get("antibiosisFeeNum");
        Double chemotherapyMalignantTumorsNum = medicalRecordIndicator.get("chemotherapyMalignantTumorsNum");
        MedIndexFilterVO timeVo = new MedIndexFilterVO();
        BeanUtils.copyProperties(filterVO,timeVo);
        timeTrans(timeVo);
        Map<String, Object> baseIndex = behospitalInfoFacade.getBaseMapper().getBaseIndex(timeVo);
        //指标一、二、三
        if (QualityContent.FOURSTR.contains(indexName)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = ManageIndexMethod(indexName, hashMapArrayList, baseIndex, timeVo);
        }
        //入院记录24小时完成率
        if(indexName.equals(QualityContent.RYJL_24H_WCL)||indexName.equals(QualityContent.QB)) {
            if (InHospitalRecords != null && startCount != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.RYJL_24H_WCL);
                stringStringHashMap.put("notCopiedStr", getResult(startCount - InHospitalRecords, startCount.doubleValue()));
                stringStringHashMap.put("num", Double.valueOf(startCount).toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //手术记录完成率
        if (indexName.equals(QualityContent.SSJL_24H_WCL)||indexName.equals(QualityContent.QB)) {
            if (operationFeeNum != null && operationCount != null) {
                LinkedHashMap<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.SSJL_24H_WCL);
                stringStringHashMap.put("notCopiedStr", getResult(operationFeeNum - operationCount, operationFeeNum));
                stringStringHashMap.put("num", operationFeeNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //出院记录24小时完成率
        if(indexName.equals(QualityContent.CYJL_24H_WCL)||indexName.equals(QualityContent.QB)) {
            if (OutHospitalRecords != null && startCount != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.CYJL_24H_WCL);
                stringStringHashMap.put("notCopiedStr", getResult(startCount - OutHospitalRecords, startCount.doubleValue()));
                stringStringHashMap.put("num", Double.valueOf(startCount).toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //病案首页24小时完成率
        if(indexName.equals(QualityContent.BASY_24H_WCL)||indexName.equals(QualityContent.QB)) {
            if (recordHomePage != null && startCount != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.BASY_24H_WCL);
                stringStringHashMap.put("notCopiedStr", getResult(startCount - recordHomePage, startCount.doubleValue()));
                stringStringHashMap.put("num", Double.valueOf(startCount).toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //CT完成率
        if (indexName.equals(QualityContent.CT_JCJL_FHL)||indexName.equals(QualityContent.QB)) {
            if (CTNum != null && ct != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.CT_JCJL_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(CTNum - ct, CTNum));
                stringStringHashMap.put("num", CTNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //MRI完成率
        if (indexName.equals(QualityContent.MR_JCJL_FHL)||indexName.equals(QualityContent.QB)) {
            if (MRNum != null && mr != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.MR_JCJL_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(MRNum - mr, MRNum));
                stringStringHashMap.put("num", MRNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //病理完成率为
        if (indexName.equals(QualityContent.BL_JCJL_FHL)||indexName.equals(QualityContent.QB)) {
            if (operationPathologyFeeNum != null && operationPathologyFee != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.BL_JCJL_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(operationPathologyFeeNum - operationPathologyFee, operationPathologyFeeNum));
                stringStringHashMap.put("num", operationPathologyFeeNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //细菌培养
        if (indexName.equals(QualityContent.XJPY_JCJL_FHL)||indexName.equals(QualityContent.QB)) {
            if (bacterialCultureNum != null && bacterialCulture != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.XJPY_JCJL_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(bacterialCultureNum - bacterialCulture, bacterialCultureNum));
                stringStringHashMap.put("num", bacterialCultureNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //抗菌
        if (indexName.equals(QualityContent.KJYW_SY_FHL)||indexName.equals(QualityContent.QB)) {
            if (antibiosisFeeNum != null && antibiosis != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.KJYW_SY_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(antibiosisFeeNum - antibiosis, antibiosisFeeNum));
                stringStringHashMap.put("num", antibiosisFeeNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //恶性肿瘤化学治疗
        if (indexName.equals(QualityContent.EXZL_HXZL_FHL)||indexName.equals(QualityContent.QB)) {
            if (chemotherapyMalignantTumorsNum != null && chemotherapyMalignantTumors != null) {
                Map<String, String> stringStringHashMap = new LinkedHashMap<>();
                stringStringHashMap.put("name", QualityContent.EXZL_HXZL_FHL);
                stringStringHashMap.put("notCopiedStr", getResult(chemotherapyMalignantTumorsNum - chemotherapyMalignantTumors, chemotherapyMalignantTumorsNum));
                stringStringHashMap.put("num", chemotherapyMalignantTumorsNum.toString());
                hashMapArrayList.add(stringStringHashMap);
            }
        }
        //指标十四 恶性肿瘤放射治疗
        if (indexName.equals(QualityContent.EXZL_FSZL_FHL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = tumorRadiationMethod(medicalRecordIndicator,hashMapArrayList,triggeringRulesMap);
        }

        //指标十五 手术记录
        if (indexName.equals(QualityContent.SS_XGJL_WCL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = operationMethod(medicalRecordIndicator,hashMapArrayList,triggeringRulesMap);
        }

        //指标十六 植入物相关记录符合率
        if (indexName.equals(QualityContent.ZRW_XGJL_WCL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = implantsMethod(medicalRecordIndicator,hashMapArrayList,triggeringRulesMap);
        }

        //指标十七  临床用血相关记录符合率
        if (indexName.equals(QualityContent.LCYX_XGJL_WCL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = bloodMethod(hashMapArrayList,medicalRecordIndicator,triggeringRulesMap);
        }

        //指标十八 医师查房记录
        if (indexName.equals(QualityContent.YSCF_JL_WCL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = WardRoundMethod(hashMapArrayList, baseIndex, triggeringRulesMap);
        }

        //指标十九 抢救记录
        if (indexName.equals(QualityContent.HZQJ_JLJS_WCL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = rescueMethod(medicalRecordIndicator,hashMapArrayList,triggeringRulesMap);
        }

        //指标二十 出院患者病历2日归档
        if (indexName.equals(QualityContent.CCHZ_2DAY_GDL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = getfileSecAmount(medicalRecordIndicator,hashMapArrayList, baseIndex);
        }

        //指标二十一 出院患者病历归档完整率
        if (indexName.equals(QualityContent.CCHZ_GD_WZL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = fileCompleteMethod(medicalRecordIndicator,hashMapArrayList, baseIndex);
        }

        //指标二十二 不合理复制病历发生率
        if (indexName.equals(QualityContent.BHL_FZ_FSL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = unreasonedCopyMethod(hashMapArrayList, baseIndex, triggeringRulesMap);
        }

        //指标二十三 知情同意书规范签署率
        if (indexName.equals(QualityContent.ZQTYS_GF_QSL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = contentSignMethod(hashMapArrayList, baseIndex, triggeringRulesMap);
        }

        //指标二十四 甲级率
        if (indexName.equals(QualityContent.JJBLL)||indexName.equals(QualityContent.QB)) {
            hashMapArrayList = firMethod(hashMapArrayList,baseIndex);
        }
        return hashMapArrayList;
    }



    /**
     * 病案指标数据保存
     *
     * @param IndexTimeVO
     * @return 2170
     */
    @Transactional
    public Boolean saveMedicaIndicator(IndexTimeVO IndexTimeVO){
        FilterVO filterVO = new FilterVO();
        BeanUtils.copyProperties(IndexTimeVO,filterVO);
        List<String> timeList = filterFacade.getTimeList(filterVO);
        QueryWrapper<IndexResult> qw = new QueryWrapper<>();
        qw.eq("hospital_id",filterVO.getHospitalId());
        qw.ge("start_date",filterVO.getStartDate()+" 00:00:00");
        qw.le("end_date", handleTime(filterVO.getEndDate()));
        medIndexResultFacade.remove(qw);
        QueryWrapper<IndexRelevance> qwa = new QueryWrapper<>();
        qwa.eq("hospital_id",filterVO.getHospitalId());
        qwa.ge("start_date",filterVO.getStartDate()+" 00:00:00");
        qwa.le("end_date", handleTime(filterVO.getEndDate()));
        medIndexRelevanceFacade.remove(qwa);
        for (String startTime : timeList) {
            filterVO.setStartDate(startTime);
            filterVO.setEndDate( handleTime(startTime));
            getMedicaIndicator(filterVO);
        }
        return true;
    }

    /**
     * 病案指标数据保存
     *
     * @param filterVO
     * @return 2170
     */
    public void getMedicaIndicator(FilterVO filterVO){
        IndexResult medIndexResult = new IndexResult();
        List<Map<String, String>> selectOperationNumMap = behospitalInfoFacade.getBaseMapper().selectOperationNum(filterVO);
        Set<String> outHospitalCode = new CopyOnWriteArraySet<>();
        Set<String> operationCode = new CopyOnWriteArraySet<>();
        Set<String> operationPathologyCode = new CopyOnWriteArraySet<>();
        Set<String> antibiosisCode = new CopyOnWriteArraySet<>();
        List<String> adviceVerify = adviceVerify(filterVO);
        Long operationFeeNum = 0L;
        filterVO.setFlagStr("2");
        List<MedManageParamsDTO> doctorAdviceMedManageParams = behospitalInfoFacade.getBaseMapper().getMedManageParams(filterVO);
        Map<String, Set<String>> adviceNum = getAdviceNum(filterVO, doctorAdviceMedManageParams);
        if(ListUtil.isNotEmpty(selectOperationNumMap)) {
            for (Map<String, String> stringStringMap : selectOperationNumMap) {
                String behospitalCode = stringStringMap.get("behospitalCode");
                //出院code
                outHospitalCode.add(behospitalCode);
                String operationFee = stringStringMap.get("operationFee");
                String pathologyFee = stringStringMap.get("pathologyFee");
                String antibiosisFee = stringStringMap.get("antibiosisFee");
                if(StringUtil.isNotEmpty(operationFee)) {
                    double of = 0.0;
                    try {
                        of = Double.parseDouble(operationFee);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    if (of > 0) {
                        //手术code
                        operationCode.add(behospitalCode);
                    }
                }
                if(StringUtil.isNotEmpty(operationFee)&&StringUtil.isNotEmpty(pathologyFee)) {
                    double of = 0.0;
                    double pf = 0.0;
                    try {
                        of = Double.parseDouble(operationFee);
                        pf = Double.parseDouble(pathologyFee);
                    } catch (NumberFormatException e) {
                        e.printStackTrace();
                    }
                    if (of > 0 && pf > QualityContent.pathologyFee) {
                        //病理code
                        operationPathologyCode.add(behospitalCode);
                    }
                }
                if(StringUtil.isNotEmpty(antibiosisFee)) {
                    double af = 0.0;
                    try {
                        af = Double.parseDouble(antibiosisFee);
                    } catch (NumberFormatException e) {
                        e.printStackTrace();
                    }
                    if (af > 0) {
                        //抗菌code
                        antibiosisCode.add(behospitalCode);
                    }
                }
            }
        }
        if(ListUtil.isNotEmpty(adviceVerify)){
            for (String code : adviceVerify) {
                outHospitalCode.add(code);
            }
        }
        Long startCount = Long.valueOf(outHospitalCode.size());
        //手术人
        Set<String> operationList = adviceNum.get("operation");
        if(operationList!=null&&operationList.size()>0){
            for (String s : operationList) {
                operationCode.add(s);
            }
        }
        if(operationCode.size()>0&&operationCode!=null) {
            operationFeeNum = Long.valueOf(operationCode.size());
        }
        //CT人数
        Set<String> ctList = adviceNum.get("CT");
        Long CTNum=0L;
        if(ctList!=null&&ctList.size()>0) {
            CTNum = Long.valueOf(ctList.size());
        }
        //MR人数
        Set<String> mrList = adviceNum.get("MR");
        Long MRNum=0L;
        if(mrList!=null&&mrList.size()>0) {
            MRNum = Long.valueOf(mrList.size());
        }
        //手术病理人数
        Long operationPathologyFeeNum=0L;
        Set<String> operationPathologyList = adviceNum.get("operationPathology");
        if(operationPathologyList!=null&&operationPathologyList.size()>0){
            for (String s : operationPathologyList) {
                operationPathologyCode.add(s);
            }
        }
        if(operationPathologyCode.size()>0&&operationPathologyCode!=null) {
            operationPathologyFeeNum = Long.valueOf(operationPathologyCode.size());
        }
        //细菌培养人数
        Set<String> bacterialCultureList = adviceNum.get("bacterialCulture");
        Long bacterialCultureNum=0L;
        if(bacterialCultureList!=null&&bacterialCultureList.size()>0) {
            bacterialCultureNum = Long.valueOf(bacterialCultureList.size());
        }
        //抗菌药物人数
        Set<String> antibiosisList = adviceNum.get("antibiosis");
        if(antibiosisList!=null&&antibiosisList.size()>0) {
            for (String s : antibiosisList) {
                antibiosisCode.add(s);
            }
        }
        Long antibiosisFeeNum =0L;
        if(antibiosisCode.size()>0&&antibiosisCode!=null) {
            antibiosisFeeNum = Long.valueOf(antibiosisCode.size());
        }
        //恶性肿瘤化学治疗人数
        Set<String> chemotherapyMalignantTumorsList = adviceNum.get("chemotherapyMalignantTumors");
        Long chemotherapyMalignantTumorsNum =0L;
        if(chemotherapyMalignantTumorsList!=null&&chemotherapyMalignantTumorsList.size()>0) {
            chemotherapyMalignantTumorsNum=Long.valueOf(chemotherapyMalignantTumorsList.size());
        }
        //出院人数
        if(startCount!=null){
            medIndexResult.setAdmissionResult(Double.valueOf(startCount));
        }
        //手术完成人数为
        if (operationFeeNum != null) {
            medIndexResult.setOperationResult(Double.valueOf(operationFeeNum));
        }
        //CT人数
        if (CTNum != null) {
            medIndexResult.setCtResult(Double.valueOf(CTNum));
        }
        //MRI人数
        if (MRNum != null ) {
            medIndexResult.setMriResult(Double.valueOf(MRNum));
        }
        //病理人数
        if (operationPathologyFeeNum != null) {
            medIndexResult.setPathologyResult(Double.valueOf(operationPathologyFeeNum));
        }
        //细菌培养人数
        if (bacterialCultureNum != null) {
            medIndexResult.setGermResult(Double.valueOf(bacterialCultureNum));
        }
        //抗菌人数
        if (antibiosisFeeNum != null ) {
            medIndexResult.setAntibiosisResult(Double.valueOf(antibiosisFeeNum));
        }
        //恶性肿瘤化学治疗人数
        if (chemotherapyMalignantTumorsNum != null ) {
            medIndexResult.setTumorChemistryResult(Double.valueOf(chemotherapyMalignantTumorsNum));
        }

        /**
         * ================================保存数据-标示线========================================
         */
        MedIndexFilterVO timeVo = new MedIndexFilterVO();
        BeanUtils.copyProperties(filterVO,timeVo);
        timeTrans(timeVo);
        Set<String> rescueCode = new CopyOnWriteArraySet<>();
        Set<String> bloodCode = new CopyOnWriteArraySet<>();
        //指标二十七
        //指标14--恶性肿瘤放射治疗记录符合率
        Set<String> tumorRadiationSet = tumorRadiationMethods(adviceNum, medIndexResult);

        //指标15 手术记录
        medIndexResult.setOperationCompleteResult(Double.parseDouble(operationFeeNum+""));

        //指标16--植入物相关记录符合率
        Set<String> implantsCode = implantsMethods(filterVO, medIndexResult);

        //指标17  临床用血相关记录符合率
        bloodMethods(doctorAdviceMedManageParams, filterVO,medIndexResult,bloodCode);

        //指标18 医师查房记录 查询直接返回 -code需要加载

        //指标19 抢救记录
        rescueMethods(doctorAdviceMedManageParams,medIndexResult,rescueCode);

        //指标20 出院患者病历2日归档
        getfileSecAmounts(filterVO,medIndexResult);

        //指标21 出院患者病历归档完整率
        fileCompleteMethods(filterVO,medIndexResult);

        //指标22 不合理复制病历发生率 查询直接返回

        //指标23  知情同意书规范签署率 //分子通过规则
        // contentSignMethods(contentFile,medIndexResult);

        //指标24 甲级率 查询直接返回


        medIndexResult.setHospitalId(filterVO.getHospitalId());
        medIndexResult.setStartDate(filterVO.getStartDate());
        medIndexResult.setEndDate(filterVO.getEndDate());
        medIndexResult.setGmtCreate(new Date());
        medIndexResultFacade.save(medIndexResult);
        Integer id = medIndexResult.getId();
        IndexRelevance medIndexRelevance = new IndexRelevance();
        medIndexRelevance.setIndexResultId(id);
        medIndexRelevance.setHospitalId(filterVO.getHospitalId());
        medIndexRelevance.setStartDate(filterVO.getStartDate());
        medIndexRelevance.setEndDate(filterVO.getEndDate());
        medIndexRelevance.setGmtCreate(new Date());
        if(outHospitalCode.size()>0&&outHospitalCode!=null) {
            medIndexRelevance.setRelevanceType(4);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
            medIndexRelevance.setRelevanceType(6);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
            medIndexRelevance.setRelevanceType(7);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(operationCode.size()>0&&operationCode!=null) {
            medIndexRelevance.setRelevanceType(5);
            for (String Code : operationCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(ctList!=null&&ctList.size()>0) {
            medIndexRelevance.setRelevanceType(8);
            for (String CTCode : ctList) {
                medIndexRelevance.setBehospitalCode(CTCode);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(mrList!=null&&mrList.size()>0) {
            medIndexRelevance.setRelevanceType(9);
            for (String MrCode : mrList) {
                medIndexRelevance.setBehospitalCode(MrCode);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(operationPathologyCode.size()>0&&operationPathologyCode!=null) {
            medIndexRelevance.setRelevanceType(10);
            for (String Code : operationPathologyCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(bacterialCultureList!=null&&bacterialCultureList.size()>0) {
            medIndexRelevance.setRelevanceType(11);
            for (String Code : bacterialCultureList) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(antibiosisCode.size()>0&&antibiosisCode!=null) {
            medIndexRelevance.setRelevanceType(12);
            for (String Code : antibiosisCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(chemotherapyMalignantTumorsList!=null&&chemotherapyMalignantTumorsList.size()>0) {
            medIndexRelevance.setRelevanceType(13);
            for (String Code : chemotherapyMalignantTumorsList) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(tumorRadiationSet.size()>0 && tumorRadiationSet != null) {
            medIndexRelevance.setRelevanceType(14);
            for (String Code : tumorRadiationSet) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(operationCode.size() > 0 && operationCode != null) {
            medIndexRelevance.setRelevanceType(15);
            for (String Code : operationCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(implantsCode.size() > 0 && implantsCode != null) {
            medIndexRelevance.setRelevanceType(16);
            for (String Code : implantsCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(bloodCode.size() > 0 && bloodCode != null) {
            medIndexRelevance.setRelevanceType(17);
            for (String Code : bloodCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(outHospitalCode.size() > 0 && outHospitalCode != null) {
            medIndexRelevance.setRelevanceType(18);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(rescueCode.size() > 0 && rescueCode != null) {
            medIndexRelevance.setRelevanceType(19);
            for (String Code : rescueCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(outHospitalCode.size() > 0 && outHospitalCode != null) {
            medIndexRelevance.setRelevanceType(22);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
        if(outHospitalCode.size() > 0 && outHospitalCode != null) {
            medIndexRelevance.setRelevanceType(23);
            for (String Code : outHospitalCode) {
                medIndexRelevance.setBehospitalCode(Code);
                medIndexRelevanceFacade.save(medIndexRelevance);
            }
        }
    }
    /**
     * 两数据相除得到百分比
     * @param i
     * @param y
     * @return
     */
    public String getResult(double i , Double y){
        DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(2);
        df.setMinimumFractionDigits(2);
        if(String.valueOf(y).equals("0.0")){
            return "0.00%";
        }
        String k= df.format((i * 100.00) / y) + "%";
        return k;
    }
    /**
     * 获取无病案首页病人code
     * @param filterVO
     * @return
     */
    public List<String>  adviceVerify(FilterVO filterVO){
        List<Map<String, String>> medicalRecords = behospitalInfoFacade.getBaseMapper().getMedicalRecords(filterVO);
        List<String> behospitalCodeList = new CopyOnWriteArrayList<>();
        for (Map<String, String> medicalRecord : medicalRecords) {
            String behospitalCode = medicalRecord.get("behospital_code");
            behospitalCodeList.add(behospitalCode);
        }
        return behospitalCodeList;
    }
    /**
     * 医嘱二级条件判断 或且
     * @param name
     * @param arg
     * @return
     */
    public Boolean  secondLevelCondition(String name,String[][] arg){
        for (int i = 0; i < arg.length; i++) {
            String[] strings = arg[i];
            int z=0;
            for (int y = 0; y < strings.length; y++) {
                if(name.contains(strings[y])){
                    z++;
                }
            }
            if(z==strings.length){
                return  false;
            }else {
                continue;
            }
        }
        return true;
    }
    /**
     * 一级条件判断 或且
     * @param name
     * @param arg
     * @return
     */
    public Boolean  stairCondition(String name,String[][] arg){
        for (int i = 0; i < arg.length; i++) {
            String[] strings = arg[i];
            int z=0;
            for (int y = 0; y < strings.length; y++) {
                if(name.contains(strings[y])){
                    z++;
                }
            }
            if(z==strings.length){
                return true;
            }else {
                continue;
            }
        }
        return false;
    }
    /**
     * 遍历医嘱信息存入相关数据
     */
    public Map<String,Set<String>> getAdviceNum(FilterVO filterVO, List<MedManageParamsDTO> doctorAdviceMedManageParamsList) {
        CopyOnWriteArrayList<MedManageParamsDTO> doctorAdviceMedManageParams = new CopyOnWriteArrayList<>(doctorAdviceMedManageParamsList);
        String indexName = filterVO.getIndexName();
        Map mapCode = new HashMap<String,List<String>>();
        Set<String> CTBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> operationPathologyBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> MRBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> operationBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> pathologyBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> bacterialCultureCode = new CopyOnWriteArraySet<>();
        Set<String> antibiosisBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> chemotherapyMalignantTumorsBehospitalCode = new CopyOnWriteArraySet<>();
        Set<String> tumorRadiationBehospitalCode = new CopyOnWriteArraySet<>();
        //无首页code
        List<String> behospitalCodeList = adviceVerify(filterVO);
        //首页为肿瘤code
        List<String> malignancyCodeList = getMalignancyCode(filterVO);
        ForkJoinPool myPool = new ForkJoinPool(6);
        try {
            myPool.submit(() ->
                    doctorAdviceMedManageParams.parallelStream().forEach(adviceMedManageParam->{
                        String daItemName = adviceMedManageParam.getDaItemName();
                        String behospitalCode = adviceMedManageParam.getBehospitalCode();
                        String doctorAdviceType = adviceMedManageParam.getDoctorAdviceType();
                        String daStatus = adviceMedManageParam.getDaStatus();
                        if(StringUtil.isEmpty(daStatus) || (!daStatus.equals(QualityContent.CANCELLATION_ORDER) && StringUtil.isNotEmpty(daStatus))){
                            //得到与手术相关
                            if (StringUtil.isNotEmpty(daItemName)) {
                                if (QualityContent.NORecord_NEED_MEDICAL_ADVICE.contains(indexName)) {
                                    //当code存在于无首页code集合中
                                    if (behospitalCodeList.contains(behospitalCode)) {
                                        //code也不存在于以保存的code集合
                                        //手术
                                        if (indexName.equals(QualityContent.SSJL_24H_WCL) || indexName.equals(QualityContent.BL_JCJL_FHL) || indexName.equals(QualityContent.QB)) {
                                            if (!operationBehospitalCode.contains(behospitalCode)) {
                                                String[][] operationList = QualityContent.OPERATION_LIST;
                                                Boolean stair = stairCondition(daItemName, operationList);
                                                if (stair) {
                                                    String[][] operationArg = QualityContent.OPERATION_ARG;
                                                    Boolean levelCondition = secondLevelCondition(daItemName, operationArg);
                                                    if (levelCondition) {
                                                        operationBehospitalCode.add(behospitalCode);
                                                    }
                                                }
                                            }
                                        }
                                        //手术病理
                                        if (indexName.equals(QualityContent.BL_JCJL_FHL) || indexName.equals(QualityContent.QB)) {
                                            if (!pathologyBehospitalCode.contains(behospitalCode)) {
                                                Boolean condition = stairCondition(daItemName, QualityContent.BL_LIST);
                                                if (condition) {
                                                    pathologyBehospitalCode.add(behospitalCode);
                                                }
                                            }
                                        }
                                        //取临时和长期
                                        if (doctorAdviceType.equals(QualityContent.STAT_ORDER) || doctorAdviceType.equals(QualityContent.STANDING_ORDER)) {
                                            //抗菌药物
                                            if (indexName.equals(QualityContent.KJYW_SY_FHL) || indexName.equals(QualityContent.QB)) {
                                                if (!antibiosisBehospitalCode.contains(behospitalCode)) {
                                                    List<String> antimicrobialDrugList = QualityContent.Antimicrobial_Drug_List;
                                                    for (String antimicrobialDrug : antimicrobialDrugList) {
                                                        if (daItemName.contains(antimicrobialDrug)) {
                                                            //包含抗菌药物
                                                            antibiosisBehospitalCode.add(behospitalCode);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                //得到为临时医嘱
                                if (StringUtil.isNotEmpty(doctorAdviceType)) {
                                    if (doctorAdviceType.equals(QualityContent.STAT_ORDER)) {
                                        if (indexName.equals(QualityContent.CT_JCJL_FHL) || indexName.equals(QualityContent.QB)) {
                                            if (!CTBehospitalCode.contains(behospitalCode)) {
                                                //筛选与CT相关
                                                //一级条件
                                                String[][] ctList = QualityContent.CT_LIST;
                                                Boolean stair = stairCondition(daItemName, ctList);
                                                if (stair) {
                                                    String[][] ctRegulation = QualityContent.CT_REGULATION;
                                                    Boolean levelCondition = secondLevelCondition(daItemName, ctRegulation);
                                                    if (levelCondition) {
                                                        CTBehospitalCode.add(behospitalCode);
                                                    }
                                                }
                                            }
                                        }
                                        //与MR相关
                                        if (indexName.equals(QualityContent.MR_JCJL_FHL) || indexName.equals(QualityContent.QB)) {
                                            if (!MRBehospitalCode.contains(behospitalCode)) {
                                                String[][] mrList = QualityContent.MR_LIST;
                                                Boolean stair = stairCondition(daItemName, mrList);
                                                if (stair) {
                                                    String[][] mrRegulation = QualityContent.MR_REGULATION;
                                                    Boolean levelCondition = secondLevelCondition(daItemName, mrRegulation);
                                                    if (levelCondition) {
                                                        MRBehospitalCode.add(behospitalCode);
                                                    }
                                                }
                                            }
                                        }
                                        //细菌培养
                                        if (indexName.equals(QualityContent.XJPY_JCJL_FHL) || indexName.equals(QualityContent.QB)) {
                                            if(!bacterialCultureCode.contains(behospitalCode)){
                                                if(daItemName.contains(QualityContent.BACTERIAL_CULTURE)){
                                                    bacterialCultureCode.add(behospitalCode);
                                                }
                                            }
                                        }
                                    }
                                }
                                //获取为长期医嘱或临时医嘱的医嘱
                                if (StringUtil.isNotEmpty(doctorAdviceType)) {
                                    if (indexName.equals(QualityContent.EXZL_HXZL_FHL) || indexName.equals(QualityContent.QB)) {
                                        if (ListUtil.isNotEmpty(malignancyCodeList)) {
                                            if (malignancyCodeList.contains(behospitalCode)) {
                                                if (!chemotherapyMalignantTumorsBehospitalCode.contains(behospitalCode)) {
                                                    if (doctorAdviceType.equals(QualityContent.STAT_ORDER) || doctorAdviceType.equals(QualityContent.STANDING_ORDER)) {
                                                        List<String> chemotherapyDrugList = QualityContent.Chemotherapy_Drug_List;
                                                        for (String chemotherapyDrug : chemotherapyDrugList) {
                                                            if (daItemName.contains(chemotherapyDrug)) {
                                                                //医嘱含有化疗药物
                                                                chemotherapyMalignantTumorsBehospitalCode.add(behospitalCode);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    if (indexName.equals(QualityContent.EXZL_FSZL_FHL) || indexName.equals(QualityContent.QB)) {
                                        if (ListUtil.isNotEmpty(malignancyCodeList)) {
                                            if (malignancyCodeList.contains(behospitalCode)) {
                                                if (!tumorRadiationBehospitalCode.contains(behospitalCode)) {
                                                    if (doctorAdviceType.equals(QualityContent.STAT_ORDER) || doctorAdviceType.equals(QualityContent.STANDING_ORDER)) {
                                                        List<String> tumorRadiationDrugList = QualityContent.radiotherapyDrugList;
                                                        for (String tumorRadiationDrug : tumorRadiationDrugList) {
                                                            if (daItemName.contains(tumorRadiationDrug)) {
                                                                //医嘱含有放射治疗药物
                                                                tumorRadiationBehospitalCode.add(behospitalCode);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    })).get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }finally {
            myPool.shutdown();

            if(CTBehospitalCode !=null && CTBehospitalCode.size()>0){
                mapCode.put("CT",CTBehospitalCode);
            }else {
                mapCode.put("CT",null);
            }
            if(MRBehospitalCode!=null && MRBehospitalCode.size()>0){
                mapCode.put("MR",MRBehospitalCode);
            }else {
                mapCode.put("MR",null);
            }
            if(operationBehospitalCode!=null && operationBehospitalCode.size()>0){
                mapCode.put("operation",operationBehospitalCode);
            }else {
                mapCode.put("operation",null);
            }
            if((operationBehospitalCode!=null)&&(pathologyBehospitalCode!=null&&pathologyBehospitalCode.size()>0)){
                for (String operation : operationBehospitalCode) {
                    if(pathologyBehospitalCode.contains(operation)){
                        operationPathologyBehospitalCode.add(operation);
                    }
                }
                mapCode.put("operationPathology",operationPathologyBehospitalCode);
            }else {
                mapCode.put("operationPathology",null);
            }
            if(bacterialCultureCode!=null && bacterialCultureCode.size()>0){
                mapCode.put("bacterialCulture",bacterialCultureCode);

            }else {
                mapCode.put("bacterialCulture",null);
            }
            if(antibiosisBehospitalCode!=null && antibiosisBehospitalCode.size()>0){
                mapCode.put("antibiosis",antibiosisBehospitalCode);
            }else {
                mapCode.put("antibiosis",null);
            }
            if(chemotherapyMalignantTumorsBehospitalCode!=null && chemotherapyMalignantTumorsBehospitalCode.size()>0){
                mapCode.put("chemotherapyMalignantTumors",chemotherapyMalignantTumorsBehospitalCode);
            }else {
                mapCode.put("chemotherapyMalignantTumors",null);
            }
            if(tumorRadiationBehospitalCode!=null && tumorRadiationBehospitalCode.size()>0){
                mapCode.put("tumorRadiationSecNum",tumorRadiationBehospitalCode);
            }else {
                mapCode.put("tumorRadiationSecNum",null);
            }
            return mapCode;
        }
    }
    /**
     * 获取首页为恶性肿瘤病历code
     */
    public List<String> getMalignancyCode(FilterVO filterVO){
        List<Map<String,String>> malignancyCodeList = behospitalInfoFacade.getBaseMapper().malignancy(filterVO);
        ArrayList<String> arrayList = new ArrayList<>();
        if(ListUtil.isNotEmpty(malignancyCodeList)) {
            for (Map<String, String> map : malignancyCodeList) {
                String behospitalCode = map.get("behospital_code");
                arrayList.add(behospitalCode);
            }
        }
        return arrayList;
    }


    //指标十四 恶性肿瘤放射治疗
    private List<Map<String, String>> tumorRadiationMethod(Map<String, Double> medManageParams,List<Map<String, String>> retList, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        //接受肿瘤放射治疗的质控病历总数
        Double count = medManageParams.get("tumorRadiationResult");
        String num = null;
        if( null == count){
            count = 0d;
        }
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            double tumorRadiationDo = currencyCal(count, triggeringRulesMap.get("tumorRadiationNum"));//triggeringRules(filterVO)
            num = String.valueOf(tumorRadiationDo);
            if(tumorRadiationDo<0d){
                num = null;
            }
        };

        firMap.put("name","恶性肿瘤放射治疗记录符合率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        firMap.put("num",String.valueOf(count));
        retList.add(firMap);
        return retList;
    }

    //指标十六 植入物相关记录符合率
    private List<Map<String, String>> implantsMethod(Map<String, Double> medManageParams,List<Map<String, String>> retList, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        //使用植入物的质控病历总数
        Double count = medManageParams.get("implantsResult");
        if( null == count){
            count = 0d;
        }
        String num = null;
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            double implantsDo = currencyCal(count, triggeringRulesMap.get("implantsNum"));//triggeringRules(filterVO)
            num = String.valueOf(implantsDo);
            if(implantsDo<0d){
                num = null;
            }
        };

        firMap.put("name","植入物相关记录符合率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        firMap.put("num",String.valueOf(count));
        retList.add(firMap);
        return retList;
    }

    //指标十八 抢救记录
    private  List<Map<String, String>> rescueMethod( Map<String, Double> medManageParams, List<Map<String, String>> retList,Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        //接受抢救的质控病历总数
        Double count = medManageParams.get("rescueResult");
        if( null == count){
            count = 0d;
        }
        String num = null;
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            double rescueDo = currencyCal(count, triggeringRulesMap.get("rescueNum"));
            num = String.valueOf(rescueDo);
            if(rescueDo<0d){
                num = null;
            }
        };

        firMap.put("name","患者抢救记录及时完成率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        firMap.put("num",String.valueOf(count));
        retList.add(firMap);
        return retList;
    };

    //指标十七 医师查房记录
    private List<Map<String, String>> WardRoundMethod( List<Map<String, String>> retList, Map<String, Object> baseIndex, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        String num = null;
        Object forWorkNum = baseIndex.get("forWorkNum");
        if( triggeringRulesMap.containsKey("WardRoundNum") && baseIndex.containsKey("forWorkNum") && !baseIndex.get("forWorkNum").toString().equals("0")){
            double wardRoundDo = currencyCal(baseIndex.get("forWorkNum"), triggeringRulesMap.get("WardRoundNum"));
            num = String.valueOf(wardRoundDo);
            if(wardRoundDo<0d){
                num = null;
            }
        };
        firMap.put("name","医师查房记录完成率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        if(null == forWorkNum){
            firMap.put("num","0");
        }else{
            firMap.put("num",baseIndex.get("forWorkNum").toString());
        }
        retList.add(firMap);
        return retList;
    }


    //指标二十二 不合理复制病历发生率
    private List<Map<String, String>> unreasonedCopyMethod( List<Map<String, String>> retList, Map<String, Object> baseIndex, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        String num = null;
        Object forWorkNum = baseIndex.get("forWorkNum");
        if( triggeringRulesMap.containsKey("unreasonedCopyNum") && baseIndex.containsKey("forWorkNum") && !baseIndex.get("forWorkNum").toString().equals("0")){
            double wardRoundDo = currencyCal(baseIndex.get("forWorkNum"), triggeringRulesMap.get("unreasonedCopyNum"));
            num = String.valueOf(wardRoundDo);
            if(wardRoundDo<0d){
                num = null;
            }
        };
        firMap.put("name","不合理复制病历发生率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        if(null == forWorkNum){
            firMap.put("num","0");
        }else{
            firMap.put("num",baseIndex.get("forWorkNum").toString());
        }
        retList.add(firMap);
        return retList;
    };


    //指标二十三 知情同意书规范签署率
    private List<Map<String, String>> contentSignMethod(List<Map<String, String>> retList, Map<String, Object> baseIndex, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        Object forWorkNum = baseIndex.get("forWorkNum");
        String num = null;
        if( triggeringRulesMap.containsKey("InformedNum") && baseIndex.containsKey("forWorkNum") && !baseIndex.get("forWorkNum").toString().equals("0")){
            double InformedDo = currencyCal(baseIndex.get("forWorkNum"), triggeringRulesMap.get("InformedNum"));
            num = String.valueOf(InformedDo);
            if(InformedDo<0d){
                num = null;
            }
        }
        firMap.put("name","知情同意书规范签署率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        if(null == forWorkNum){
            firMap.put("num","0");
        }else{
            firMap.put("num",baseIndex.get("forWorkNum").toString());
        }

        retList.add(firMap);
        return retList;
    }

    //指标十四 手术记录
    private  List<Map<String, String>> operationMethod(Map<String, Double> medManageParams, List<Map<String, String>>retList, Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        String operationFeeNum = medManageParams.get("operationFeeNum")+"";
        if(StringUtils.isEmpty(operationFeeNum)){
            operationFeeNum = "0";
        }
        String num = null;
        if( triggeringRulesMap.containsKey("operationRecordNum") && StringUtils.isNotEmpty(operationFeeNum) && ! operationFeeNum.equals("0")){
            double operationRecordDo = currencyCal(operationFeeNum, triggeringRulesMap.get("operationRecordNum"));
            num = String.valueOf(operationRecordDo);
            if(operationRecordDo<0d){
                num = null;
            }
        };
        firMap.put("name","手术相关记录完成率");
        if("0.0".equals(num)){
            num = "0.00";
        }
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        firMap.put("num",operationFeeNum);

        retList.add(firMap);
        return retList;
    }

    //指标十六  临床用血相关记录符合率
    private  List<Map<String, String>> bloodMethod( List<Map<String, String>>retList,Map<String, Double> bloodRes,Map<String, Long> triggeringRulesMap){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        Double count = bloodRes.get("bloodResult");
        if( null == count){
            count = 0d;
        }
        String num = null;
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            //指标十七
            double bloodDo = currencyCal(count, triggeringRulesMap.get("bloodNum"));
            num = String.valueOf(bloodDo);
            if(bloodDo<0d){
                num = null;
            }
        }

        firMap.put("name","临床用血相关记录符合率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        firMap.put("num",String.valueOf(count));
        retList.add(firMap);
        return retList;
    }

    private double currencyCal(Object objFir,Object objSec){
        DecimalFormat df = new DecimalFormat("#0.00");
        double firDouble = Double.parseDouble(objFir.toString()) - Double.parseDouble(objSec.toString());
        double thrWorkNum = Double.parseDouble(objFir.toString());
        double retData = 0d;
        if(thrWorkNum != 0){
            retData = Double.parseDouble(df.format(firDouble * 100 / thrWorkNum));
        }
        return retData;
    };

    //指标十九 出院患者病历2日归档
    private List<Map<String, String>> getfileSecAmount(Map<String, Double> medManageParams, List<Map<String, String>>retList, Map<String, Object> baseIndex){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        DecimalFormat df = new DecimalFormat("#0.00");
        Double count = medManageParams.get("secFileResult");
        Object forWorkNum = baseIndex.get("forWorkNum");
        if( null == count){
            count = 0d;
        }
        String num = null;
        if(StringUtils.isNotEmpty( baseIndex.get("forWorkNum").toString()) && ! baseIndex.get("forWorkNum").toString().equals("0") ){
            double retNum =count* 100/Double.parseDouble(baseIndex.get("forWorkNum").toString());
            double firGradeStr = Double.parseDouble(df.format(retNum));
            num = String.valueOf(firGradeStr);

        }

        firMap.put("name","出院患者病历2日归档率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        if(null == forWorkNum){
            firMap.put("num","0");
        }else{
            firMap.put("num",baseIndex.get("forWorkNum").toString());
        }
        retList.add(firMap);
        return retList;
    };

    //指标21 --出院患者病历归档完整率
    private  List<Map<String, String>> fileCompleteMethod(Map<String, Double> medManageParams, List<Map<String, String>>retList, Map<String, Object> baseIndex){
        LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
        Double count = medManageParams.get("fileCompleteResult");
        Object forWorkNum = baseIndex.get("forWorkNum");
        if( null == count){
            count = 0d;
        }
        String num = null;
        if(StringUtils.isNotEmpty( baseIndex.get("forWorkNum").toString()) && ! baseIndex.get("forWorkNum").toString().equals("0") ){
            double retNum = currencyCal( baseIndex.get("forWorkNum").toString(),count);
            num = String.valueOf(retNum);
        }

        firMap.put("name","出院患者病历归档完整率");
        firMap.put("notCopiedStr",StringUtils.isEmpty(num) ? "0.00%":(num+"%"));
        if(null == forWorkNum){
            firMap.put("num","0");
        }else{
            firMap.put("num",baseIndex.get("forWorkNum").toString());
        }
        retList.add(firMap);
        return retList;
    }


    private List<Map<String, String>> ManageIndexMethod(String indexName,List<Map<String, String>> retList, Map<String, Object> baseIndex, MedIndexFilterVO timeVo){
        QueryWrapper<ManagementInfo> query = new QueryWrapper<>();
        query.eq("hospital_id", timeVo.getHospitalId())
                .eq("is_deleted", IsDeleteEnum.N)
                .isNotNull("start_date")
                .isNotNull("end_date")
                .ge("start_date",timeVo.getLastStartDate())
                .le("end_date",timeVo.getLastEndDate())
                .select("position_id","work_person");
        List<ManagementInfo> list = managementInfoFacade.list(query);


        //单位时间同期住院病案管理人员实际工作总月数
        if(indexName.equals(QualityContent.ZYBA_CYHZ)||indexName.equals(QualityContent.QB)) {
            Optional<Double> secOp = list.stream().filter(obj->obj.getPositionId().equals("1")).map(ManagementInfo::getWorkPerson).reduce(Double::sum);
            Double secMouth = 0d;
            if(secOp.isPresent()){
                secMouth = secOp.get();
            }
            if(secMouth == 0d){
                secMouth = 5d;
            }
            //指标一
            String num1 = "0";
            if(secMouth != 0d && secMouth != null && StringUtils.isNotEmpty( baseIndex.get("firWorkNum").toString())){
                num1 = String.valueOf(Math.round(Double.parseDouble(baseIndex.get("firWorkNum").toString()) / secMouth));
            }
            LinkedHashMap<String, String> firMap = new LinkedHashMap<>();
            firMap.put("name","住院病案管理人员月均负担出院患者病历数");
            firMap.put("notCopiedStr",num1);
            retList.add(firMap);
        }

        //单位时间同期门诊病案管理人员实际工作总月数
        if(indexName.equals(QualityContent.MZBA_CYHZ)||indexName.equals(QualityContent.QB)) {
            Optional<Double> firOp = list.stream().filter(obj -> obj.getPositionId().equals("3")).map(ManagementInfo::getWorkPerson).reduce(Double::sum);
            Double firMouth = 0d;
            if(firOp.isPresent()){
                firMouth = firOp.get();
            }
            if(firMouth == 0d){
                firMouth = 5d;
            }
            String num3 = "0";
            if(firMouth != 0d && firMouth != null  && StringUtils.isNotEmpty( baseIndex.get("secWorkNum").toString())){
                //指标二
                num3 = String.valueOf(Math.round(Double.parseDouble(baseIndex.get("secWorkNum").toString())/firMouth));
            }
            LinkedHashMap<String, String> thrMap = new LinkedHashMap<>();
            thrMap.put("name","门诊病案管理人员月均负担门诊患者病历数");
            thrMap.put("notCopiedStr",num3);
            retList.add(thrMap);
        }

        //单位时间同期病案编码管理人员实际工作总月数
        if(indexName.equals(QualityContent.BABM_CYHZ)||indexName.equals(QualityContent.QB)) {
            Optional<Double> thrOp = list.stream().filter(obj->obj.getPositionId().equals("2")).map(ManagementInfo::getWorkPerson).reduce(Double::sum);
            Double thrMouth = 0d;
            if(thrOp.isPresent()){
                thrMouth = thrOp.get();
            }
            if(thrMouth == 0d){
                thrMouth = 5d;
            }
            //指标三
            String num2 = "0";
            if(thrMouth != 0d && thrMouth != null && StringUtils.isNotEmpty( baseIndex.get("firWorkNum").toString())){
                num2 = String.valueOf(Math.round(Double.parseDouble(baseIndex.get("firWorkNum").toString())/thrMouth));

            }
            LinkedHashMap<String, String> secMap = new LinkedHashMap<>();
            secMap.put("name","病案编码人员月均负担出院患者病历数");
            secMap.put("notCopiedStr",num2);
            retList.add(secMap);
        }
        return retList;
    };


    private List<Map<String, String>> firMethod(List<Map<String, String>> retList, Map<String, Object> baseIndex){
        DecimalFormat df = new DecimalFormat("0.00");
        //指标二十七 -甲级病历率
        LinkedHashMap<String, String> fouMap = new LinkedHashMap<>();
        Double firRecord = 0d;
        String num4= null;
        if(StringUtils.isNotEmpty( baseIndex.get("forWorkNum").toString())){
            firRecord =Double.parseDouble(baseIndex.get("forWorkNum").toString());
        }
        if( firRecord != 0d && StringUtils.isNotEmpty( baseIndex.get("sixWorkNum").toString()) ){
            double retNum = Double.parseDouble(baseIndex.get("sixWorkNum").toString()) * 100 / firRecord;
            double firGradeStr = Double.parseDouble(df.format(retNum));
            num4 = String.valueOf(firGradeStr);
        }
        fouMap.put("name","甲级病历率");
        fouMap.put("notCopiedStr",StringUtils.isEmpty(num4) ? "0.00%":(num4+"%"));
        fouMap.put("num",String.valueOf(firRecord));
        retList.add(fouMap);
        return retList;
    };


    /**
     * ==============================后台维护===============================
     */
    private  Set<String> tumorRadiationMethods( Map<String, Set<String>> adviceNum,IndexResult medIndexResult){
        Set<String> tumorRadiationSecNum = new CopyOnWriteArraySet<>();
        if(adviceNum.containsKey("tumorRadiationSecNum")&& adviceNum.get("tumorRadiationSecNum") != null && adviceNum.get("tumorRadiationSecNum").size()>0){
            tumorRadiationSecNum = adviceNum.get("tumorRadiationSecNum");
            double count = tumorRadiationSecNum.size();
            if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
                medIndexResult.setTumorRadiationResult(count);
            }
        }
        return tumorRadiationSecNum;
    }
    //指标19 抢救记录
    private  void rescueMethods(List<MedManageParamsDTO> medManageParams,IndexResult medIndexResult, Set<String> rescueCode){
        List<MedManageParamsDTO> manageList= new ArrayList<>();
        List<String> rescueDrugList = QualityContent.RESCUEDRUGLIST;
        for (int i = 0; i < medManageParams.size(); i++) {
            String daItemName = medManageParams.get(i).getDaItemName();
            String doctorAdviceType = medManageParams.get(i).getDoctorAdviceType();
            String da_status = medManageParams.get(i).getDaStatus();
            for (String str : rescueDrugList) {
                if(StringUtils.isNotEmpty(daItemName) && daItemName.contains(str) && StringUtils.isNotEmpty(doctorAdviceType) && doctorAdviceType.equals(QualityContent.STAT_ORDER) && (StringUtils.isEmpty(da_status) || (StringUtils.isNotEmpty(da_status) && ! da_status.equals(QualityContent.CANCELLATION_ORDER)))){
                    manageList.add(medManageParams.get(i));
                    rescueCode.add(medManageParams.get(i).getBehospitalCode());
                }
            }
        }
        //接受抢救的质控病历总数
        double count = manageList.stream().map(MedManageParamsDTO::getBehospitalCode).distinct().count();
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            medIndexResult.setRescueResult(count);
        }
    }

    //指标17  临床用血相关记录符合率
    private void bloodMethods(List<MedManageParamsDTO> doctorAdviceList,FilterVO filterVO,IndexResult medIndexResult, Set<String> bloodCode){
        filterVO.setFlagStr("1");
        List<MedManageParamsDTO> medManageParams = behospitalInfoFacade.getBaseMapper().getMedManageParams(filterVO);
        Set<String> manageList= new CopyOnWriteArraySet<>();
        List<String> blood_list = QualityContent.BLOODLIST;
        //lis项目存在用血住院病历号
        List<String> beCode = medManageParams.stream()
                .filter(obj -> StringUtils.isNotEmpty(obj.getRepName()) && blood_list.contains(obj.getRepName()))
                .map(MedManageParamsDTO::getBehospitalCode).distinct()
                .collect(Collectors.toList());

        //advice体现lis项目存在用血的住院病历号
        doctorAdviceList.parallelStream()
                .filter(obj -> beCode.contains(obj.getBehospitalCode()) && StringUtils.isNotEmpty(obj.getDoctorAdviceType()) && obj.getDoctorAdviceType()
                        .equals(QualityContent.STAT_ORDER) && blood_list.contains(obj.getDaItemName()) &&
                        (StringUtils.isEmpty(obj.getDaStatus()) || (StringUtils.isNotEmpty(obj.getDaStatus()) && ! obj.getDaStatus()
                                .equals(QualityContent.CANCELLATION_ORDER))))
                .distinct()
                .map(obj -> {manageList.add(obj.getBehospitalCode());return bloodCode.add(obj.getBehospitalCode());})
                .collect(Collectors.toList());

        //临床用血的质控病历总数
        double count = manageList.size();
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            //指标十七
            medIndexResult.setBloodResult(count);
        }
    }


    //指标20 出院患者病历2日归档
    private void getfileSecAmounts(FilterVO filterVO,IndexResult medIndexResult){
        QueryWrapper<BehospitalInfo> query = new QueryWrapper<>();
        DecimalFormat df = new DecimalFormat("#0.00");
        query.eq("hospital_id", filterVO.getHospitalId())
                .eq("is_deleted", IsDeleteEnum.N)
                .ne("qc_type_id", 0)
                .eq("is_placefile", 1)
                // .eq("is_placefile", filterVO.getIsPlacefile())
                .isNotNull("leave_hospital_date")
                .ge("leave_hospital_date",filterVO.getStartDate())
                .le("leave_hospital_date",filterVO.getEndDate());
        List<BehospitalInfo> list = behospitalInfoFacade.list(query);
        Iterator<BehospitalInfo> iterator = list.iterator();
        while (iterator.hasNext()){
            BehospitalInfo beh = iterator.next();
            Boolean flag = getTrueSecFile(beh.getLeaveHospitalDate(), beh.getPlacefileDate(), TimeContent.HolidaysTimeList);
            //时间截止则删除元素
            if(!flag){
                iterator.remove();
            }
        }
        double count = list.stream().distinct().map(BehospitalInfo::getBedCode).count();
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            medIndexResult.setSecFileResult(count);
        }

    }

    private Boolean getTrueSecFile(Date date1,Date date2, List<String> holidaysTimeList){
        if(date1.after(date2)){
            return false;
        }
        Date date = date1;
        Date dateSec = date2;
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat simpleDateFormatRe = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal = null;
        int count = 0;
        try {
            date1 = simpleDateFormatRe.parse(simpleDateFormatRe.format(date1));
            date2 = simpleDateFormatRe.parse(simpleDateFormatRe.format(date2));
            cal = Calendar.getInstance();
            cal.setTime(date);
            count = 0;
            for (String str : holidaysTimeList) {
                Date date3 = simpleDateFormatRe.parse(str);
                if(date3.after(date1) && date3.before(date2)){
                    count++;
                }
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        cal.add(Calendar.DAY_OF_YEAR,count+2);
        String endTime = simpleDateFormat.format(cal.getTime());
        LocalDateTime ldt = LocalDateTime.parse(endTime,dateTimeFormatter);
        ZoneId zone = ZoneId.systemDefault();
        Instant instant = ldt.atZone(zone).toInstant();
        Date endDate = Date.from(instant);
        if(endDate.after(dateSec)){
            return true;
        }
        return false;
    }


    private void timeTrans( MedIndexFilterVO  filterVO){
        SimpleDateFormat myFormatter = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal1 = Calendar.getInstance();
        Calendar cal2 = Calendar.getInstance();
        try {
            cal1.setTime(myFormatter.parse(filterVO.getStartDate()));
            cal2.setTime(myFormatter.parse(filterVO.getEndDate()));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        cal1.set(Calendar.DAY_OF_MONTH, 1);//当月的第一天
        cal2.set(Calendar.DAY_OF_MONTH, 1);
        cal2.set(Calendar.DATE, 1);//设为当前月的1号
        cal2.roll(Calendar.DATE, -1);
        String startTime = myFormatter.format(cal1.getTime())+" 00:00:00";
        String endTime = myFormatter.format(cal2.getTime())+" 23:59:59";
        filterVO.setLastStartDate(startTime);
        filterVO.setLastEndDate(endTime);
    };

    //指标 16--植入物相关记录符合率
    private Set<String> implantsMethods(FilterVO filterVO,IndexResult medIndexResult){
        filterVO.setFlagStr("4");
        List<MedManageParamsDTO> fileLists = behospitalInfoFacade.getBaseMapper().getMedManageParams(filterVO);
        Set<String> collect = fileLists.parallelStream().filter(obj -> {
            String content = obj.getContent();
            Map contenMap = JSON.parseObject(content, Map.class);
            return contenMap.containsKey("使用植入性材料") && StringUtils.isNotEmpty(contenMap.get("使用植入性材料").toString())
                    && "true".equals(contenMap.get("使用植入性材料").toString());
        }).map(MedManageParamsDTO::getBehospitalCode).collect(Collectors.toSet());
        double count = collect.size();
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            medIndexResult.setImplantsResult(count);
        }
        return collect;
    }

    //指标21 出院患者病历归档完整率
    private void fileCompleteMethods(FilterVO filterVO,IndexResult medIndexResult){
        filterVO.setFlagStr("3");
        List<MedManageParamsDTO> fileLists = behospitalInfoFacade.getBaseMapper().getMedManageParams(filterVO);
        double count = fileLists.stream().map(MedManageParamsDTO::getBehospitalCode).distinct().count();
        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
            medIndexResult.setFileCompleteResult(count);
        }
    }

    //指标23  知情同意书规范签署率
//    private void contentSignMethods(List<MedManageParamsDTO>fileLists,MedIndexResult medIndexResult){
//        Set<String> collect = fileLists.parallelStream().filter(obj -> {
//            String content = obj.getContent();
//            Map contenMap = JSON.parseObject(content, Map.class);
//            return contenMap.containsKey("签署") && StringUtils.isNotEmpty(contenMap.get("签署").toString());
//        }).map(MedManageParamsDTO::getBehospitalCode).collect(Collectors.toSet());
//        double count = collect.size();
//        if(StringUtils.isNotEmpty(String.valueOf(count)) &&  count != 0d){
//            medIndexResult.setRecordSignResult(count);
//        }
//    }

    public String handleTime(String startDate){
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date parse = null;
        try {
            parse = dateFormat.parse(startDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return (dateFormat.format(parse)+" 23:59:59");
    };
}