package com.lantone.report.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.EncrypDES;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.report.DataImportVO;
import com.lantone.dblayermbg.entity.report.BehospitalInfo;
import com.lantone.dblayermbg.entity.report.CrisisInfo;
import com.lantone.dblayermbg.entity.report.DoctorAdvice;
import com.lantone.dblayermbg.entity.report.HomeDiagnoseInfo;
import com.lantone.dblayermbg.entity.report.HomeOperationInfo;
import com.lantone.dblayermbg.entity.report.HomePage;
import com.lantone.dblayermbg.entity.report.MedicalRecord;
import com.lantone.dblayermbg.entity.report.MedicalRecordContent;
import com.lantone.dblayermbg.service.report.BehospitalInfoService;
import com.lantone.dblayermbg.service.report.CrisisInfoService;
import com.lantone.dblayermbg.service.report.DoctorAdviceService;
import com.lantone.dblayermbg.service.report.HomeDiagnoseInfoService;
import com.lantone.dblayermbg.service.report.HomeOperationInfoService;
import com.lantone.dblayermbg.service.report.HomePageService;
import com.lantone.dblayermbg.service.report.MedicalRecordContentService;
import com.lantone.dblayermbg.service.report.MedicalRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import java.util.List;
import java.util.stream.Collectors;


@Component
public class PublicMaintenanceFacade {
    @Value("${encrypt.enable}")
    Boolean encryptFlag;
    @Autowired
    @Qualifier("behospitalInfoServiceImpl")
    private BehospitalInfoService behospitalInfoService;
    @Autowired
    @Qualifier("crisisInfoServiceImpl")
    private CrisisInfoService crisisInfoService;
    @Qualifier("doctorAdviceServiceImpl")
    private DoctorAdviceService doctorAdviceService;
    @Autowired
    @Qualifier("medicalRecordContentServiceImpl")
    private MedicalRecordContentService medicalRecordContentService;
    @Autowired
    @Qualifier("medicalRecordServiceImpl")
    private MedicalRecordService medicalRecordService;
    @Autowired
    @Qualifier("homeOperationInfoServiceImpl")
    private HomeOperationInfoService homeOperationInfoService;
    @Autowired
    @Qualifier("homePageServiceImpl")
    private HomePageService homePageService;
    @Autowired
    @Qualifier("homeDiagnoseInfoServiceImpl")
    private HomeDiagnoseInfoService homeDiagnoseInfoService;

    /**
     * 数据导入【效率低，备用】
     *
     * @param dataImportVO
     * @return
     */
    public Boolean dataImportPrepareFacade(DataImportVO dataImportVO) {
        Long hospitalId = dataImportVO.getHospitalId();
        // 病历信息导入
        List<BehospitalInfo> behospitalInfoList = BeanUtil.listCopyTo(dataImportVO.getBehospitalInfoList(), BehospitalInfo.class);
        if (ListUtil.isNotEmpty(behospitalInfoList)) {
            behospitalInfoList.forEach(s -> {
                QueryWrapper<BehospitalInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("behospital_code", s.getBehospitalCode());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("is_deleted", IsDeleteEnum.N);
                // 姓名加密
                if (StringUtil.isNotBlank(s.getName())) {
                    s.setName(s.getName().substring(0, 1) + "**");
                }
                int count = behospitalInfoService.count(queryWrapper);
                if (count > 0) {
                    behospitalInfoService.update(s, queryWrapper);
                } else {
                    behospitalInfoService.save(s);
                }
            });
        }

        // 危急值信息
        List<CrisisInfo> medCrisisInfoList = BeanUtil.listCopyTo(dataImportVO.getMedCrisisInfoList(), CrisisInfo.class);
        if (ListUtil.isNotEmpty(medCrisisInfoList)) {
            medCrisisInfoList.forEach(s -> {
                QueryWrapper<CrisisInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("rec_id", s.getRecId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("behospital_code", s.getBehospitalCode());
                queryWrapper.eq("rec_type", s.getRecType());
                int count = crisisInfoService.count(queryWrapper);
                if (count > 0) {
                    crisisInfoService.update(s, queryWrapper);
                } else {
                    crisisInfoService.save(s);
                }
            });
        }

        // 病人医嘱
        List<DoctorAdvice> doctorAdviceList = BeanUtil.listCopyTo(dataImportVO.getDoctorAdviceList(), DoctorAdvice.class);
        if (ListUtil.isNotEmpty(doctorAdviceList)) {
            doctorAdviceList.forEach(s -> {
                QueryWrapper<DoctorAdvice> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("doctor_advice_id", s.getDoctorAdviceId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                int count = doctorAdviceService.count(queryWrapper);
                if (count > 0) {
                    doctorAdviceService.update(s, queryWrapper);
                } else {
                    doctorAdviceService.save(s);
                }
            });
        }

        // 病案首页诊断
        List<HomeDiagnoseInfo> homeDiagnoseInfoList = BeanUtil.listCopyTo(dataImportVO.getHomeDiagnoseInfoList(), HomeDiagnoseInfo.class);
        if (ListUtil.isNotEmpty(homeDiagnoseInfoList)) {
            homeDiagnoseInfoList.forEach(s -> {
                QueryWrapper<HomeDiagnoseInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("home_page_id", s.getHomePageId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("diagnose_order_no", s.getDiagnoseOrderNo());
                int count = homeDiagnoseInfoService.count(queryWrapper);
                if (count > 0) {
                    homeDiagnoseInfoService.update(s, queryWrapper);
                } else {
                    homeDiagnoseInfoService.save(s);
                }
            });
        }

        // 病案首页
        List<HomePage> homePageList = BeanUtil.listCopyTo(dataImportVO.getHomePageList(), HomePage.class);
        if (ListUtil.isNotEmpty(homePageList)) {
            homePageList.stream().forEach(s -> {
                // 姓名
                if (StringUtil.isNotBlank(s.getName())) {
                    s.setName(s.getName().substring(0, 1) + "**");
                }
                // 身份证
                if (StringUtil.isNotBlank(s.getIdentityCardNo())) {
                    s.setIdentityCardNo(s.getIdentityCardNo().substring(0, 1) + "*****************");
                }
                // 现住址电话
                if (StringUtil.isNotBlank(s.getCurPhone())) {
                    s.setCurPhone(s.getCurPhone().substring(0, 1) + "**");
                }
                // 工作单位电话
                if (StringUtil.isNotBlank(s.getWorkPhone())) {
                    s.setWorkPhone(s.getWorkPhone().substring(0, 1) + "**");
                }
                // 联系人姓名
                if (StringUtil.isNotBlank(s.getContactName())) {
                    s.setContactName(s.getContactName().substring(0, 1) + "**");
                }
                // 联系人电话
                if (StringUtil.isNotBlank(s.getContactPhone())) {
                    s.setContactPhone(s.getContactPhone().substring(0, 1) + "**");
                }

                QueryWrapper<HomePage> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("home_page_id", s.getHomePageId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("behospital_code", s.getBehospitalCode());
                queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                int count = homePageService.count(queryWrapper);
                if (count > 0) {
                    homePageService.update(s, queryWrapper);
                } else {
                    homePageService.save(s);
                }
            });
        }

        // 病案首页手术信息
        List<HomeOperationInfo> homeOperationInfoList = BeanUtil.listCopyTo(dataImportVO.getHomeOperationInfoList(), HomeOperationInfo.class);
        if (ListUtil.isNotEmpty(homeOperationInfoList)) {
            homeOperationInfoList.forEach(s -> {
                if(s.getHomePageId()!=null && !"".equals(s.getOperationOrderNo())) {
                    QueryWrapper<HomeOperationInfo> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("home_page_id", s.getHomePageId());
                    queryWrapper.eq("hospital_id", s.getHospitalId());
                    queryWrapper.eq("operation_order_no", s.getOperationOrderNo());
                    int count = homeOperationInfoService.count(queryWrapper);
                    if (count > 0) {
                        homeOperationInfoService.update(s, queryWrapper);
                    } else {
                        homeOperationInfoService.save(s);
                    }
                }
            });
        }

        // 文书信息
        List<MedicalRecord> medicalRecordList = BeanUtil.listCopyTo(dataImportVO.getMedicalRecordList(), MedicalRecord.class);
        if (ListUtil.isNotEmpty(medicalRecordList)) {
            medicalRecordList.forEach(s -> {
                QueryWrapper<MedicalRecord> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("rec_id", s.getRecId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                int count = medicalRecordService.count(queryWrapper);
                if (count > 0) {
                    medicalRecordService.update(s, queryWrapper);
                } else {
                    medicalRecordService.save(s);
                }
            });
        }

        // 文书明细信息
        List<MedicalRecordContent> medicalRecordContentList = BeanUtil.listCopyTo(dataImportVO.getMedicalRecordContentList(), MedicalRecordContent.class);
        if (ListUtil.isNotEmpty(medicalRecordContentList)) {
            // 加密数据
            if (encryptFlag) {
                String recId = "";
                try {
                    EncrypDES encrypDES = new EncrypDES();
                    for (MedicalRecordContent s : medicalRecordContentList) {
                        recId = s.getRecId();
                        if (StringUtil.isNotBlank(s.getXmlText())) {
                            s.setXmlText(encrypDES.encrytor(s.getXmlText()));
                        }
                        QueryWrapper<MedicalRecordContent> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("rec_id", s.getRecId());
                        queryWrapper.eq("hospital_id", s.getHospitalId());
                        queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                        int count = medicalRecordContentService.count(queryWrapper);
                        if (count > 0) {
                            medicalRecordContentService.update(s, queryWrapper);
                        } else {
                            medicalRecordContentService.save(s);
                        }
                    }
                } catch (Exception e) {
                    throw new ApiException( "加密xml错误!recId=【" + recId + "】");
                }
            }
        }
        return true;
    }

    /**
     * 数据导入
     *
     * @param dataImportVO
     * @return
     */
    public Boolean dataImportFacade(DataImportVO dataImportVO) {
        Long hospitalId = dataImportVO.getHospitalId();
        // 病历信息导入
        List<BehospitalInfo> behospitalInfoList = BeanUtil.listCopyTo(dataImportVO.getBehospitalInfoList(), BehospitalInfo.class);
        if (ListUtil.isNotEmpty(behospitalInfoList)) {
            List<String> behospitalCodeList = behospitalInfoList.stream().map(r -> r.getBehospitalCode()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(behospitalCodeList)) {
                // 先删除
                behospitalInfoService.remove(new QueryWrapper<BehospitalInfo>()
                        .eq("hospital_id", hospitalId)
                        .in("behospital_code", behospitalCodeList)
                );
                // 加密
                for (BehospitalInfo behospitalInfo : behospitalInfoList) {
                    // 姓名
                    if (StringUtil.isNotBlank(behospitalInfo.getName())) {
                        behospitalInfo.setName(behospitalInfo.getName().substring(0, 1) + "**");
                    }
                }
                behospitalInfoService.saveBatch(behospitalInfoList);
            }
        }

        // 危急值信息
        List<CrisisInfo> medCrisisInfoList = BeanUtil.listCopyTo(dataImportVO.getMedCrisisInfoList(), CrisisInfo.class);
        if (ListUtil.isNotEmpty(medCrisisInfoList)) {
            List<String> recIdList = medCrisisInfoList.stream().map(r -> r.getRecId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(recIdList)) {
                // 先删除
                crisisInfoService.remove(new QueryWrapper<CrisisInfo>()
                        .eq("hospital_id", hospitalId)
                        .in("rec_id", recIdList)
                );
                crisisInfoService.saveBatch(medCrisisInfoList);
            }
        }

        // 病人医嘱
        List<DoctorAdvice> doctorAdviceList = BeanUtil.listCopyTo(dataImportVO.getDoctorAdviceList(), DoctorAdvice.class);
        if (ListUtil.isNotEmpty(doctorAdviceList)) {
            List<String> doctorAdviceIdList = doctorAdviceList.stream().map(r -> r.getDoctorAdviceId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(doctorAdviceIdList)) {
                // 先删除
                doctorAdviceService.remove(new QueryWrapper<DoctorAdvice>()
                        .eq("hospital_id", hospitalId)
                        .in("doctor_advice_id", doctorAdviceIdList)
                );
                doctorAdviceService.saveBatch(doctorAdviceList);
            }
        }

        // 病案首页诊断
        List<HomeDiagnoseInfo> homeDiagnoseInfoList = BeanUtil.listCopyTo(dataImportVO.getHomeDiagnoseInfoList(), HomeDiagnoseInfo.class);
        if (ListUtil.isNotEmpty(homeDiagnoseInfoList)) {
            StringBuffer sb = new StringBuffer();
            sb.append(" ( ");
            for (int i = 0; i < homeDiagnoseInfoList.size(); i++) {
                HomeDiagnoseInfo s = homeDiagnoseInfoList.get(i);
                if (i == 0) {
                    sb.append(" (home_page_id = '" + s.getHomePageId() + "'"
                            + " and diagnose_order_no = '" + s.getDiagnoseOrderNo() + "'"
                            + ") " );
                } else {
                    sb.append(" or ( home_page_id = '" + s.getHomePageId() + "'"
                            + " and diagnose_order_no = '" + s.getDiagnoseOrderNo() + "'"
                            + ") " );
                }
            }
            sb.append(")");

            // 先删除
            homeDiagnoseInfoService.remove(new QueryWrapper<HomeDiagnoseInfo>()
                    .eq("hospital_id", hospitalId)
                    .apply(sb.toString())
            );
            homeDiagnoseInfoService.saveBatch(homeDiagnoseInfoList);
        }

        // 病案首页
        List<HomePage> homePageList = BeanUtil.listCopyTo(dataImportVO.getHomePageList(), HomePage.class);
        if (ListUtil.isNotEmpty(homePageList)) {
            List<String> homePageIdList = homePageList.stream().map(r -> r.getHomePageId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(homePageIdList)) {
                // 先删除
                homePageService.remove(new QueryWrapper<HomePage>()
                        .eq("hospital_id", hospitalId)
                        .in("home_page_id", homePageIdList)
                );
                // 加密
                for (HomePage homePage : homePageList) {
                    // 姓名
                    if (StringUtil.isNotBlank(homePage.getName())) {
                        homePage.setName(homePage.getName().substring(0, 1) + "**");
                    }
                    // 身份证
                    if (StringUtil.isNotBlank(homePage.getIdentityCardNo())) {
                        homePage.setIdentityCardNo(homePage.getIdentityCardNo().substring(0, 1) + "*****************");
                    }
                    // 现住址电话
                    if (StringUtil.isNotBlank(homePage.getCurPhone())) {
                        homePage.setCurPhone(homePage.getCurPhone().substring(0, 1) + "**");
                    }
                    // 工作单位电话
                    if (StringUtil.isNotBlank(homePage.getWorkPhone())) {
                        homePage.setWorkPhone(homePage.getWorkPhone().substring(0, 1) + "**");
                    }
                    // 联系人姓名
                    if (StringUtil.isNotBlank(homePage.getContactName())) {
                        homePage.setContactName(homePage.getContactName().substring(0, 1) + "**");
                    }
                    // 联系人电话
                    if (StringUtil.isNotBlank(homePage.getContactPhone())) {
                        homePage.setContactPhone(homePage.getContactPhone().substring(0, 1) + "**");
                    }
                }
                homePageService.saveBatch(homePageList);
            }
        }

        // 病案首页手术信息
        List<HomeOperationInfo> homeOperationInfoList = BeanUtil.listCopyTo(dataImportVO.getHomeOperationInfoList(), HomeOperationInfo.class);
        if (ListUtil.isNotEmpty(homeOperationInfoList)) {
            StringBuffer sb = new StringBuffer();
            sb.append(" ( ");
            for (int i = 0; i < homeOperationInfoList.size(); i++) {
                HomeOperationInfo s = homeOperationInfoList.get(i);
                if (i == 0) {
                    sb.append(" (home_page_id = '" + s.getHomePageId() + "'"
                            + " and operation_order_no = '" + s.getOperationOrderNo() + "'"
                            + ") " );
                } else {
                    sb.append(" or ( home_page_id = '" + s.getHomePageId() + "'"
                            + " and operation_order_no = '" + s.getOperationOrderNo() + "'"
                            + ") " );
                }
            }
            sb.append(")");
            // 先删除
            homeOperationInfoService.remove(new QueryWrapper<HomeOperationInfo>()
                    .eq("hospital_id", hospitalId)
                    .apply(sb.toString())
            );
            homeOperationInfoService.saveBatch(homeOperationInfoList);
        }

        // 文书信息
        List<MedicalRecord> medicalRecordList = BeanUtil.listCopyTo(dataImportVO.getMedicalRecordList(), MedicalRecord.class);
        if (ListUtil.isNotEmpty(medicalRecordList)) {
            List<String> recIdList = medicalRecordList.stream().map(r -> r.getRecId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(recIdList)) {
                // 先删除
                medicalRecordService.remove(new QueryWrapper<MedicalRecord>()
                        .eq("hospital_id", hospitalId)
                        .in("rec_id", recIdList)
                );
                medicalRecordService.saveBatch(medicalRecordList);
            }
        }

        // 文书明细信息
        List<MedicalRecordContent> medicalRecordContentList = BeanUtil.listCopyTo(dataImportVO.getMedicalRecordContentList(), MedicalRecordContent.class);
        if (ListUtil.isNotEmpty(medicalRecordContentList)) {
            List<String> recIdList = medicalRecordContentList.stream().map(r -> r.getRecId()).collect(Collectors.toList());
            if (ListUtil.isNotEmpty(recIdList)) {
                // 先删除
                medicalRecordContentService.remove(new QueryWrapper<MedicalRecordContent>()
                        .eq("hospital_id", hospitalId)
                        .in("rec_id", recIdList));

                // 加密数据
                if (encryptFlag) {
                    String recId = "";
                    try {
                        EncrypDES encrypDES = new EncrypDES();
                        for (MedicalRecordContent medicalRecordContent : medicalRecordContentList) {
                            recId = medicalRecordContent.getRecId();
                            if (StringUtil.isNotBlank(medicalRecordContent.getXmlText())) {
                                medicalRecordContent.setXmlText(encrypDES.encrytor(medicalRecordContent.getXmlText()));
                            }
                        }
                    } catch (Exception e) {
                        throw new ApiException("加密xml错误!recId=【" + recId + "】");
                    }
                }
                medicalRecordContentService.saveBatch(medicalRecordContentList, 100);
            }
        }
        return true;
    }
}