package com.lantone.report.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.collect.Lists;
import com.lantone.common.dto.report.ColumnDTO;
import com.lantone.common.dto.report.HomePageImproveDTO;
import com.lantone.common.dto.report.MedicalCheckDTO;
import com.lantone.common.dto.report.MedicalCheckExportDTO;
import com.lantone.common.dto.report.MedicalCheckIdNameDTO;
import com.lantone.common.util.ExcelUtils;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.util.SysUserUtils;
import com.lantone.common.vo.report.FilterMedicalCheckVO;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.dblayermbg.entity.HospitalSet;
import com.lantone.dblayermbg.facade.HospitalSetFacade;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.List;


@Component
public class QcCheckStatisticsFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;
    @Autowired
    private HospitalSetFacade hospitalSetFacade;

    /**
     * 病历稽查表
     *
     * @param filterVO
     * @return
     */
    public IPage<MedicalCheckDTO> getMedicalCheck(@Param("filterVO") FilterMedicalCheckVO filterVO) {
        filterFacade.getMedicalCheckVOSet(filterVO);
        //colums
        QueryWrapper<HospitalSet> hospitalSetQueryWrapper = new QueryWrapper<>();
        hospitalSetQueryWrapper.eq("is_deleted", 'N')
                .eq("hospital_id", filterVO.getHospitalId())
                .eq("code", "medical_check_form");
        HospitalSet hospitalSet = hospitalSetFacade.getOne(hospitalSetQueryWrapper);

        //表头生成
        List<String> columnSet = Arrays.asList(hospitalSet.getVal().split(","));

        //目标条目
        List<Long> casesEntryIds = Lists.newArrayList();
        //遍历第一个条目
        for (String valueStr : columnSet) {
            if (StringUtil.isBlank(valueStr)) {
                continue;
            }
            String[] keyValue = valueStr.split("--");
            if (keyValue != null || keyValue.length > 1) {
                casesEntryIds.add(Long.valueOf(keyValue[0]));
            }
        }
        IPage<MedicalCheckDTO> medicalCheck = getMedicalCheck(filterVO, casesEntryIds);
        return medicalCheck;
    }

    /**
     * 病历稽查表（title）
     *
     * @param
     * @return
     */
    public List<ColumnDTO> getMedicalCheckTitle() {
        //病历稽查入参拼接
        String hospitalId = String.valueOf(SysUserUtils.getCurrentHospitalId());
        //colums
        QueryWrapper<HospitalSet> hospitalSetQueryWrapper = new QueryWrapper<>();
        hospitalSetQueryWrapper.eq("is_deleted", 'N')
                .eq("hospital_id", hospitalId)
                .eq("code", "medical_check_form");
        List<ColumnDTO> columns = Lists.newLinkedList();
        HospitalSet hospitalSet = hospitalSetFacade.getOne(hospitalSetQueryWrapper);
        if (hospitalSet == null || StringUtil.isBlank(hospitalSet.getVal())) {
            return columns;
        }

        //表头生成
        List<String> columnSet = Arrays.asList(hospitalSet.getVal().split(","));
        Integer orderNo = 1;
        ColumnDTO columnDeptId = new ColumnDTO();
        columnDeptId.setOrderNo(orderNo);
        columnDeptId.setFieldName("deptId");
        columnDeptId.setColumnName("科室编码");
        columnDeptId.setIsShow(0);
        columns.add(columnDeptId);
        orderNo++;
        ColumnDTO columnDeptName = new ColumnDTO();
        columnDeptName.setOrderNo(orderNo);
        columnDeptName.setFieldName("deptName");
        columnDeptName.setColumnName("科室");
        columnDeptName.setIsShow(1);
        columns.add(columnDeptName);
        orderNo++;
        ColumnDTO columnMedicalName = new ColumnDTO();
        columnMedicalName.setOrderNo(orderNo);
        columnMedicalName.setFieldName("doctorName");
        columnMedicalName.setColumnName("医疗组");
        columnMedicalName.setIsShow(1);
        columns.add(columnMedicalName);
        orderNo++;
        for (String valueStr : columnSet) {
            String[] keyValue = valueStr.split("--");
            if (keyValue != null || keyValue.length > 1) {
                ColumnDTO columnNum = new ColumnDTO();
                columnNum.setOrderNo(orderNo);
                columnNum.setId(Long.valueOf(keyValue[0]));
                columnNum.setFieldName("entry_" + keyValue[0] + "_name");
                columnNum.setColumnName(keyValue[1]);
                columnNum.setIsShow(1);
                columns.add(columnNum);
                orderNo++;
            }
        }

        return columns;
    }


    /**
     * 病历稽查表
     *
     * @param filterVO
     * @return
     */
    public IPage<MedicalCheckDTO> getMedicalCheck(@Param("filterVO") FilterMedicalCheckVO filterVO, @Param("casesEntryIds") List<Long> casesEntryIds) {
        IPage<MedicalCheckDTO> medicalCheck = behospitalInfoFacade.getBaseMapper().getMedicalCheck(filterVO);
        List<MedicalCheckIdNameDTO> medicalEntryIds = behospitalInfoFacade.getBaseMapper().getMedicalEntryIds(casesEntryIds);

        List<MedicalCheckDTO> records = medicalCheck.getRecords();
        for (MedicalCheckDTO record : records) {
            for (MedicalCheckIdNameDTO medicalEntryId : medicalEntryIds) {
                if ("2170".equals(medicalEntryId.getId())) {
                    record.setEntry_2170_id(medicalEntryId.getId());
                    record.setEntry_2170_name(medicalEntryId.getName());
                }
                if ("2217".equals(medicalEntryId.getId())) {
                    record.setEntry_2217_id(medicalEntryId.getId());
                    record.setEntry_2217_name(medicalEntryId.getName());
                }
                if ("2218".equals(medicalEntryId.getId())) {
                    record.setEntry_2218_id(medicalEntryId.getId());
                    record.setEntry_2218_name(medicalEntryId.getName());
                }
                if ("2220".equals(medicalEntryId.getId())) {
                    record.setEntry_2220_id(medicalEntryId.getId());
                    record.setEntry_2220_name(medicalEntryId.getName());
                }
                if ("2229".equals(medicalEntryId.getId())) {
                    record.setEntry_2229_id(medicalEntryId.getId());
                    record.setEntry_2229_name(medicalEntryId.getName());
                }
                if ("2287".equals(medicalEntryId.getId())) {
                    record.setEntry_2287_id(medicalEntryId.getId());
                    record.setEntry_2287_name(medicalEntryId.getName());
                }
                if ("2486".equals(medicalEntryId.getId())) {
                    record.setEntry_2486_id(medicalEntryId.getId());
                    record.setEntry_2486_name(medicalEntryId.getName());
                }
                if ("2495".equals(medicalEntryId.getId())) {
                    record.setEntry_2495_id(medicalEntryId.getId());
                    record.setEntry_2495_name(medicalEntryId.getName());
                }
                if ("2654".equals(medicalEntryId.getId())) {
                    record.setEntry_2654_id(medicalEntryId.getId());
                    record.setEntry_2654_name(medicalEntryId.getName());
                }
                if ("2655".equals(medicalEntryId.getId())) {
                    record.setEntry_2655_id(medicalEntryId.getId());
                    record.setEntry_2655_name(medicalEntryId.getName());
                }
                if ("2657".equals(medicalEntryId.getId())) {
                    record.setEntry_2657_id(medicalEntryId.getId());
                    record.setEntry_2657_name(medicalEntryId.getName());
                }
                if ("2658".equals(medicalEntryId.getId())) {
                    record.setEntry_2658_id(medicalEntryId.getId());
                    record.setEntry_2658_name(medicalEntryId.getName());
                }
                if ("2686".equals(medicalEntryId.getId())) {
                    record.setEntry_2686_id(medicalEntryId.getId());
                    record.setEntry_2686_name(medicalEntryId.getName());
                }
                if ("2852".equals(medicalEntryId.getId())) {
                    record.setEntry_2852_id(medicalEntryId.getId());
                    record.setEntry_2852_name(medicalEntryId.getName());
                }
                if ("2900".equals(medicalEntryId.getId())) {
                    record.setEntry_2900_id(medicalEntryId.getId());
                    record.setEntry_2900_name(medicalEntryId.getName());
                }
                if ("2930".equals(medicalEntryId.getId())) {
                    record.setEntry_2930_id(medicalEntryId.getId());
                    record.setEntry_2930_name(medicalEntryId.getName());
                }
                if ("3010".equals(medicalEntryId.getId())) {
                    record.setEntry_3010_id(medicalEntryId.getId());
                    record.setEntry_3010_name(medicalEntryId.getName());
                }
                if ("3025".equals(medicalEntryId.getId())) {
                    record.setEntry_3025_id(medicalEntryId.getId());
                    record.setEntry_3025_name(medicalEntryId.getName());
                }
                if ("3062".equals(medicalEntryId.getId())) {
                    record.setEntry_3062_id(medicalEntryId.getId());
                    record.setEntry_3062_name(medicalEntryId.getName());
                }
                if ("3063".equals(medicalEntryId.getId())) {
                    record.setEntry_3063_id(medicalEntryId.getId());
                    record.setEntry_3063_name(medicalEntryId.getName());
                }
            }
        }

        return medicalCheck;

    }

    /**
     * 稽查统计导出（首页）
     *
     * @param filterVO
     * @return
     */
    public void medicalCheckExport(HttpServletResponse response, FilterMedicalCheckVO filterVO) {
        filterVO.setCurrent(1L);
        filterVO.setSize(Long.MAX_VALUE);
        filterVO.setSearchCount(false);
        String fileName = null;
        if ("1".equals(filterVO.getIsPlacefile())) {
            fileName = "终末病历稽查表.xls";
        } else {
            fileName = "运行病历稽查表.xls";
        }
        filterFacade.getMedicalCheckVOSet(filterVO);
        List<MedicalCheckExportDTO> records = behospitalInfoFacade.getBaseMapper().getMedicalCheckExport(filterVO);
        ExcelUtils.exportExcel(records, null, "sheet1", MedicalCheckExportDTO.class, fileName, response, 15.8f);
    }

    /**
     * 质控核查统计（内页）
     *
     * @param filterOrderVO
     * @return
     */
    public List<HomePageImproveDTO> qcCheckStatistics(FilterOrderVO filterOrderVO) {
        filterFacade.filterOrderVOSet(filterOrderVO);
        List<HomePageImproveDTO> records = behospitalInfoFacade.getBaseMapper().qcCheckStatistics(filterOrderVO);
        //增加全院数据
        if (StringUtil.isBlank(filterOrderVO.getDeptName()) || filterOrderVO.getDeptName().equals("全院")) {
            HomePageImproveDTO item = getGlobleRecord_improve(records);
            if (item != null) {
                records.add(0, item);
            }
        }
        return records;
    }

    /**
     * 质控核查统计导出
     *
     * @param filterOrderVO
     * @return
     */
    public void qcCheckStatisticsExport(HttpServletResponse response, FilterOrderVO filterOrderVO) {
        List<HomePageImproveDTO> records = this.qcCheckStatistics(filterOrderVO);
        String fileName = "质控核查统计.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", HomePageImproveDTO.class, fileName, response, 12.8f);
    }


    /**
     * 增加全院记录
     *
     * @param records
     * @return
     */
    public HomePageImproveDTO getGlobleRecord_improve(List<HomePageImproveDTO> records) {
        DecimalFormat df = new DecimalFormat("#0.00");
        HomePageImproveDTO item = new HomePageImproveDTO();

        if (ListUtil.isEmpty(records)) {
            return null;
        }

        item.setDeptName("全院");

        //总病历数
        Integer mrNum = records
                .stream()
                .map(HomePageImproveDTO::getMrNum)
                .reduce(0, Integer::sum);
        if (mrNum == null || mrNum == 0) {
            return null;
        }
        item.setMrNum(mrNum);
        //核查病历数
        Integer checkedNum = records
                .stream()
                .map(HomePageImproveDTO::getCheckedNum)
                .reduce(0, Integer::sum);
        item.setCheckedNum(checkedNum);
        if (mrNum != null && !mrNum.equals(0)) {
            //核查病历占比
            Double checkedPercent = BigDecimal.valueOf(checkedNum)
                    .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            String checkedPercentStr
                    = df.format(BigDecimal.valueOf(checkedPercent).multiply(BigDecimal.valueOf(100))) + "%";
            item.setCheckedPercent(checkedPercent);
            item.setCheckedPercentStr(checkedPercentStr);
        }
        //带病案首页的病历数
        Integer homePageMRNum = records
                .stream()
                .map(HomePageImproveDTO::getHomePageMRNum)
                .reduce(0, Integer::sum);
        item.setHomePageMRNum(homePageMRNum);
        if (homePageMRNum == null || homePageMRNum.equals(0)) {
            return item;
        }
        //改善病历数
        Integer improveNum = records
                .stream()
                .map(HomePageImproveDTO::getImproveNum)
                .reduce(0, Integer::sum);
        //改善率
        Double improvePercent = BigDecimal.valueOf(improveNum)
                .divide(BigDecimal.valueOf(homePageMRNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String improvePercentStr
                = df.format(BigDecimal.valueOf(improvePercent).multiply(BigDecimal.valueOf(100))) + "%";
        //改善至合格病历数
        Integer improveToPassNum = records
                .stream()
                .map(HomePageImproveDTO::getImproveToPassNum)
                .reduce(0, Integer::sum);
        //改善至合格占比
        Double improveToPassPercent = BigDecimal.valueOf(improveToPassNum)
                .divide(BigDecimal.valueOf(homePageMRNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String improveToPassPercentStr
                = df.format(BigDecimal.valueOf(improveToPassPercent).multiply(BigDecimal.valueOf(100))) + "%";
        //改善至满分病历数
        Integer improveToFullNum = records
                .stream()
                .map(HomePageImproveDTO::getImproveToFullNum)
                .reduce(0, Integer::sum);
        //改善至满分占比
        Double improveToFullPercent = BigDecimal.valueOf(improveToFullNum)
                .divide(BigDecimal.valueOf(homePageMRNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String improveToFullPercentStr
                = df.format(BigDecimal.valueOf(improveToFullPercent).multiply(BigDecimal.valueOf(100))) + "%";

        item.setImproveNum(improveNum);
        item.setImprovePercent(improvePercent);
        item.setImprovePercentStr(improvePercentStr);
        item.setImproveToPassNum(improveToPassNum);
        item.setImproveToPassPercent(improveToPassPercent);
        item.setImproveToPassPercentStr(improveToPassPercentStr);
        item.setImproveToFullNum(improveToFullNum);
        item.setImproveToFullPercent(improveToFullPercent);
        item.setImproveToFullPercentStr(improveToFullPercentStr);
        return item;
    }
}