package com.lantone.report.facade;

import cn.hutool.core.bean.BeanUtil;
import com.google.common.collect.Lists;
import com.lantone.common.dto.report.AverageStatisticsDTO;
import com.lantone.common.dto.report.DeptNumDTO;
import com.lantone.common.dto.report.LevelStatisticsDTO;
import com.lantone.common.dto.report.LevelStatisticsTZDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.ExcelUtils;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.QcresultFilterVO;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import com.lantone.dblayermbg.facade.report.QcresultInfoFacade;
import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Component
public class QcScoreFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;


    /**
     * 质控病历统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterFacade.filterVOSet(filterVO);
        List<NumDTO> mrCountList = mrCounts(filterVO);
        if (ListUtil.isNotEmpty(mrCountList)) {
            retMap.put("病历数统计", mrCountList);
        }
        return retMap;
    }

    /**
     * 各科室甲/乙/丙级病历占比
     *
     * @param filterVO
     * @return
     */
    public List<DeptNumDTO> qcResultLevelPercent(FilterVO filterVO) {
        if (StringUtil.isBlank(filterVO.getLevel())) {
            throw new ApiException("请输入病历等级");
        }
        List<DeptNumDTO> retList = Lists.newLinkedList();
        filterFacade.filterVOSet(filterVO);
        Integer limitCount = filterVO.getLimitCount();
        List<DeptNumDTO> numDTOList = qcresultInfoFacade.getBaseMapper().qcResultLevelPercent(filterVO);
        if (ListUtil.isNotEmpty(numDTOList)) {
            if (numDTOList.size() <= limitCount) {
                retList = numDTOList;
            } else {
                retList = numDTOList.subList(0, limitCount);
            }
        }
        return retList;
    }

    /**
     * 质控病历统计
     *
     * @return
     */
    public List<NumDTO> mrCounts(FilterVO filterVO) {
        List<NumDTO> retList = Lists.newLinkedList();
        QcresultFilterVO qcresultFilterVO = new QcresultFilterVO();
        BeanUtil.copyProperties(filterVO, qcresultFilterVO);
        NumDTO totleNumDTO = new NumDTO();
        NumDTO firstLevelNumDTO = new NumDTO();
        NumDTO secondLevelNumDTO = new NumDTO();
        NumDTO thirdLevelNumDTO = new NumDTO();
        Map<String,Long> resultMap = qcresultInfoFacade.getBaseMapper().resultCount(qcresultFilterVO);
        if(MapUtils.isNotEmpty(resultMap)){
            if(resultMap.containsKey("totleNum")){
                Long value = resultMap.get("totleNum");
                if(null == value){
                    return retList;
                }
                totleNumDTO.setName("累计质控病历数");
                totleNumDTO.setTotleNum(value.intValue());
                totleNumDTO.setNum(value.intValue());
                retList.add(totleNumDTO);
            }
            resultMap.forEach((key,value)->{
                if("firstLevelNum".equals(key)){
                    firstLevelNumDTO.setName("甲级病历");
                    firstLevelNumDTO.setNum(value.intValue());
                    countNumDto(firstLevelNumDTO, totleNumDTO, value.intValue());
                }

                if("secondLevelNum".equals(key)){
                    secondLevelNumDTO.setName("乙级病历");
                    countNumDto(secondLevelNumDTO, totleNumDTO, value.intValue());
                }
                if("thirdLevelNum".equals(key)){
                    thirdLevelNumDTO.setName("丙级病历");
                    countNumDto(thirdLevelNumDTO, totleNumDTO, value.intValue());
                }
            });
                retList.add(firstLevelNumDTO);
                retList.add(secondLevelNumDTO);
                retList.add(thirdLevelNumDTO);
        }
        return retList;
    }

    private void countNumDto(NumDTO numDTO,NumDTO totleNum,Integer value){
        numDTO.setNum(value);
        DecimalFormat df = new DecimalFormat("#0.00");
        numDTO.setTotleNum(totleNum.getTotleNum());
        Double percent = BigDecimal.valueOf(value)
                .divide(BigDecimal.valueOf(totleNum.getTotleNum()), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String percentStr = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
        numDTO.setPercent(percent);
        numDTO.setPercentStr(percentStr);
    }

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO) {
        filterFacade.filterOrderVOSame(filterOrderVO);
        List<LevelStatisticsDTO> records = behospitalInfoFacade.getBaseMapper().levelStatistics(filterOrderVO);
        //没有科室过滤时增加全院数据
        if (StringUtil.isBlank(filterOrderVO.getDeptName()) || filterOrderVO.getDeptName().equals("全院")) {
            LevelStatisticsDTO item = getGlobleTitle(records);
            if (item != null) {
                records.add(0, item);
            }
        }
        if (ListUtil.isNotEmpty(records)) {
            for (LevelStatisticsDTO record : records) {
                BigDecimalMethod(record);
            }
        }
        return records;
    }

    /**
     * 各科室缺陷占比-导出
     *
     * @param filterOrderVO
     * @return
     */
    public void levelExport(HttpServletResponse response, FilterOrderVO filterOrderVO) {
        List<LevelStatisticsDTO> records = this.levelStatisticsStr(filterOrderVO);
        String fileName = "各科室缺陷占比.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", LevelStatisticsDTO.class, fileName, response, 12.8f);
    }

    /**
     * 各科室缺陷占比（组合）-导出
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatisticsStr(FilterOrderVO filterOrderVO) {
        filterFacade.filterOrderVOSame(filterOrderVO);
        List<LevelStatisticsDTO> records = behospitalInfoFacade.getBaseMapper().levelStatistics(filterOrderVO);
        //没有科室过滤时增加全院数据
        if (StringUtil.isBlank(filterOrderVO.getDeptName()) || filterOrderVO.getDeptName().equals("全院")) {
            LevelStatisticsDTO item = getGlobleTitle(records);
            if (item != null) {
                records.add(0, item);
            }
        }
        if (ListUtil.isNotEmpty(records)) {
            for (LevelStatisticsDTO record : records) {
                BigDecimalStrExport(record);
            }
        }
        return records;
    }


    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计（台州）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsTZDTO> levelStatisticsByDeptClass_TZ(FilterOrderVO filterOrderVO) {
        filterFacade.filterOrderVOSame(filterOrderVO);
        List<LevelStatisticsTZDTO> records = behospitalInfoFacade.getBaseMapper().levelStatisticsByDeptClass_TZ(filterOrderVO);
        if (StringUtil.isBlank(filterOrderVO.getDeptName())) {
            LevelStatisticsTZDTO item = getGlobleRecords(records, filterOrderVO);
            if (item != null) {
                records.add(0, item);
            }
        }
        if (ListUtil.isNotEmpty(records)) {
            for (LevelStatisticsTZDTO record : records) {
                BigDecimalMethod(record);
            }
        }

        return records;
    }

    public void BigDecimalStrExport(LevelStatisticsDTO record) {

        BigDecimal entrySameBigDecimal = getSubtract(new BigDecimal(record.getEntryNum()), new BigDecimal(record.getSameNum() == null ? 0 : record.getSameNum()));

        if (entrySameBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setEntryStrNum(record.getEntryNum() + "↑");
        } else if (entrySameBigDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setEntryStrNum(record.getEntryNum() + "↓");
        }

        BigDecimal sameMrNumBigDecimal = getSubtract(new BigDecimal(record.getMrNum()), new BigDecimal(record.getSameMrNum() == null ? 0 : record.getSameMrNum()));
        if (sameMrNumBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setMrStrNum(record.getMrNum() + "↑");
        } else if (sameMrNumBigDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setMrStrNum(record.getMrNum() + "↓");
        }

        BigDecimal sameAvgValueDecimal = getSubtract(new BigDecimal(record.getAverageValue()), new BigDecimal(record.getSameAvgValue() == null ? 0 : record.getSameAvgValue()));
        if (sameAvgValueDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setAverageStrValue(record.getAverageValue() + "↑");
        } else if (sameAvgValueDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setAverageStrValue(record.getAverageValue() + "↓");
        }

        BigDecimal SameFirstLevelBigDecimal = getSubtract(new BigDecimal(record.getFirstLevelNum()), new BigDecimal(record.getSameFirstLevelNum() == null ? 0 : record.getSameFirstLevelNum()));
        if (SameFirstLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setFirstLevelStrNum(record.getFirstLevelNum() + "↑");
        } else if (SameFirstLevelBigDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setFirstLevelStrNum(record.getFirstLevelNum() + "↓");
        }

        BigDecimal SameSecondLevelBigDecimal = getSubtract(new BigDecimal(record.getSecondLevelNum()), new BigDecimal(record.getSameSecondLevelNum() == null ? 0 : record.getSameSecondLevelNum()));
        if (SameSecondLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setSecondLevelStrNum(record.getSecondLevelNum() + "↑");
        } else if (SameSecondLevelBigDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setSecondLevelStrNum(record.getSecondLevelNum() + "↓");
        }

        BigDecimal SameThirdLevelBigDecimal = getSubtract(new BigDecimal(record.getThirdLevelNum()), new BigDecimal(record.getSameThirdLevelNum() == null ? 0 : record.getSameThirdLevelNum()));
        if (SameThirdLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setThirdLevelStrNum(record.getThirdLevelNum() + "↑");
        } else if (SameThirdLevelBigDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setThirdLevelStrNum(record.getThirdLevelNum() + "↓");
        }

        BigDecimal SameFirstLevelPeDecimal = getSubtract(new BigDecimal(record.getFirstLevelPercent()), new BigDecimal(record.getSameFirstLevelPercent() == null ? 0 : record.getSameFirstLevelPercent()));
        if (SameFirstLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setFirstLevelPercentStr(record.getFirstLevelPercentStr() + "↑");
        } else if (SameFirstLevelPeDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setFirstLevelPercentStr(record.getFirstLevelPercentStr() + "↓");
        }

        BigDecimal SameSecondLevelPeDecimal = getSubtract(new BigDecimal(record.getSecondLevelPercent()), new BigDecimal(record.getSameSecondLevelPercent() == null ? 0 : record.getSameSecondLevelPercent()));
        if (SameSecondLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setSecondLevelPercentStr(record.getSecondLevelPercentStr() + "↑");
        } else if (SameSecondLevelPeDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setSecondLevelPercentStr(record.getSecondLevelPercentStr() + "↓");
        }

        BigDecimal SameThirdLevelPeDecimal = getSubtract(new BigDecimal(record.getThirdLevelPercent()), new BigDecimal(record.getSameThirdLevelPercent() == null ? 0 : record.getSameThirdLevelPercent()));
        if (SameThirdLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setThirdLevelPercentStr(record.getThirdLevelPercentStr() + "↑");
        } else if (SameThirdLevelPeDecimal.compareTo(BigDecimal.ZERO) < 0) {
            record.setThirdLevelPercentStr(record.getThirdLevelPercentStr() + "↓");
        }
    }


    /**
     * 各科室缺陷占比（组合）-台州-导出
     *
     * @param filterOrderVO
     * @return
     */
    public void levelExport_TZ(HttpServletResponse response, FilterOrderVO filterOrderVO) {
        List<LevelStatisticsTZDTO> records = this.levelStatisticsByDeptStr_TZ(filterOrderVO);
        String fileName = "各科室缺陷占比-台州.xls";
        ExcelUtils.exportExcel(records, null, "sheet1", LevelStatisticsTZDTO.class, fileName, response, 12.8f);
    }

    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计（台州）-导出
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsTZDTO> levelStatisticsByDeptStr_TZ(FilterOrderVO filterOrderVO) {
        filterFacade.filterOrderVOSame(filterOrderVO);
        List<LevelStatisticsTZDTO> records = behospitalInfoFacade.getBaseMapper().levelStatisticsByDeptClass_TZ(filterOrderVO);
        if (StringUtil.isBlank(filterOrderVO.getDeptName())) {
            LevelStatisticsTZDTO item = getGlobleRecords(records, filterOrderVO);
            if (item != null) {
                records.add(0, item);
            }
        }
        if (ListUtil.isNotEmpty(records)) {
            for (LevelStatisticsTZDTO record : records) {
                BigDecimalStrExport(record);
            }
        }

        return records;
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageScore(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterFacade.filterVOSame(filterVO);
        List<AverageStatisticsDTO> averageScoreList = this.getAverageScores(filterVO);

        AverageStatisticsDTO item = getGlobleAverageTitle(averageScoreList);
        if (item != null) {
            averageScoreList.add(0, item);
        }

        if (ListUtil.isNotEmpty(averageScoreList)) {
            retMap.put("各科室质控平均分", averageScoreList);
        }
        return retMap;
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScores(FilterVO filterVO) {
        List<AverageStatisticsDTO> retAverageScoreList = Lists.newLinkedList();
        List<AverageStatisticsDTO> averageScoreList = qcresultInfoFacade.getBaseMapper().getAverageScore(filterVO);
        Integer limitCount = filterVO.getLimitCount();
        //质控平均分
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retAverageScoreList = getLimitAverageList(averageScoreList, limitCount);
        }
        return retAverageScoreList;
    }

    /**
     * 各科室质控平均分（首页）-根据内外科系统统计
     *
     * @param filterVO
     * @return
     */
    public List<AverageStatisticsDTO> getAverageScoreByDeptClass(FilterVO filterVO) {
        filterFacade.filterVOSame(filterVO);
        List<AverageStatisticsDTO> retAverageScoreList = Lists.newLinkedList();
        List<AverageStatisticsDTO> averageScoreList = qcresultInfoFacade.getBaseMapper().getAverageScoreByDeptClass(filterVO);
        Integer limitCount = filterVO.getLimitCount();
        //质控平均分
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retAverageScoreList = this.getLimitAverageList(averageScoreList, limitCount);
        }
        AverageStatisticsDTO item = getGlobleInOutTitle(retAverageScoreList, filterVO);
        if (item != null) {
            retAverageScoreList.add(0, item);
        }
        return retAverageScoreList;
    }

    /**
     * 增加内外科记录
     *
     * @param records
     * @return
     */
    public AverageStatisticsDTO getGlobleInOutTitle(List<AverageStatisticsDTO> records, FilterVO filterVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        AverageStatisticsDTO item = new AverageStatisticsDTO();
        if (ListUtil.isEmpty(records)) {
            return null;
        }

        //数量
        Integer num = records
                .stream()
                .map(AverageStatisticsDTO::getNum)
                .reduce(0, Integer::sum);

        //同期数量
        Integer sameNum = records
                .stream()
                .map(AverageStatisticsDTO::getSameNum)
                .reduce(0, Integer::sum);

        Double totleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        Double sameTotleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getSameTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        Double averageValue = 0d;
        if (0 != num) {
            averageValue = BigDecimal.valueOf(totleValue)
                    .divide(BigDecimal.valueOf(num), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }
        Double lastYearAverageValue = 0d;
        if (0 != sameNum) {
            lastYearAverageValue = BigDecimal.valueOf(sameTotleValue)
                    .divide(BigDecimal.valueOf(sameNum), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }

        if ("内科".equals(filterVO.getDeptClass())) {
            item.setName("内科系统");
        } else {
            item.setName("外科系统");
        }
        item.setNum(num);
        item.setSameNum(sameNum);
        item.setAverageValue(averageValue);
        item.setTotleValue(totleValue);
        item.setSameTotleValue(sameTotleValue);
        item.setLastYearAverageValue(lastYearAverageValue);
        return item;
    }

    /**
     * 根据限制数量重组统计结果
     *
     * @param averageList
     * @param limitCount
     * @return
     */
    public List<AverageStatisticsDTO> getLimitAverageList(List<AverageStatisticsDTO> averageList, Integer limitCount) {
        List<AverageStatisticsDTO> retAverageList = Lists.newLinkedList();
        if (averageList.size() <= limitCount) {
            retAverageList = com.lantone.common.util.BeanUtil.listCopyTo(averageList, AverageStatisticsDTO.class);
        } else {
            retAverageList = averageList.subList(0, limitCount - 2);
            List<AverageStatisticsDTO> otherList = averageList.subList(limitCount - 2, averageList.size());
            Double totleValue = otherList
                    .stream()
                    .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add).doubleValue();
            Double sameTotleValue = otherList
                    .stream()
                    .map(i -> BigDecimal.valueOf(i.getSameTotleValue()))
                    .reduce(BigDecimal.ZERO, BigDecimal::add).doubleValue();
            Integer num = otherList
                    .stream()
                    .map(AverageStatisticsDTO::getNum)
                    .reduce(0, Integer::sum);
            Integer sameNum = otherList
                    .stream()
                    .map(AverageStatisticsDTO::getSameNum)
                    .reduce(0, Integer::sum);
            Double averageValue = BigDecimal.valueOf(totleValue)
                    .divide(BigDecimal.valueOf(num), 2, RoundingMode.HALF_UP)
                    .doubleValue();
            Double lastYearAverageValue = 0d;
            if (0 != sameNum) {
                lastYearAverageValue = BigDecimal.valueOf(sameTotleValue)
                        .divide(BigDecimal.valueOf(sameNum), 2, RoundingMode.HALF_UP)
                        .doubleValue();
            }


            AverageStatisticsDTO retAverageStatistics = new AverageStatisticsDTO();
            retAverageStatistics.setName("其他");
            retAverageStatistics.setNum(num);
            retAverageStatistics.setSameNum(sameNum);
            retAverageStatistics.setSameTotleValue(sameTotleValue);
            retAverageStatistics.setLastYearAverageValue(lastYearAverageValue);
            retAverageStatistics.setAverageValue(averageValue);
            retAverageStatistics.setTotleValue(totleValue);
            retAverageList.add(retAverageStatistics);
        }
        return retAverageList;
    }

    /**
     * 增加全院记录
     *
     * @param records
     * @return
     */
    public AverageStatisticsDTO getGlobleAverageTitle(List<AverageStatisticsDTO> records) {
        DecimalFormat df = new DecimalFormat("#0.00");
        AverageStatisticsDTO item = new AverageStatisticsDTO();
        if (ListUtil.isEmpty(records)) {
            return null;
        }

        //数量
        Integer num = records
                .stream()
                .map(AverageStatisticsDTO::getNum)
                .reduce(0, Integer::sum);

        //同期数量
        Integer sameNum = records
                .stream()
                .map(AverageStatisticsDTO::getSameNum)
                .reduce(0, Integer::sum);

        Double totleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        Double sameTotleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getSameTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        Double averageValue = 0d;
        if (0 != num) {
            averageValue = BigDecimal.valueOf(totleValue)
                    .divide(BigDecimal.valueOf(num), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }
        Double lastYearAverageValue = 0d;
        if (0 != sameNum) {
            lastYearAverageValue = BigDecimal.valueOf(sameTotleValue)
                    .divide(BigDecimal.valueOf(sameNum), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }

        item.setName("全院");
        item.setNum(num);
        item.setSameNum(sameNum);
        item.setAverageValue(averageValue);
        item.setTotleValue(totleValue);
        item.setSameTotleValue(sameTotleValue);
        item.setLastYearAverageValue(lastYearAverageValue);
        return item;
    }


    /**
     * 增加全院记录
     *
     * @param records
     * @return
     */
    public LevelStatisticsTZDTO getGlobleRecords(List<LevelStatisticsTZDTO> records, FilterOrderVO filterOrderVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        LevelStatisticsTZDTO item = new LevelStatisticsTZDTO();
        if (ListUtil.isEmpty(records)) {
            return null;
        }

        //缺陷总数
        Integer entryNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getEntryNum)
                .reduce(0, Integer::sum);
        if (entryNum == null || entryNum == 0) {
            return null;
        }
        //同比总数
        Integer sameNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSameNum)
                .reduce(0, Integer::sum);

        //质控病历数
        Integer mrNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getMrNum)
                .reduce(0, Integer::sum);
        //同比病历数
        Integer sameMrNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSameMrNum)
                .reduce(0, Integer::sum);

        //质控评分
        Double totleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();
        //同比质控评分
        Double sameTotleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getSameTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        //质控平均分
        Double averageValue = BigDecimal.valueOf(totleValue)
                .divide(BigDecimal.valueOf(mrNum), 2, RoundingMode.HALF_UP)
                .doubleValue();

        Double sameAvgValue = 0d;
        //同比平均分
        if (0d != sameTotleValue) {
            sameAvgValue = BigDecimal.valueOf(sameTotleValue)
                    .divide(BigDecimal.valueOf(sameMrNum), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }


        //甲级病历数
        Integer firstLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getFirstLevelNum)
                .reduce(0, Integer::sum);

        //同比甲级病历数
        Integer sameFirstLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSameFirstLevelNum)
                .reduce(0, Integer::sum);


        //乙级病历数
        Integer secondLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSecondLevelNum)
                .reduce(0, Integer::sum);

        //同比乙级病历数
        Integer sameSecondLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSameSecondLevelNum)
                .reduce(0, Integer::sum);


        //丙级病历数
        Integer thirdLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getThirdLevelNum)
                .reduce(0, Integer::sum);

        //同比丙级病历数
        Integer sameThirdLevelNum = records
                .stream()
                .map(LevelStatisticsTZDTO::getSameThirdLevelNum)
                .reduce(0, Integer::sum);


        //甲级病历占比
        Double firstLevelPercent = BigDecimal.valueOf(firstLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String firstLevelPercentStr
                = df.format(BigDecimal.valueOf(firstLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        //同比甲级病历占比
        Double sameFirstLevelPercent = 0d;
        String sameFirstLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameFirstLevelPercent = BigDecimal.valueOf(sameFirstLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameFirstLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameFirstLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }


        //乙级病历占比
        Double secondLevelPercent = BigDecimal.valueOf(secondLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String secondLevelPercentStr
                = df.format(BigDecimal.valueOf(secondLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";

        //同比乙级病历占比
        Double sameSecondLevelPercent = 0d;
        String sameSecondLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameSecondLevelPercent = BigDecimal.valueOf(sameSecondLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameSecondLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameSecondLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }
        //丙级病历占比
        Double thirdLevelPercent = BigDecimal.valueOf(thirdLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String thirdLevelPercentStr
                = df.format(BigDecimal.valueOf(thirdLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";

        //同比丙级病历占比
        Double sameThirdLevelPercent = 0d;
        String sameThirdLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameThirdLevelPercent = BigDecimal.valueOf(sameThirdLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameThirdLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameThirdLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }
        if (StringUtil.isBlank(filterOrderVO.getDeptClass()) || filterOrderVO.getDeptClass().equals("全院")) {
            item.setDeptName("全院");
        } else if ("内科".equals(filterOrderVO.getDeptClass())) {
            item.setDeptClassId("01");
            item.setDeptClass("内科");
            item.setDeptName("全部科室");
        } else if ("外科".equals(filterOrderVO.getDeptClass())) {
            item.setDeptClassId("1030000");
            item.setDeptClass("外科");
            item.setDeptName("全部科室");
        }
        item.setEntryNum(entryNum);
        item.setSameNum(sameNum);
        item.setMrNum(mrNum);
        item.setSameMrNum(sameMrNum);
        item.setTotleValue(totleValue);
        item.setSameTotleValue(sameTotleValue);
        item.setAverageValue(averageValue);
        item.setSameAvgValue(sameAvgValue);
        item.setFirstLevelNum(firstLevelNum);
        item.setSameFirstLevelNum(sameFirstLevelNum);
        item.setFirstLevelPercent(firstLevelPercent);
        item.setSameFirstLevelPercent(sameFirstLevelPercent);
        item.setFirstLevelPercentStr(firstLevelPercentStr);
        item.setSameFirstLevelPercentStr(sameFirstLevelPercentStr);
        item.setSecondLevelNum(secondLevelNum);
        item.setSameSecondLevelNum(sameSecondLevelNum);
        item.setSecondLevelPercent(secondLevelPercent);
        item.setSameSecondLevelPercent(sameSecondLevelPercent);
        item.setSecondLevelPercentStr(secondLevelPercentStr);
        item.setSameSecondLevelPercentStr(sameSecondLevelPercentStr);
        item.setThirdLevelNum(thirdLevelNum);
        item.setSameThirdLevelNum(sameThirdLevelNum);
        item.setThirdLevelPercent(thirdLevelPercent);
        item.setSameThirdLevelPercent(sameThirdLevelPercent);
        item.setThirdLevelPercentStr(thirdLevelPercentStr);
        item.setSameThirdLevelPercentStr(sameThirdLevelPercentStr);
        return item;
    }


    /**
     * 增加全院记录
     *
     * @param records
     * @return
     */
    public LevelStatisticsDTO getGlobleTitle(List<LevelStatisticsDTO> records) {
        DecimalFormat df = new DecimalFormat("#0.00");
        LevelStatisticsDTO item = new LevelStatisticsDTO();
        if (ListUtil.isEmpty(records)) {
            return null;
        }

        //缺陷总数
        Integer entryNum = records
                .stream()
                .map(LevelStatisticsDTO::getEntryNum)
                .reduce(0, Integer::sum);
        if (entryNum == null || entryNum == 0) {
            return null;
        }
        //同比总数
        Integer sameNum = records
                .stream()
                .map(LevelStatisticsDTO::getSameNum)
                .reduce(0, Integer::sum);

        //质控病历数
        Integer mrNum = records
                .stream()
                .map(LevelStatisticsDTO::getMrNum)
                .reduce(0, Integer::sum);
        //同比病历数
        Integer sameMrNum = records
                .stream()
                .map(LevelStatisticsDTO::getSameMrNum)
                .reduce(0, Integer::sum);

        //质控评分
        Double totleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();
        //同比质控评分
        Double sameTotleValue = records
                .stream()
                .map(i -> BigDecimal.valueOf(i.getSameTotleValue()))
                .reduce(BigDecimal.ZERO, BigDecimal::add)
                .doubleValue();

        //质控平均分
        Double averageValue = BigDecimal.valueOf(totleValue)
                .divide(BigDecimal.valueOf(mrNum), 2, RoundingMode.HALF_UP)
                .doubleValue();

        Double sameAvgValue = 0d;
        //同比平均分
        if (0d != sameTotleValue) {
            sameAvgValue = BigDecimal.valueOf(sameTotleValue)
                    .divide(BigDecimal.valueOf(sameMrNum), 2, RoundingMode.HALF_UP)
                    .doubleValue();
        }


        //甲级病历数
        Integer firstLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getFirstLevelNum)
                .reduce(0, Integer::sum);

        //同比甲级病历数
        Integer sameFirstLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getSameFirstLevelNum)
                .reduce(0, Integer::sum);


        //乙级病历数
        Integer secondLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getSecondLevelNum)
                .reduce(0, Integer::sum);

        //同比乙级病历数
        Integer sameSecondLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getSameSecondLevelNum)
                .reduce(0, Integer::sum);


        //丙级病历数
        Integer thirdLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getThirdLevelNum)
                .reduce(0, Integer::sum);

        //同比丙级病历数
        Integer sameThirdLevelNum = records
                .stream()
                .map(LevelStatisticsDTO::getSameThirdLevelNum)
                .reduce(0, Integer::sum);


        //甲级病历占比
        Double firstLevelPercent = BigDecimal.valueOf(firstLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String firstLevelPercentStr
                = df.format(BigDecimal.valueOf(firstLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        //同比甲级病历占比
        Double sameFirstLevelPercent = 0d;
        String sameFirstLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameFirstLevelPercent = BigDecimal.valueOf(sameFirstLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameFirstLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameFirstLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }


        //乙级病历占比
        Double secondLevelPercent = BigDecimal.valueOf(secondLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String secondLevelPercentStr
                = df.format(BigDecimal.valueOf(secondLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";

        //同比乙级病历占比
        Double sameSecondLevelPercent = 0d;
        String sameSecondLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameSecondLevelPercent = BigDecimal.valueOf(sameSecondLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameSecondLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameSecondLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }
        //丙级病历占比
        Double thirdLevelPercent = BigDecimal.valueOf(thirdLevelNum)
                .divide(BigDecimal.valueOf(mrNum), 4, RoundingMode.HALF_UP)
                .doubleValue();
        String thirdLevelPercentStr
                = df.format(BigDecimal.valueOf(thirdLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";

        //同比丙级病历占比
        Double sameThirdLevelPercent = 0d;
        String sameThirdLevelPercentStr = "0.00%";
        if (0 != sameFirstLevelNum) {
            sameThirdLevelPercent = BigDecimal.valueOf(sameThirdLevelNum)
                    .divide(BigDecimal.valueOf(sameMrNum), 4, RoundingMode.HALF_UP)
                    .doubleValue();
            sameThirdLevelPercentStr
                    = df.format(BigDecimal.valueOf(sameThirdLevelPercent).multiply(BigDecimal.valueOf(100))) + "%";
        }
        item.setDeptName("全院");
        item.setEntryNum(entryNum);
        item.setSameNum(sameNum);
        item.setMrNum(mrNum);
        item.setSameMrNum(sameMrNum);
        item.setTotleValue(totleValue);
        item.setSameTotleValue(sameTotleValue);
        item.setAverageValue(averageValue);
        item.setSameAvgValue(sameAvgValue);
        item.setFirstLevelNum(firstLevelNum);
        item.setSameFirstLevelNum(sameFirstLevelNum);
        item.setFirstLevelPercent(firstLevelPercent);
        item.setSameFirstLevelPercent(sameFirstLevelPercent);
        item.setFirstLevelPercentStr(firstLevelPercentStr);
        item.setSameFirstLevelPercentStr(sameFirstLevelPercentStr);
        item.setSecondLevelNum(secondLevelNum);
        item.setSameSecondLevelNum(sameSecondLevelNum);
        item.setSecondLevelPercent(secondLevelPercent);
        item.setSameSecondLevelPercent(sameSecondLevelPercent);
        item.setSecondLevelPercentStr(secondLevelPercentStr);
        item.setSameSecondLevelPercentStr(sameSecondLevelPercentStr);
        item.setThirdLevelNum(thirdLevelNum);
        item.setSameThirdLevelNum(sameThirdLevelNum);
        item.setThirdLevelPercent(thirdLevelPercent);
        item.setSameThirdLevelPercent(sameThirdLevelPercent);
        item.setThirdLevelPercentStr(thirdLevelPercentStr);
        item.setSameThirdLevelPercentStr(sameThirdLevelPercentStr);
        return item;
    }

    public void BigDecimalMethod(LevelStatisticsDTO record) {
        BigDecimal entrySameBigDecimal = getSubtract(new BigDecimal(record.getEntryNum()), new BigDecimal(record.getSameNum() == null ? 0 : record.getSameNum()));

        if (entrySameBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setEntrySameNum(1);
        } else if (entrySameBigDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setEntrySameNum(0);
        } else {
            record.setEntrySameNum(-1);
        }

        BigDecimal sameMrNumBigDecimal = getSubtract(new BigDecimal(record.getMrNum()), new BigDecimal(record.getSameMrNum() == null ? 0 : record.getSameMrNum()));
        if (sameMrNumBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setMrSameNum(1);
        } else if (sameMrNumBigDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setMrSameNum(0);
        } else {
            record.setMrSameNum(-1);
        }

        BigDecimal sameAvgValueDecimal = getSubtract(new BigDecimal(record.getAverageValue()), new BigDecimal(record.getSameAvgValue() == null ? 0 : record.getSameAvgValue()));
        if (sameAvgValueDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setAvSameValue(1);
        } else if (sameAvgValueDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setAvSameValue(0);
        } else {
            record.setAvSameValue(-1);
        }

        BigDecimal SameFirstLevelBigDecimal = getSubtract(new BigDecimal(record.getFirstLevelNum()), new BigDecimal(record.getSameFirstLevelNum() == null ? 0 : record.getSameFirstLevelNum()));
        if (SameFirstLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setFirSameLevelNum(1);
        } else if (SameFirstLevelBigDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setFirSameLevelNum(0);
        } else {
            record.setFirSameLevelNum(-1);
        }

        BigDecimal SameSecondLevelBigDecimal = getSubtract(new BigDecimal(record.getSecondLevelNum()), new BigDecimal(record.getSameSecondLevelNum() == null ? 0 : record.getSameSecondLevelNum()));
        if (SameSecondLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setSecSameLevelNum(1);
        } else if (SameSecondLevelBigDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setSecSameLevelNum(0);
        } else {
            record.setSecSameLevelNum(-1);
        }

        BigDecimal SameThirdLevelBigDecimal = getSubtract(new BigDecimal(record.getThirdLevelNum()), new BigDecimal(record.getSameThirdLevelNum() == null ? 0 : record.getSameThirdLevelNum()));
        if (SameThirdLevelBigDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setThiSameLevelNum(1);
        } else if (SameThirdLevelBigDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setThiSameLevelNum(0);
        } else {
            record.setThiSameLevelNum(-1);
        }

        BigDecimal SameFirstLevelPeDecimal = getSubtract(new BigDecimal(record.getFirstLevelPercent()), new BigDecimal(record.getSameFirstLevelPercent() == null ? 0 : record.getSameFirstLevelPercent()));
        if (SameFirstLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setFirSameLevelPercent(1);
        } else if (SameFirstLevelPeDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setFirSameLevelPercent(0);
        } else {
            record.setFirSameLevelPercent(-1);
        }

        BigDecimal SameSecondLevelPeDecimal = getSubtract(new BigDecimal(record.getSecondLevelPercent()), new BigDecimal(record.getSameSecondLevelPercent() == null ? 0 : record.getSameSecondLevelPercent()));
        if (SameSecondLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setSecSameLevelPercent(1);
        } else if (SameSecondLevelPeDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setSecSameLevelPercent(0);
        } else {
            record.setSecSameLevelPercent(-1);
        }

        BigDecimal SameThirdLevelPeDecimal = getSubtract(new BigDecimal(record.getThirdLevelPercent()), new BigDecimal(record.getSameThirdLevelPercent() == null ? 0 : record.getSameThirdLevelPercent()));
        if (SameThirdLevelPeDecimal.compareTo(BigDecimal.ZERO) > 0) {
            record.setThiSameLevelPercent(1);
        } else if (SameThirdLevelPeDecimal.compareTo(BigDecimal.ZERO) == 0) {
            record.setThiSameLevelPercent(0);
        } else {
            record.setThiSameLevelPercent(-1);
        }
    }

    /**
     * 差值计算
     *
     * @param bigDecimalOne
     * @param bigDecimalTwo
     * @return
     */
    private BigDecimal getSubtract(BigDecimal bigDecimalOne, BigDecimal bigDecimalTwo) {
        BigDecimal newBigDecimal = new BigDecimal(0);
        if (null == bigDecimalOne && null != bigDecimalTwo) {
            newBigDecimal = newBigDecimal.subtract(bigDecimalTwo);
        } else if (null != bigDecimalOne && null == bigDecimalTwo) {
            newBigDecimal = bigDecimalOne.subtract(newBigDecimal);
        } else if (null != bigDecimalOne && null != bigDecimalTwo) {
            newBigDecimal = bigDecimalOne.subtract(bigDecimalTwo);
        }
        return newBigDecimal;
    }
}