package com.lantone.report.facade;

import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.lantone.common.dto.report.ColumnDTO;
import com.lantone.common.dto.report.UnModifyMRDTO;
import com.lantone.common.dto.report.UnModifyMRDetailDTO;
import com.lantone.common.exception.ApiException;
import com.lantone.common.util.ClassUtil;
import com.lantone.common.util.EntityUtil;
import com.lantone.common.util.ExcelUtils;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.ObjectUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.common.vo.report.FilterUnModifyMRVO;
import com.lantone.dblayermbg.entity.HospitalSet;
import com.lantone.dblayermbg.entity.report.CasesEntry;
import com.lantone.dblayermbg.facade.HospitalSetFacade;
import com.lantone.dblayermbg.facade.report.BehospitalInfoFacade;
import com.lantone.dblayermbg.facade.report.CasesEntryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


@Component
public class TimelinessFacade {

    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;
    @Autowired
    private HospitalSetFacade hospitalSetFacade;
    @Autowired
    private CasesEntryFacade casesEntryFacade;

    /**
     * 未整改病历统计
     *
     * @param filterUnModifyMRVO
     * @return
     */
    public UnModifyMRDTO unModifyMRStatistics(FilterUnModifyMRVO filterUnModifyMRVO) {
        UnModifyMRDTO unModifyMRDTO = new UnModifyMRDTO();
        List<Object> retList = Lists.newLinkedList();
        filterFacade.filterUnModifyMRVOSet(filterUnModifyMRVO);

        //colums
        QueryWrapper<HospitalSet> hospitalSetQueryWrapper = new QueryWrapper<>();
        hospitalSetQueryWrapper.eq("is_deleted", 'N')
                .eq("hospital_id", filterUnModifyMRVO.getHospitalId())
                .eq("code", "unmodify_mr_entry");
        HospitalSet hospitalSet = hospitalSetFacade.getOne(hospitalSetQueryWrapper);
        if (hospitalSet == null || StringUtil.isBlank(hospitalSet.getVal())) {
            return unModifyMRDTO;
        }

        //表头生成
        List<String> columnSet = Arrays.asList(hospitalSet.getVal().split(","));
        List<ColumnDTO> columns = Lists.newLinkedList();
        Integer orderNo = 1;
        ColumnDTO columnDeptId = new ColumnDTO();
        columnDeptId.setOrderNo(orderNo);
        columnDeptId.setFieldName("deptId");
        columnDeptId.setColumnName("科室编码");
        columnDeptId.setIsShow(0);
        columns.add(columnDeptId);
        orderNo++;
        ColumnDTO columnDeptName = new ColumnDTO();
        columnDeptName.setOrderNo(orderNo);
        columnDeptName.setFieldName("deptName");
        columnDeptName.setColumnName("科室");
        columnDeptName.setIsShow(1);
        columns.add(columnDeptName);
        orderNo++;
        List<Long> casesEntryIds = Lists.newArrayList();
        for (String valueStr : columnSet) {
            if (StringUtil.isBlank(valueStr)) {
                continue;
            }
            String[] keyValue = valueStr.split("--");
            if (keyValue != null || keyValue.length > 1) {
                casesEntryIds.add(Long.valueOf(keyValue[0]));
                ColumnDTO columnId = new ColumnDTO();
                columnId.setOrderNo(orderNo);
                columnId.setId(Long.valueOf(keyValue[0]));
                columnId.setFieldName("entry_" + keyValue[0] + "_id");
                columnId.setColumnName(keyValue[1] + "id");
                columnId.setIsShow(0);
                columns.add(columnId);
                orderNo++;
                ColumnDTO columnName = new ColumnDTO();
                columnName.setOrderNo(orderNo);
                columnName.setId(Long.valueOf(keyValue[0]));
                columnName.setFieldName("entry_" + keyValue[0] + "_name");
                columnName.setColumnName(keyValue[1] + "name");
                columnName.setIsShow(0);
                columns.add(columnName);
                orderNo++;
                ColumnDTO columnNum = new ColumnDTO();
                columnNum.setOrderNo(orderNo);
                columnNum.setId(Long.valueOf(keyValue[0]));
                columnNum.setFieldName("entry_" + keyValue[0] + "_num");
                columnNum.setColumnName(keyValue[1]);
                columnNum.setIsShow(1);
                columns.add(columnNum);
                orderNo++;
            }
        }
        //条目未维护
        if (ListUtil.isEmpty(casesEntryIds)) {
            return unModifyMRDTO;
        }

        //casesEntryIds
        filterUnModifyMRVO.setCasesEntryIds(casesEntryIds);
        QueryWrapper<CasesEntry> qcCasesEntryQueryWrapper = new QueryWrapper<>();
        qcCasesEntryQueryWrapper.eq("is_deleted", "N")
                .in("id", casesEntryIds);
        List<CasesEntry> casesEntryList = casesEntryFacade.list(qcCasesEntryQueryWrapper);
        // 实体类中将实体类转化为map形式，其中条目id对应目标条目名称
        Map<Long, String> basCasesEntryMap = EntityUtil.makeMapWithKeyValue(casesEntryList, "id", "name");

        List<UnModifyMRDetailDTO> records = behospitalInfoFacade.getBaseMapper().unModifyMRStatistics(filterUnModifyMRVO);
        //以科室分组，将符合科室的每条数据以map形式存储
        Map<String, List<UnModifyMRDetailDTO>> deptMap = EntityUtil.makeEntityListMap(records, "deptName");
        //科室对应科室id
        Map<String, String> deptInfoMap = EntityUtil.makeMapWithKeyValue(records, "deptName", "deptId");
        Map<String, Map<Long, UnModifyMRDetailDTO>> deptEntryMap = new LinkedHashMap<>();
        for (Map.Entry<String, List<UnModifyMRDetailDTO>> entry : deptMap.entrySet()) {
            Map<Long, UnModifyMRDetailDTO> entryMap = EntityUtil.makeEntityMap(entry.getValue(), "casesEntryId");
            deptEntryMap.put(entry.getKey(), entryMap);
        }

        try {
            for (Map.Entry<String, Map<Long, UnModifyMRDetailDTO>> entry : deptEntryMap.entrySet()) {
                Object obj = new Object();
                HashMap addMap = new HashMap();
                HashMap addValMap = new HashMap();

                addMap.put("deptName", Class.forName("java.lang.String"));
                addValMap.put("deptName", entry.getKey());
                addMap.put("deptId", Class.forName("java.lang.String"));
                addValMap.put("deptId", deptInfoMap.get(entry.getKey()));

                for (ColumnDTO item : columns) {
                    if (item.getFieldName().lastIndexOf("_id") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.Long"));
                        if (item.getId() != null) {
                            addValMap.put(item.getFieldName(), item.getId());
                        } else {
                            addValMap.put(item.getFieldName(), null);
                        }
                    } else if (item.getFieldName().lastIndexOf("_name") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.String"));
                        if (basCasesEntryMap.containsKey(item.getId())) {
                            addValMap.put(item.getFieldName(), basCasesEntryMap.get(item.getId()));
                        } else {
                            addValMap.put(item.getFieldName(), "");
                        }
                    } else if (item.getFieldName().lastIndexOf("_num") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.Integer"));
                        if (entry.getValue().containsKey(item.getId())) {
                            addValMap.put(item.getFieldName(), entry.getValue().get(item.getId()).getCasesEntryNum());
                        } else {
                            addValMap.put(item.getFieldName(), 0);
                        }
                    }
                }
                obj = new ClassUtil().dynamicClass(obj, addMap, addValMap);
                retList.add(obj);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        //排序
        Collections.sort(retList, new Comparator<Object>() {
            @Override
            public int compare(Object o1, Object o2) {

                if (StringUtil.isNotBlank(filterUnModifyMRVO.getAsc())) {
                    //升序
                    Object o1Val = ObjectUtil.getValueByKey(o1, filterUnModifyMRVO.getAsc());
                    Object o2Val = ObjectUtil.getValueByKey(o2, filterUnModifyMRVO.getAsc());
                    if (filterUnModifyMRVO.getAsc().lastIndexOf("_num") > 1) {
                        return ((Integer) o1Val).compareTo((Integer) o2Val);
                    }
                    if (filterUnModifyMRVO.getAsc().lastIndexOf("_id") > 1) {
                        return ((Long) o1Val).compareTo((Long) o2Val);
                    } else {
                        return ((String) o1Val).compareTo((String) o2Val);
                    }
                } else if (StringUtil.isNotBlank(filterUnModifyMRVO.getDesc())) {
                    //降序
                    Object o1Val = ObjectUtil.getValueByKey(o1, filterUnModifyMRVO.getDesc());
                    Object o2Val = ObjectUtil.getValueByKey(o2, filterUnModifyMRVO.getDesc());
                    if (filterUnModifyMRVO.getDesc().lastIndexOf("_num") > 1) {
                        return ((Integer) o2Val).compareTo((Integer) o1Val);
                    }
                    if (filterUnModifyMRVO.getDesc().lastIndexOf("_id") > 1) {
                        return ((Long) o2Val).compareTo((Long) o1Val);
                    } else {
                        return ((String) o2Val).compareTo((String) o1Val);
                    }
                } else {
                    return 0;
                }
            }
        });
        //增加全院数据
        if (ListUtil.isNotEmpty(retList) &&
                (StringUtil.isBlank(filterUnModifyMRVO.getDeptName())
                        || filterUnModifyMRVO.getDeptName().equals("全院"))) {
            try {

                Object globleObj = new Object();
                HashMap addMap = new HashMap();
                HashMap addValMap = new HashMap();
                addMap.put("deptId", Class.forName("java.lang.String"));
                addValMap.put("deptId", "");
                addMap.put("deptName", Class.forName("java.lang.String"));
                addValMap.put("deptName", "全院");

                for (ColumnDTO item : columns) {
                    if (item.getFieldName().lastIndexOf("_id") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.Long"));
                        if (item.getId() != null) {
                            addValMap.put(item.getFieldName(), item.getId());
                        } else {
                            addValMap.put(item.getFieldName(), null);
                        }

                    } else if (item.getFieldName().lastIndexOf("_name") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.String"));
                        if (basCasesEntryMap.containsKey(item.getId())) {
                            addValMap.put(item.getFieldName(), basCasesEntryMap.get(item.getId()));
                        } else {
                            addValMap.put(item.getFieldName(), "");
                        }

                    } else if (item.getFieldName().lastIndexOf("_num") > 0) {
                        addMap.put(item.getFieldName(), Class.forName("java.lang.Integer"));
                        Integer entryNum = retList
                                .stream()
                                .map(i -> ObjectUtil.getValueByKey(i, item.getFieldName()))
                                .filter(i -> i != null)
                                .map(i -> (Integer) i)
                                .reduce(0, Integer::sum);
                        addValMap.put(item.getFieldName(), entryNum);
                    }
                }
                globleObj = new ClassUtil().dynamicClass(globleObj, addMap, addValMap);
                retList.add(0, globleObj);

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        unModifyMRDTO.setColumns(columns);
        unModifyMRDTO.setData(retList);
        return unModifyMRDTO;
    }

    /**
     * 未整改病历统计导出
     *
     * @param response
     * @param filterUnModifyMRVO
     */
    public void unModifyMRStatisticsExport(HttpServletResponse response, FilterUnModifyMRVO filterUnModifyMRVO) {
        filterFacade.filterUnModifyMRVOSet(filterUnModifyMRVO);
        List<ExcelExportEntity> colList = Lists.newLinkedList();
        ExcelExportEntity deptNameCol = new ExcelExportEntity("科室", "deptName");
        deptNameCol.setWidth(50);
        colList.add(deptNameCol);

        QueryWrapper<HospitalSet> hospitalSetQueryWrapper = new QueryWrapper<>();
        hospitalSetQueryWrapper.eq("is_deleted", 'N')
                .eq("hospital_id", filterUnModifyMRVO.getHospitalId())
                .eq("code", "unmodify_mr_entry");
        HospitalSet hospitalSet = hospitalSetFacade.getOne(hospitalSetQueryWrapper);
        //缺陷条目未维护
        if (hospitalSet == null || StringUtil.isBlank(hospitalSet.getVal())) {
            throw new ApiException("未整改病历条目未设置");
        }

        List<String> columnSet = Arrays.asList(hospitalSet.getVal().split(","));
        for (String valueStr : columnSet) {
            if (StringUtil.isBlank(valueStr)) {
                continue;
            }
            String[] keyValue = valueStr.split("--");
            if (keyValue != null || keyValue.length > 1) {
                ExcelExportEntity entryCol = new ExcelExportEntity(keyValue[1], "entry_" + keyValue[0] + "_num");
                entryCol.setWidth(20);
                colList.add(entryCol);
            }
        }
        UnModifyMRDTO unModifyMRDTO = this.unModifyMRStatistics(filterUnModifyMRVO);
        List<Object> data = Lists.newLinkedList();
        if (unModifyMRDTO != null && ListUtil.isNotEmpty(unModifyMRDTO.getData())) {
            data = unModifyMRDTO.getData();
        }
        String fileName = "未整改病历统计.xls";
        ExcelUtils.exportExcelDynamicCol(colList, data, null, "sheet1", fileName, response);
    }
}