package com.lantone.report.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.EntryNumDTO;
import com.lantone.common.dto.report.EntryNumGroupDTO;
import com.lantone.common.dto.report.EntryStatisticsDTO;
import com.lantone.common.dto.report.NumDTO;
import com.lantone.common.vo.report.EntryStatisticsVO;
import com.lantone.common.vo.report.FilterPageVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.report.facade.EntryCaseFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 条目缺陷管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "条目缺陷管理统计相关API", tags = { "条目缺陷管理统计相关API" })
@SuppressWarnings("unchecked")
public class EntryCaseController {
    @Autowired
    private EntryCaseFacade entryCaseFacade;

    /**
     * 条目缺陷占比-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-控制台[by:zhaops]")
    @PostMapping("/entryCountGroupByEntry")
    public CommonResult<Map<String, Object>> entryCountGroupByEntry(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(entryCaseFacade.entryCountGroupByEntry(filterVO));
    }

    /**
     * 条目缺陷占比-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/entryCountGroupByEntry")
    public CommonResult<Map<String, Object>> runEntryCountGroupByEntry(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(entryCaseFacade.entryCountGroupByEntry(filterVO));
    }

    /**
     * 条目缺陷占比-报表
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-报表[by:zhaops]")
    @PostMapping("/entryGroupByEntryInnerPage")
    public CommonResult<IPage<EntryNumDTO>> entryGroupByEntryInnerPage(@RequestBody @Valid FilterPageVO filterPageVO) {
        return CommonResult.success(entryCaseFacade.entryGroupByEntryInnerPage(filterPageVO));
    }

    /**
     * 条目缺陷占比-报表-运行质控
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-报表-运行质控[by:zhaops]")
    @PostMapping("/run/entryGroupByEntryInnerPage")
    public CommonResult<IPage<EntryNumDTO>> runEntryGroupByEntryInnerPage(@RequestBody @Valid FilterPageVO filterPageVO) {
        return CommonResult.success(entryCaseFacade.entryGroupByEntryInnerPage(filterPageVO));
    }

    /**
     * 条目缺陷占比-报表导出
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-报表导出[by:gaodm]")
    @PostMapping("/entryGroupByEntryExport")
    public void entryGroupByEntryExport(HttpServletResponse response, @RequestBody @Valid FilterPageVO filterPageVO) {
        entryCaseFacade.entryGroupByEntryExport(response, filterPageVO);
    }

    /**
     * 条目缺陷占比-报表导出-运行质控
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷占比-报表导出-运行质控[by:gaodm]")
    @PostMapping("/run/entryGroupByEntryExport")
    public void runEntryGroupByEntryExport(HttpServletResponse response, @RequestBody @Valid FilterPageVO filterPageVO) {
        entryCaseFacade.entryGroupByEntryExport(response, filterPageVO);
    }

    /**
     * 单项否决缺陷占比-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "单项否决缺陷占比-控制台[by:zhaops]")
    @PostMapping("/entryRejectPercent")
    public CommonResult<List<EntryNumDTO>> entryRejectPercent(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(entryCaseFacade.entryRejectPercent(filterVO));
    }

    /**
     * 单项否决缺陷占比-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "单项否决缺陷占比-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/entryRejectPercent")
    public CommonResult<List<EntryNumDTO>> runEntryRejectPercent(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(entryCaseFacade.entryRejectPercent(filterVO));
    }

    /**
     * 入院记录24h内未完成
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "入院记录24h内未完成-控制台[by:zhaops]")
    @PostMapping("/casesEntryStatisticsById")
    public CommonResult<List<NumDTO>> casesEntryStatisticsById(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(entryCaseFacade.casesEntryStatisticsById(filterVO));
    }

    /**
     * 关键条目缺陷占比-报表
     *
     * @param entryStatisticsVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-报表[by:zhaops]")
    @PostMapping("/entryStatistics")
    public CommonResult<List<EntryStatisticsDTO>> entryStatistics(@RequestBody @Valid EntryStatisticsVO entryStatisticsVO) {
        return CommonResult.success(entryCaseFacade.entryStatistics(entryStatisticsVO));
    }

    /**
     * 关键条目缺陷占比-报表-运行质控
     *
     * @param entryStatisticsVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-报表-运行质控[by:zhaops]")
    @PostMapping("/run/entryStatistics")
    public CommonResult<List<EntryStatisticsDTO>> runEntryStatistics(@RequestBody @Valid EntryStatisticsVO entryStatisticsVO) {
        return CommonResult.success(entryCaseFacade.entryStatistics(entryStatisticsVO));
    }

    /**
     * 关键条目缺陷占比-报表导出
     *
     * @param entryStatisticsVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-报表导出[by:gaodm]")
    @PostMapping("/entryStatisticsExport")
    public void entryStatisticsExport(HttpServletResponse response, @RequestBody @Valid EntryStatisticsVO entryStatisticsVO) {
        entryCaseFacade.entryStatisticsExport(response, entryStatisticsVO);
    }

    /**
     * 关键条目缺陷占比-报表导出-运行质控
     *
     * @param entryStatisticsVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-报表导出-运行质控[by:gaodm]")
    @PostMapping("/run/entryStatisticsExport")
    public void runEntryStatisticsExport(HttpServletResponse response, @RequestBody @Valid EntryStatisticsVO entryStatisticsVO) {
        entryCaseFacade.entryStatisticsExport(response, entryStatisticsVO);
    }

    /**
     * 关键条目缺陷占比-缺陷列表-报表
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-缺陷列表-报表[by:zhaops]")
    @PostMapping("/entryCountGroupByEntryPage")
    public CommonResult<IPage<EntryNumGroupDTO>> entryCountGroupByEntryPage(@RequestBody @Valid FilterPageVO filterPageVO) {
        return CommonResult.success(entryCaseFacade.entryCountGroupByEntryPage(filterPageVO));
    }

    /**
     * 关键条目缺陷占比-缺陷列表-报表导出
     *
     * @param filterPageVO
     * @return
     */
    @ApiOperation(value = "关键条目缺陷占比-缺陷列表-报表导出[by:gaodm]")
    @PostMapping("/entryCountGroupByEntryExport")
    public void entryCountGroupByEntryExport(HttpServletResponse response, @RequestBody @Valid FilterPageVO filterPageVO) {
        entryCaseFacade.entryCountGroupByEntryExport(response, filterPageVO);
    }
}
