package com.lantone.report.web;

import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.HomePageNumDTO;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.report.facade.HomePageInfoFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 病案首页管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "病案首页管理统计相关API", tags = { "病案首页管理统计相关API" })
@SuppressWarnings("unchecked")
public class HomePageController {
    @Autowired
    private HomePageInfoFacade homePageInfoFacade;

    /**
     * 病案首页病历统计-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病案首页病历统计-控制台[by:zhaops]")
    @PostMapping("/homePageMRCount")
    public CommonResult<Map<String, Object>> homePageMRCount(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(homePageInfoFacade.homePageMRCount(filterVO));
    }

    /**
     * 病案首页病历统计-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病案首页病历统计-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/homePageMRCount")
    public CommonResult<Map<String, Object>> runHomePageMRCount(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(homePageInfoFacade.homePageMRCount(filterVO));
    }


    /**
     * 病案首页合格率占比-报表
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "病案首页合格率占比-报表[by:zhaops]")
    @PostMapping("/homePageLevelStatistics")
    public CommonResult<List<HomePageNumDTO>> homePageLevelStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(homePageInfoFacade.homePageLevelStatistics(filterOrderVO));
    }

    /**
     * 病案首页合格率占比-报表-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "病案首页合格率占比-报表-运行质控[by:zhaops]")
    @PostMapping("/run/homePageLevelStatistics")
    public CommonResult<List<HomePageNumDTO>> runHomePageLevelStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(homePageInfoFacade.homePageLevelStatistics(filterOrderVO));
    }

    /**
     * 病案首页合格率占比-报表导出
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "病案首页合格率占比-报表导出[by:gaodm]")
    @PostMapping("/homePageLevelExport")
    public void homePageLevelExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        homePageInfoFacade.homePageLevelExport(response, filterOrderVO);
    }

    /**
     * 病案首页合格率占比-报表导出-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "病案首页合格率占比-报表导出-运行质控[by:gaodm]")
    @PostMapping("/run/homePageLevelExport")
    public void runHomePageLevelExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        homePageInfoFacade.homePageLevelExport(response, filterOrderVO);
    }
}
