package com.lantone.report.web;


import com.lantone.common.dto.RespDTO;
import com.lantone.common.dto.appeal.GetAppealInfoDTO;
import com.lantone.common.dto.appeal.GetReviewerDTO;
import com.lantone.common.vo.appeal.AddAppealInfoVO;
import com.lantone.common.vo.appeal.CancelAppealInfoVO;
import com.lantone.common.vo.appeal.GetAppealInfoVO;
import com.lantone.common.vo.appeal.GetAppealOperationTypeVO;
import com.lantone.common.vo.appeal.GetReviewerVO;
import com.lantone.common.vo.appeal.MedAppealInfoVO;
import com.lantone.dblayermbg.entity.appeal.WorkFlowNode;
import com.lantone.report.facade.MedAppealInfoManagementFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 申诉审核 前端控制器
 * </p>
 *
 * @author wangfeng
 * @since 2020-07-06
 */
@RestController
@RequestMapping("/medAppealInfoManage")
@Api(value = "申诉审核数据模块相关接口API", tags = {"申诉审核数据模块相关接口API"})
public class MedAppealInfoManagementController {
    @Autowired
    MedAppealInfoManagementFacade medAppealInfoManagementFacade;



    @ApiOperation(value = "申诉[by:songxl]",
            notes = "申诉")
    @PostMapping("/addAppealInfo")
    @Transactional
    public RespDTO<Boolean> addAppealInfo(@RequestBody @Valid AddAppealInfoVO addAppealInfoVO) {
        return RespDTO.onSuc(medAppealInfoManagementFacade.addAppealInfo(addAppealInfoVO));
    }

    @ApiOperation(value = "获取审核人[by:songxl]",
            notes = "获取审核人")
    @PostMapping("/getReviewer")
    public RespDTO<List<GetReviewerDTO>> getReviewer(@RequestBody GetReviewerVO getReviewerVO) {
        List<GetReviewerDTO> reviewers = medAppealInfoManagementFacade.getReviewer(getReviewerVO);
        return RespDTO.onSuc(reviewers);
    }

    @ApiOperation(value = "再次申诉获取操作类型[by:songxl]",
            notes = "再次申诉获取操作类型")
    @PostMapping("/getAppealOperationType")
    public RespDTO<Map<String, String>> getAppealOperationType(@RequestBody @Valid GetAppealOperationTypeVO getAppealOperationTypeVO) {
        return RespDTO.onSuc(medAppealInfoManagementFacade.getAppealOperationType(getAppealOperationTypeVO));
    }

    @ApiOperation(value = "撤回申诉[by:songxl]",
            notes = "撤回申诉")
    @PostMapping("/cancelAppealInfo")
    @Transactional
    public RespDTO<Boolean> cancelAppealInfo(@RequestBody @Valid CancelAppealInfoVO cancelAppealInfoVO) {
        return RespDTO.onSuc(medAppealInfoManagementFacade.cancelAppealInfo(cancelAppealInfoVO));
    }

    @ApiOperation(value = "申诉记录详情-查看[by:songxl]",
            notes = "申诉记录详情-查看")
    @PostMapping("/getAppealInfo")
    public RespDTO<GetAppealInfoDTO> getAppealInfo(@RequestBody GetAppealInfoVO getAppealInfoVO) {
        return RespDTO.onSuc(medAppealInfoManagementFacade.getAppealInfo(getAppealInfoVO));
    }
}
