package com.lantone.report.web;
import com.lantone.common.api.CommonResult;
import com.lantone.common.vo.report.FilterVO;
import com.lantone.common.vo.report.IndexTimeVO;
import com.lantone.report.facade.MrIndicatorFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 病案指标管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "病案指标管理统计相关API", tags = { "病案指标管理统计相关API" })
@SuppressWarnings("unchecked")
public class MrIndicatorController {
    @Autowired
    private MrIndicatorFacade mrIndicatorFacade;

    /**
     * 病案指标管理统计相关API
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病案指标管理统计相关API[by:zh]")
    @PostMapping("/medicalRecordIndicator")
    public CommonResult<List<Map<String,String>>> medicalRecordIndicator(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(mrIndicatorFacade.medicalRecordIndicator(filterVO));
    }

    /**
     * 病案指标后台维护接口
     * @param indexTimeVO
     * @return
     */
    @ApiOperation(value = "病案指标数据存储[by:cy]",
            notes = "startDate： 起始时间-样例[2021-01-01]请严格按照日历时间传参！！！ <br>" +
                    "endDate： 结束时间-样例[2021-01-01]请严格按照日历时间传参！！！<br>")
    @PostMapping("/saveMedicaIndicator")
    public CommonResult<Boolean> saveMedicaIndicator(@RequestBody @Valid IndexTimeVO indexTimeVO) {
        Boolean flag = mrIndicatorFacade.saveMedicaIndicator(indexTimeVO);
        return CommonResult.success(flag);
    }
}
