package com.lantone.report.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.QualityControlDTO;
import com.lantone.common.vo.report.BehospitalPageVO;
import com.lantone.report.facade.MrQcCaseFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;


/**
 * <p>
 * 病历质控报表管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "病历质控报表管理统计相关API", tags = { "病历质控报表管理统计相关API" })
@SuppressWarnings("unchecked")
public class MrQcCaseController {
    @Autowired
    private MrQcCaseFacade mrQcCaseFacade;

    /**
     * 病历质控报表-报表
     *
     * @param behospitalPageVO
     * @return
     */
    @ApiOperation(value = "病历质控报表-报表[by:cy]")
    @PostMapping("/qualityControl")
    public CommonResult<IPage<QualityControlDTO>> getQualityControl(@RequestBody BehospitalPageVO behospitalPageVO) {
        return CommonResult.success(mrQcCaseFacade.getQualityControl(behospitalPageVO));
    }

    /**
     * 病历质控报表-报表-运行质控
     *
     * @param behospitalPageVO
     * @return
     */
    @ApiOperation(value = "病历质控报表-报表-运行质控[by:cy]")
    @PostMapping("/run/qualityControl")
    public CommonResult<IPage<QualityControlDTO>> runGetQualityControl(@RequestBody BehospitalPageVO behospitalPageVO) {
        return CommonResult.success(mrQcCaseFacade.getQualityControl(behospitalPageVO));
    }

    /**
     * 病历质控报表-报表导出
     *
     * @param behospitalPageVO
     * @return
     */
    @ApiOperation(value = "病历质控报表-报表导出[by:cy]")
    @PostMapping("/qualityControlExport")
    public void qualityControlExport(HttpServletResponse response, @RequestBody BehospitalPageVO behospitalPageVO) {
        mrQcCaseFacade.qualityControlExport(response, behospitalPageVO);
    }

    /**
     * 病历质控报表-报表导出-运行质控
     *
     * @param behospitalPageVO
     * @return
     */
    @ApiOperation(value = "病历质控报表-报表导出-运行质控[by:cy]")
    @PostMapping("/run/qualityControlExport")
    public void runQualityControlExport(HttpServletResponse response, @RequestBody BehospitalPageVO behospitalPageVO) {
        mrQcCaseFacade.qualityControlExport(response, behospitalPageVO);
    }
}
