package com.lantone.report.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.QcResultShortDTO;
import com.lantone.common.vo.report.QcResultPageVO;
import com.lantone.common.vo.report.QcResultShortPageVO;
import com.lantone.report.facade.MrQcInfoFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


/**
 * <p>
 * 病历质控列表相关API
 * </p>
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "病历质控列表相关API", tags = { "病历质控列表相关API" })
@SuppressWarnings("unchecked")
public class MrQcInfoController {
    @Autowired
    private MrQcInfoFacade mrQcInfoFacade;

    /**
     * 条目缺陷病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷病历列表-报表[by:zhaops]")
    @PostMapping("/qcResultShortPage")
    public CommonResult<IPage<QcResultShortDTO>> qcResultShortPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.qcResultShortPage(qcResultShortPageVO));
    }

    /**
     * 条目缺陷病历列表-报表-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷病历列表-报表-运行质控[by:zhaops]")
    @PostMapping("/run/qcResultShortPage")
    public CommonResult<IPage<QcResultShortDTO>> runQcResultShortPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.qcResultShortPage(qcResultShortPageVO));
    }

    /**
     * 条目缺陷病历列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷病历列表-报表[by:zhaops]")
    @PostMapping("/qcResultShortPageExport")
    public void qcResultShortPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.qcResultShortPageExport(response, qcResultShortPageVO);
    }

    /**
     * 条目缺陷病历列表-报表导出-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "条目缺陷病历列表-报表-运行质控[by:zhaops]")
    @PostMapping("/run/qcResultShortPageExport")
    public void runQcResultShortPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.qcResultShortPageExport(response, qcResultShortPageVO);
    }

    /**
     * 病案首页合格|不合格病历列表-报表
     * @param qcResultPageVO
     * @return
     */
    @ApiOperation(value = "病案首页合格|不合格病历列表-报表[by:cy]")
    @PostMapping("/badLevelPage")
    public CommonResult<IPage<QcResultShortDTO>> badLevelPage(@RequestBody @Valid QcResultPageVO qcResultPageVO) {
        return CommonResult.success(mrQcInfoFacade.badLevelPage(qcResultPageVO));
    }

    /**
     * 病案首页合格|不合格病历列表-报表-运行质控
     * @param qcResultPageVO
     * @return
     */
    @ApiOperation(value = "病案首页合格|不合格病历列表-报表-运行质控[by:cy]")
    @PostMapping("/run/badLevelPage")
    public CommonResult<IPage<QcResultShortDTO>> runBadLevelPage(@RequestBody @Valid QcResultPageVO qcResultPageVO) {
        return CommonResult.success(mrQcInfoFacade.badLevelPage(qcResultPageVO));
    }

    /**
     * 病案首页合格|不合格病历列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = " 病案首页合格|不合格病历列表-报表导出[by:cy]")
    @PostMapping("/badLevelPagePageExport")
    public void badLevelPagePageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.badLevelPagePageExport(response, qcResultShortPageVO);
    }

    /**
     * 病案首页合格|不合格病历列表-报表导出-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = " 病案首页合格|不合格病历列表-报表导出-运行质控[by:cy]")
    @PostMapping("/run/badLevelPagePageExport")
    public void runBadLevelPagePageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.badLevelPagePageExport(response, qcResultShortPageVO);
    }


    /**
     * 质控核查统计A病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查统计病历列表-报表[by:zhaops]")
    @PostMapping("/qcCheckMRPage")
    public CommonResult<IPage<QcResultShortDTO>> qcCheckMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.qcCheckMRPage(qcResultShortPageVO));
    }

    /**
     * 质控核查统计A病历列表-报表-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查统计病历列表-报表-运行质控[by:zhaops]")
    @PostMapping("/run/qcCheckMRPage")
    public CommonResult<IPage<QcResultShortDTO>> runQcCheckMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.qcCheckMRPage(qcResultShortPageVO));
    }

    /**
     * 质控核查改善相关统计B病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查改善相关统计病历列表-报表[by:zhaops]")
    @PostMapping("/hmImproveMRPage")
    public CommonResult<IPage<QcResultShortDTO>> hmImproveMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.hmImproveMRPage(qcResultShortPageVO));
    }

    /**
     * 质控核查改善相关统计B病历列表-报表-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查改善相关统计病历列表-报表-运行质控[by:zhaops]")
    @PostMapping("/run/hmImproveMRPage")
    public CommonResult<IPage<QcResultShortDTO>> runHmImproveMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.hmImproveMRPage(qcResultShortPageVO));
    }

    /**
     * 质控核查统计病历AB列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查统计病历AB列表-报表导出[by:zhaops]")
    @PostMapping("/qcCheckMRPageExport")
    public void qcCheckMRPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.qcCheckMRPageExport(response, qcResultShortPageVO);
    }

    /**
     * 质控核查统计病历AB列表-报表导出-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "质控核查统计病历AB列表-报表导出-运行质控[by:zhaops]")
    @PostMapping("/run/qcCheckMRPageExport")
    public void runQcCheckMRPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.qcCheckMRPageExport(response, qcResultShortPageVO);
    }


    /**
     * 时效性|稽查条目病历列表-报表
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "时效性|稽查条目病历列表-报表[by:zhaops]")
    @PostMapping("/unModifyMRPage")
    public CommonResult<IPage<QcResultShortDTO>> unModifyMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.unModifyMRPage(qcResultShortPageVO));
    }

    /**
     * 时效性|稽查条目病历列表-报表-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "时效性|稽查条目病历列表-报表-运行质控[by:zhaops]")
    @PostMapping("/run/unModifyMRPage")
    public CommonResult<IPage<QcResultShortDTO>> runUnModifyMRPage(@RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        return CommonResult.success(mrQcInfoFacade.unModifyMRPage(qcResultShortPageVO));
    }

    /**
     * 时效性|稽查条目病历列表-报表导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "时效性|稽查条目病历列表-报表导出[by:zhaops]")
    @PostMapping("/unModifyMRPageExport")
    public void unModifyMRPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.unModifyMRPageExport(response, qcResultShortPageVO);
    }

    /**
     * 时效性|稽查条目病历列表-报表导出-运行质控
     *
     * @param qcResultShortPageVO
     * @return
     */
    @ApiOperation(value = "时效性|稽查条目病历列表-报表导出-运行质控[by:zhaops]")
    @PostMapping("/run/unModifyMRPageExport")
    public void runUnModifyMRPageExport(HttpServletResponse response, @RequestBody @Valid QcResultShortPageVO qcResultShortPageVO) {
        mrQcInfoFacade.unModifyMRPageExport(response, qcResultShortPageVO);
    }

}
