package com.lantone.report.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.ColumnDTO;
import com.lantone.common.dto.report.HomePageImproveDTO;
import com.lantone.common.dto.report.MedicalCheckDTO;
import com.lantone.common.vo.report.FilterMedicalCheckVO;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.report.facade.QcCheckStatisticsFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;


/**
 * <p>
 * 病历稽查|核查管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "病历稽查|核查管理统计相关API", tags = { "病历稽查|核查管理统计相关API" })
@SuppressWarnings("unchecked")
public class QcCheckStatisticsController {
    @Autowired
    private QcCheckStatisticsFacade qcCheckStatisticsFacade;

    /**
     * 病历稽查报表获取title
     * @param
     * @return
     */
    @ApiOperation(value = "病历稽查报表获取title[by:cy]")
    @PostMapping("/medicalCheckTitle")
    public CommonResult<List<ColumnDTO>> getMedicalCheckTitle() {
        return  CommonResult.success(qcCheckStatisticsFacade.getMedicalCheckTitle());
    }

    /**
     * 病历稽查表-报表
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历稽查统计[by:cy]")
    @PostMapping("/medicalCheckForm")
    public CommonResult<IPage<MedicalCheckDTO>> medicalCheckForm(@RequestBody @Valid FilterMedicalCheckVO filterVO) {
        return CommonResult.success(qcCheckStatisticsFacade.getMedicalCheck(filterVO));
    }

    /**
     * 病历稽查表-报表-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历稽查统计-运行质控[by:cy]")
    @PostMapping("/run/medicalCheckForm")
    public CommonResult<IPage<MedicalCheckDTO>> runMedicalCheckForm(@RequestBody @Valid FilterMedicalCheckVO filterVO) {
        return CommonResult.success(qcCheckStatisticsFacade.getMedicalCheck(filterVO));
    }

    /**
     * 病历稽查表-报表导出
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历稽查表-报表导出[by:cy]")
    @PostMapping("/medicalCheckExport")
    public void medicalCheckExport(HttpServletResponse response, @RequestBody @Valid FilterMedicalCheckVO filterVO) {
        qcCheckStatisticsFacade.medicalCheckExport(response, filterVO);
    }

    /**
     * 病历稽查表-报表导出-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历稽查表-报表导出-运行质控[by:cy]")
    @PostMapping("/run/medicalCheckExport")
    public void runMedicalCheckExport(HttpServletResponse response, @RequestBody @Valid FilterMedicalCheckVO filterVO) {
        qcCheckStatisticsFacade.medicalCheckExport(response, filterVO);
    }


    /**
     * 质控核查统计-报表
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "质控核查统计-报表[by:zhaops]")
    @PostMapping("/qcCheckStatistics")
    public CommonResult<List<HomePageImproveDTO>> qcCheckStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcCheckStatisticsFacade.qcCheckStatistics(filterOrderVO));
    }

    /**
     * 质控核查统计-报表-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "质控核查统计-报表-运行质控[by:zhaops]")
    @PostMapping("/run/qcCheckStatistics")
    public CommonResult<List<HomePageImproveDTO>> runQcCheckStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcCheckStatisticsFacade.qcCheckStatistics(filterOrderVO));
    }


    /**
     * 质控核查统计-报表导出
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "质控核查统计-报表导出[by:zhaops]")
    @PostMapping("/qcCheckStatisticsExport")
    public void qcCheckStatisticsExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcCheckStatisticsFacade.qcCheckStatisticsExport(response, filterOrderVO);
    }

    /**
     * 质控核查统计-报表导出-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "质控核查统计-报表导出-运行质控[by:zhaops]")
    @PostMapping("/run/qcCheckStatisticsExport")
    public void runQcCheckStatisticsExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcCheckStatisticsFacade.qcCheckStatisticsExport(response, filterOrderVO);
    }
}
