package com.lantone.report.web;

import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.AverageStatisticsDTO;
import com.lantone.common.dto.report.DeptNumDTO;
import com.lantone.common.dto.report.LevelStatisticsDTO;
import com.lantone.common.dto.report.LevelStatisticsTZDTO;
import com.lantone.common.vo.report.FilterOrderVO;
import com.lantone.common.vo.report.FilterVO;

import com.lantone.report.facade.QcScoreFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 质控病历分数|等级管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "质控病历分数|等级管理统计相关API", tags = { "质控病历分数|等级管理统计相关API" })
@SuppressWarnings("unchecked")
public class QcScoreController {
    @Autowired
    private QcScoreFacade qcScoreFacade;

    /**
     * 病历评分等级占比-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历评分等级占比-控制台[by:zhaops]")
    @PostMapping("/mrCount")
    public CommonResult<Map<String, Object>> mrCount(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.mrCount(filterVO));
    }

    /**
     * 病历评分等级占比-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "病历评分等级占比-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/mrCount")
    public CommonResult<Map<String, Object>> runMrCount(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.mrCount(filterVO));
    }

    /**
     * 各科室病历等级占比-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室病历等级占比-控制台[by:zhaops]")
    @PostMapping("/qcResultLevelPercent")
    public CommonResult<List<DeptNumDTO>> qcResultLevelPercent(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.qcResultLevelPercent(filterVO));
    }

    /**
     * 各科室病历等级占比-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室病历等级占比-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/qcResultLevelPercent")
    public CommonResult<List<DeptNumDTO>> runQcResultLevelPercent(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.qcResultLevelPercent(filterVO));
    }

    /**
     * 各科室缺陷占比-报表
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比-报表[by:zhaops]")
    @PostMapping("/levelStatistics")
    public CommonResult<List<LevelStatisticsDTO>> levelStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcScoreFacade.levelStatistics(filterOrderVO));
    }

    /**
     * 各科室缺陷占比-报表-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比-报表-运行质控[by:zhaops]")
    @PostMapping("/run/levelStatistics")
    public CommonResult<List<LevelStatisticsDTO>> runLevelStatistics(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcScoreFacade.levelStatistics(filterOrderVO));
    }

    /**
     * 各科室缺陷占比-报表导出
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比-报表导出[by:gaodm]")
    @PostMapping("/levelExport")
    public void levelExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcScoreFacade.levelExport(response, filterOrderVO);
    }

    /**
     * 各科室缺陷占比-报表导出-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比-报表导出-运行质控[by:gaodm]")
    @PostMapping("/run/levelExport")
    public void runLevelExport(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcScoreFacade.levelExport(response, filterOrderVO);
    }


    /**
     * 各科室缺陷占比（台州）-报表
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比（台州）-报表[by:zhaops]")
    @PostMapping("/levelStatisticsByDeptClass")
    public CommonResult<List<LevelStatisticsTZDTO>> levelStatisticsByDeptClass(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcScoreFacade.levelStatisticsByDeptClass_TZ(filterOrderVO));
    }

    /**
     * 各科室缺陷占比（台州）-报表-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比（台州）-报表-运行质控[by:zhaops]")
    @PostMapping("/run/levelStatisticsByDeptClass")
    public CommonResult<List<LevelStatisticsTZDTO>> runLevelStatisticsByDeptClass(@RequestBody @Valid FilterOrderVO filterOrderVO) {
        return CommonResult.success(qcScoreFacade.levelStatisticsByDeptClass_TZ(filterOrderVO));
    }

    /**
     * 各科室缺陷占比（台州）-报表导出
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比（台州）-报表导出[by:gaodm]")
    @PostMapping("/levelExport_TZ")
    public void levelExport_TZ(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcScoreFacade.levelExport_TZ(response, filterOrderVO);
    }

    /**
     * 各科室缺陷占比（台州）-报表导出-运行质控
     *
     * @param filterOrderVO
     * @return
     */
    @ApiOperation(value = "各科室缺陷占比（台州）-报表导出-运行质控[by:gaodm]")
    @PostMapping("/run/levelExport_TZ")
    public void runLevelExport_TZ(HttpServletResponse response, @RequestBody @Valid FilterOrderVO filterOrderVO) {
        qcScoreFacade.levelExport_TZ(response, filterOrderVO);
    }

    /**
     * 各科室质控平均分柱状图-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室质控平均分柱状图-控制台[by:zhaops]")
    @PostMapping("/getAverageScore")
    public CommonResult<Map<String, Object>> getAverageScore(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.getAverageScore(filterVO));
    }

    /**
     * 各科室质控平均分柱状图-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室质控平均分柱状图-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/getAverageScore")
    public CommonResult<Map<String, Object>> runGetAverageScore(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.getAverageScore(filterVO));
    }

    /**
     * 各科室质控平均分柱状图（台州）-控制台
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室质控平均分柱状图（台州）-控制台[by:zhaops]")
    @PostMapping("/getAverageScoreByDeptClass")
    public CommonResult<List<AverageStatisticsDTO>> getAverageScoreByDeptClass(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.getAverageScoreByDeptClass(filterVO));
    }

    /**
     * 各科室质控平均分柱状图（台州）-控制台-运行质控
     *
     * @param filterVO
     * @return
     */
    @ApiOperation(value = "各科室质控平均分柱状图（台州）-控制台-运行质控[by:zhaops]")
    @PostMapping("/run/getAverageScoreByDeptClass")
    public CommonResult<List<AverageStatisticsDTO>> runGetAverageScoreByDeptClass(@RequestBody @Valid FilterVO filterVO) {
        return CommonResult.success(qcScoreFacade.getAverageScoreByDeptClass(filterVO));
    }
}
