package com.lantone.report.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.ReBeHosDTO;
import com.lantone.common.vo.report.ReBeHosPageVO;
import com.lantone.report.facade.ReHos31DaysFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


/**
 * <p>
 * 31日再入院病人数管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "31日再入院病人数管理统计相关API", tags = { "31日再入院病人数管理统计相关API" })
@SuppressWarnings("unchecked")
public class ReHos31DaysController {
    @Autowired
    private ReHos31DaysFacade reHos31DaysFacade;

    /**
     * 31日再入院病人数统计-报表
     *
     * @param reBeHosPageVO
     * @return
     */
    @ApiOperation(value = "31日再入院病人数统计-报表[by:zhaops]")
    @PostMapping("/reHos31DaysPage")
    public CommonResult<IPage<ReBeHosDTO>> reHos31DaysPage(@RequestBody @Valid ReBeHosPageVO reBeHosPageVO) {
        return CommonResult.success(reHos31DaysFacade.reHos31DaysPage(reBeHosPageVO));
    }

    /**
     * 31日再入院病人数统计-报表-运行质控
     *
     * @param reBeHosPageVO
     * @return
     */
    @ApiOperation(value = "31日再入院病人数统计-报表-运行质控[by:zhaops]")
    @PostMapping("/run/reHos31DaysPage")
    public CommonResult<IPage<ReBeHosDTO>> runReHos31DaysPage(@RequestBody @Valid ReBeHosPageVO reBeHosPageVO) {
        return CommonResult.success(reHos31DaysFacade.reHos31DaysPage(reBeHosPageVO));
    }

    /**
     * 31日再入院病人数统计-报表导出
     *
     * @param reBeHosPageVO
     * @return
     */
    @ApiOperation(value = "31日再入院病人数统计-报表导出[by:zhaops]")
    @PostMapping("/reHos31DaysPageExport")
    public void reHos31DaysPageExport(HttpServletResponse response, @RequestBody @Valid ReBeHosPageVO reBeHosPageVO) {
        reHos31DaysFacade.reHos31DaysPageExport(response, reBeHosPageVO);
    }

    /**
     * 31日再入院病人数统计-报表导出-运行质控
     *
     * @param reBeHosPageVO
     * @return
     */
    @ApiOperation(value = "31日再入院病人数统计-报表导出-运行质控[by:zhaops]")
    @PostMapping("/run/reHos31DaysPageExport")
    public void runReHos31DaysPageExport(HttpServletResponse response, @RequestBody @Valid ReBeHosPageVO reBeHosPageVO) {
        reHos31DaysFacade.reHos31DaysPageExport(response, reBeHosPageVO);
    }
}
