package com.lantone.report.web;

import com.lantone.common.api.CommonResult;
import com.lantone.common.dto.report.UnModifyMRDTO;
import com.lantone.common.vo.report.FilterUnModifyMRVO;
import com.lantone.report.facade.TimelinessFacade;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


/**
 * <p>
 * 时效性管理统计相关API
 * </p>
 *
 * @author zhaops
 * @since 2020-04-27
 */

@RestController
@RequestMapping("/console")
@Api(value = "时效性管理统计相关API", tags = { "时效性管理统计相关API" })
@SuppressWarnings("unchecked")
public class TimelinessController {
    @Autowired
    private TimelinessFacade timelinessFacade;

    /**
     * 时效性相关统计-报表
     *
     * @param filterUnModifyMRVO
     * @return
     */
    @ApiOperation(value = "时效性相关统计[by:zhaops]")
    @PostMapping("/unModifyMRStatistics")
    public CommonResult<UnModifyMRDTO> unModifyMRStatistics(@RequestBody @Valid FilterUnModifyMRVO filterUnModifyMRVO) {
        return CommonResult.success(timelinessFacade.unModifyMRStatistics(filterUnModifyMRVO));
    }

    /**
     * 时效性相关统计-报表-运行质控
     *
     * @param filterUnModifyMRVO
     * @return
     */
    @ApiOperation(value = "时效性相关统计-运行质控[by:zhaops]")
    @PostMapping("/run/unModifyMRStatistics")
    public CommonResult<UnModifyMRDTO> runUnModifyMRStatistics(@RequestBody @Valid FilterUnModifyMRVO filterUnModifyMRVO) {
        return CommonResult.success(timelinessFacade.unModifyMRStatistics(filterUnModifyMRVO));
    }

    /**
     * 时效性相关统计-报表导出
     *
     * @param filterUnModifyMRVO
     * @return
     */
    @ApiOperation(value = "时效性相关统计-报表导出[by:zhaops]")
    @PostMapping("/unModifyMRStatisticsExport")
    public void unModifyMRStatisticsExport(HttpServletResponse response, @RequestBody @Valid FilterUnModifyMRVO filterUnModifyMRVO) {
        timelinessFacade.unModifyMRStatisticsExport(response, filterUnModifyMRVO);
    }

    /**
     * 时效性相关统计-报表导出-运行质控
     *
     * @param filterUnModifyMRVO
     * @return
     */
    @ApiOperation(value = "时效性相关统计-报表导出-运行质控[by:zhaops]")
    @PostMapping("/run/unModifyMRStatisticsExport")
    public void runUnModifyMRStatisticsExport(HttpServletResponse response, @RequestBody @Valid FilterUnModifyMRVO filterUnModifyMRVO) {
        timelinessFacade.unModifyMRStatisticsExport(response, filterUnModifyMRVO);
    }
}
